
/**
 * Todas las llamadas ajax deben ser centralizadas a traves de este objeto
 * con el fin de mantener control y consistencia sobre las llamadas al servidor
 * @returns {Ajax}
 */
function Ajax() {
    this.available = true;

    // Atributos configurables
    this.address = '';
    this.method = 'post';
    this.parameters = null;
    this.extras = null; // Opciones extras que no se envian en la consulta
    this.callback = null;
    this.container = null;
    this.type = 'json';
    this.wait = false;

    // Atributos de respuesta
    this.status = false;
    this.response = null;
    this.scripts = [];
    this.css = [];
}

Ajax.prototype.Ajax = function(address) {
    this.setAddress(address);
};

/**
 * Reserva acceso al objeto para iniciar una llamada ajax
 * @returns {Boolean} Reserva exitosa
 */
Ajax.prototype.reserveAccess = function() {

    // Si esta disponible, se realiza la reserva
    if (this.available)
    {
	this.available = false;
	return true;
    }

    return false;
};

Ajax.prototype.clearAccess = function() {
    this.available = true;
};

Ajax.prototype.isInitialized = function() {
    if (this.address === '' || this.available) {
	return false;
    }

    return true;
};

Ajax.prototype.setAddress = function(address) {
    this.address = address;
};

Ajax.prototype.getAddress = function() {
    return this.address;
};

Ajax.prototype.setMethod = function(method) {
    this.method = method;
};

Ajax.prototype.getMethod = function() {
    return this.method;
};

Ajax.prototype.setParameters = function(parameters) {
    this.parameters = parameters;
};

Ajax.prototype.getParameters = function() {
    return this.parameters;
};

Ajax.prototype.setElementRef = function(container) {
    this.container = container;
};

Ajax.prototype.getElementRef = function() {
    return this.container;
};

Ajax.prototype.setCallback = function(callback) {
    this.callback = callback;
};

Ajax.prototype.getCallback = function() {
    return this.callback;
};

Ajax.prototype.setType = function(type) {
    this.type = type;
};

Ajax.prototype.getType = function() {
    return this.type;
};

Ajax.prototype.setExtras = function(extras) {
    this.extras = extras;
};

Ajax.prototype.getExtras = function() {
    return this.extras;
};

Ajax.prototype.setWaitForAll = function(wait) {
    this.wait = wait;
};

Ajax.prototype.getWaitForAll = function() {
    return this.wait;
};

/**
 * Carga todos los assets obtenidos de la carga ajax
 */
Ajax.prototype.loadAssets = function() {
    // Se habilita el cache de 
    $.ajaxSetup({
	cache: false // TODO: Restaurar cache a true
    });
    // Carga de scripts
    for (var i = 0; i < this.scripts.length; i++) {
	var scriptName = '/' + this.scripts[i];
	$.getScript(JS_URL + '/' + this.scripts[i])
		.done(function(script, textStatus) {

		})
		.fail(function(jqxhr, settings, exception) {
		    if (DEV_MODE) {
			console.log('Error cargando script: ' + JS_URL + scriptName);
			console.log(exception);
		    }
		});
    }

    // Carga de css
    for (var i = 0; i < this.css.length; i++) {
	if (document.createStyleSheet) {
	    document.createStyleSheet(CSS_URL + this.css[i]);
	}
	else {
	    $("head").append($('<link rel="stylesheet" href="' + CSS_URL + this.css[i] + '" type="text/css" media="screen" />'));
	}
    }
};

Ajax.prototype.loadElements = function() {
    for (var i = 0; i < this.response.length; i++) {
	$(this.response[i].target).html(this.response[i].html);
    }
};

/**
 * Ejecuta la consulta con los datos definidos como atributos y ejecuta el callback si es proporcionado.
 */
Ajax.prototype.query = function() {

    if (!this.isInitialized())
	return;

    // Se habilita el cache de 
    $.ajaxSetup({
	cache: true // TODO: Restaurar cache a true
    });

    var data = (this.parameters !== null) ? $.param(this.parameters) : null;
    var parent = this;

    var queryRef = $.ajax({
	type: this.method,
	url: this.address,
	data: data,
	dataType: this.type,
	cache: true,
	success: function(data, textStatus, jqXHR) {

	    if (parent.type === 'html' || parent.type === 'jsonp') {
		parent.response = data;
		parent.status = (data !== null && data !== '');
	    }
	    else {
		parent.response = data.response;
		parent.status = data.status;
	    }

	    // Si hay scripts por cargar, se agregan a la lista
	    if (typeof data.scripts !== 'undefined') {
		parent.scripts = data.scripts;
	    }

	    // Si hay css por cargar, se agregan a la lista
	    if (typeof data.css !== 'undefined') {
		parent.css = data.css;
	    }

	    if (parent.callback !== null) {
		parent.callback(parent);
	    }

	    //TODO: borrar if
	    if (typeof app != 'undefined') {
		//app.showMessage(lang.line('msg_successfully_executed_command_js'), 'success');
	    }

	}
    }).done(function() {
	// TODO: Generate callback
    }).fail(function(data, textStatus, jqXHR) {

	if (!DEV_MODE) {
	    app.showMessage('Ha ocurrido un error ejecutando la última consulta. Compruebe su conexión a internet e intente nuevamente.', 'error');
	    var errorHtml = '<p>Ha ocurrido un error ejecutando la última consulta. Compruebe su conexión a internet e intente nuevamente.</p>';
	    errorHtml += '<p class="uk-alert uk-alert-danger">Error encontrado: ' + textStatus + '</p>';
	    errorHtml += '<h5 class="uk-text-danger">Detalle del error:</h5>';
	    errorHtml += '<div class="uk-overflow-container"><pre style="overflow-wrap: break-word;">' + data.responseText + '</pre></div>';
	    app.showDialog(errorHtml);
	}

    }).always(function() {
	parent.clearAccess();
    });

    return queryRef;
};

Ajax.prototype.clear = function() {
    this.available = true;
    this.address = '';
    this.method = 'post';
    this.parameters = null;
    this.container = null;
    this.callback = null;
    this.type = 'json';
    this.wait = false;
    this.status = false;
    this.response = null;
    this.scripts = [];
    this.css = [];
};
