
function App() {
    this.leftPanel = null;
    this.rightPanel = null;
    this.mainPanel = null;
    this.header = null;
    this.map = null;
    this.midLayer = null;
    this.topLayer = null;
    this.companyTree = null;
    this.currentTreeView = null;
    this.generalIndicatorsData = {};
    this.ytdOperationsGaugeData = {};
    this.lastMonthOperationsGaugeData = {};
    this.started = false;
}

App.prototype.setCompanyTree = function(companyTree) {
    this.companyTree = companyTree;
};
App.prototype.getCompanyTree = function() {
    return this.companyTree;
};

App.prototype.close = function(element, animate) {

    if (animate)
    {
	switch (element)
	{
	    case '#map':
		$(element).fadeOut('slow');
		break;
	    case '#mid-layer':
		$(element).fadeOut('slow');
		break;
	    case '#left-panel':
		$(element).animate({left: '-18%'}, 250, 'swing', function() {
		    structurePanelOpen = false;
		});
		break;
	    case '#right-panel':
		$(element).fadeOut(250, 'swing', function() {
		    generalInfoPanelOpen = false;
		});
		break;
	    case '#main-panel':
		$(element).fadeOut('fast');
		$('#map').removeClass('blurry');
		break;
	    case '#top-layer':
		$(element).fadeOut('fast');
		break;
	    default:
		$(element).fadeOut('fast');
	}
    }
    else {
	switch (element)
	{
	    case '#left-panel':
		$(element).stop(true).css({left: '-18%'});
		structurePanelOpen = false;
		break;
	    case '#right-panel':
		$(element).stop(true).css({right: '0', display: 'none'});
		generalInfoPanelOpen = false;
		break;
	    case '#main-panel':
		$('#map').removeClass('blurry');
	    default:
		$(element).hide();
	}
    }
};

App.prototype.open = function(element, animate) {

    if (animate)
    {
	switch (element)
	{
	    case '#map':

		app.recoverMap();
		$(element).fadeIn('slow');
		google.maps.event.trigger(map, "resize");
		break;
	    case '#mid-layer':
		$(element).fadeIn('slow');
		break;
	    case '#left-panel':
		$(element).animate({left: 0}, 250, 'swing', function() {
		    structurePanelOpen = true;
		});
		break;
	    case '#right-panel':
		$(element).fadeIn('slow', 'swing', function() {
		    generalInfoPanelOpen = true;
		});
		break;
	    case '#main-panel':
		$(element).fadeIn('fast');
		$('#map').addClass('blurry');
		break;
	    case '#top-layer':
		$(element).fadeIn('fast');
		break;
	    default:
		$(element).fadeIn('fast');
	}
    }
    else {
	switch (element)
	{
	    case '#left-panel':
		$(element).stop(true).css({left: 0});
		structurePanelOpen = true;
		break;
	    case '#right-panel':
		$(element).stop(true).css({right: 0});
		generalInfoPanelOpen = true;
		break;
	    case '#main-panel':
		$('#map').addClass('blurry');
	    default:
		$(element).show();
	}
    }
};

App.prototype.closeReport = function() {
    this.close('#main-panel', true);

};
App.prototype.showMessage = function(msg, type) {
    $.UIkit.notify(msg, {status: type, pos: 'bottom-right'});
};

App.prototype.clearUi = function(animate, deleteContent) {

    var hideableElements = ['#map', '#mid-layer', '#left-panel',
	'#right-panel', '#main-panel', '#top-layer'];

    // Recupera el mapa antes de borrar contenido
    this.recoverMap();

    for (var i = 0; i < hideableElements.length; i++)
    {
	var element = hideableElements[i];
	this.close(element, animate);
    }

    if (deleteContent) {
	// Se espera el termino de todas las animaciones para borrar el contenido
	$(":animated").promise().done(function() {
	    for (var i = 0; i < hideableElements.length; i++)
	    {
		var element = hideableElements[i];
		$(element).html('');
	    }
	});
    }


};
App.prototype.submitForm = function(element, activeMenu) {
    var form = $(element);

    var params = {};
    $.each(form.find('input'), function(i, e) {
	//console.log(e);
	params[$(e).attr('name')] = $(e).val();
    });
    this.loadAddress(form.attr('action'), form.attr('method'), params, activeMenu);
//    console.log(params);
};

App.prototype.loadAddress = function(address, method, params, activeMenu, clearUi) {

    if (!address) {
	return;
    }

    if (typeof method == 'undefined') {
	method = 'get';
    }

    if (typeof params == 'undefined') {
	params = null;
    }

    if (typeof activeMenu == 'undefined') {
	activeMenu = null;
    }

    if (typeof clearUi === 'undefined' || clearUi) {
	this.clearUi(true, false);
	console.log('clearing ui');
    }
    var ajax = new Ajax();

    if (ajax.reserveAccess()) {
	ajax.setAddress(address);
	ajax.setMethod(method);
	ajax.setParameters(params);
	ajax.setCallback(function(ajaxObject) {
	    // TODO: crear delays reales
	    ajaxObject.loadElements();
	    setTimeout(function() {
		ajaxObject.loadAssets();
		if (activeMenu != null)
		{
		    $('#menu-bar a').removeClass('active');
		    $(activeMenu).addClass('active');
		}
		for (var i = 0; i < ajaxObject.response.length; i++)
		{
		    var element = ajaxObject.response[i];
		    $(element.target).fadeIn();
		}
	    }, 500);

	});

	ajax.query();
    }




};

App.prototype.loadSelectionPreferences = function() {

    var ajax = new Ajax();

    if (ajax.reserveAccess()) {
	ajax.setAddress('configurador/selection_settings_list');
	ajax.setCallback(function(ajaxObject) {

	    var html = ''
	    for (var i = 0; i < ajaxObject.response.length; i++)
	    {
		var setting = ajaxObject.response[i];
		var value = setting.value.toString().split('"').join('\\\'');

		html += '<li class="custom-selection">';
		html += '<a href="javascript:setTreeSelection(\'' + setting.name + '\', \'' + value.replace(/"/g, '\"') + '\')">' + setting.name + '</a>';
		html += '</li>';
	    }

	    $('#selection-preferences .custom-selection').remove();
	    $('#selection-preferences .uk-nav').append(html);


	});
	ajax.query();
    }
};

App.prototype.saveNodeSelection = function() {
    var selectedNodes = app.getCompanyTree().get_selected(true);
    var selectedNodesConfig = [];

    for (var i = 0; i < selectedNodes.length; i++) {
	var node = selectedNodes[i];
	selectedNodesConfig.push({
	    id: node.original.oid,
	    node_type: node.original.node_type
	});
    }

    if (selectedNodesConfig.length > 0) {
	var ajax = new Ajax();

	if (ajax.reserveAccess()) {
	    ajax.setAddress('configurador/save_settings');
	    ajax.setParameters({
		settings_name: $('#selection-name').val(),
		settings_type_id: SETTINGS_TYPE_NODE_SELECTION,
		settings_key_index: '',
		settings_value: JSON.stringify(selectedNodesConfig),
		settings_is_active: 1
	    });

	    ajax.setCallback(function(ajaxObject) {

		if (ajaxObject.status) {
		    app.loadSelectionPreferences();
		    app.showMessage(ajaxObject.response, 'success');
		}
		else {
		    app.showMessage('Ha ocurrido un error almacenando la configuraci&oacute;n. Por favor, verifique su conexi&oacute;n e intente nuevamente.', 'error');
		}
	    });
	    ajax.query();
	}
    }
};

/**
 * Muestra un popup modal con el contenido entregado.
 * Si el elemento o dialog no existen, intenta crearlos, en caso de error realiza un alert con el contenido.
 * @param {String} content
 * @returns {Bool} success
 */
App.prototype.showDialog = function(content) {

    if (mainDialog === null) {
	var dialogCreated = App.prototype.createDialog();

	// No se pudo crear el dialog, se alerta
	if (!dialogCreated) {
	    alert(content);
	    return false;
	}
    }

    $('#main-dialog .dialog-content').html(content);

    mainDialog.show();
    return true;
};

App.prototype.createDialog = function() {
    var mainDialogElement = $('#main-dialog');

    if (mainDialogElement.length === 0) {
	$('body').append('<div id="main-dialog" class="uk-modal"><div class="uk-modal-dialog"><a class="uk-modal-close uk-close"></a><div class="dialog-content"></div></div></div>');
	alert('Elemento #main-dialog ha sido creado.');
    }

    mainDialog = new $.UIkit.modal("#main-dialog");
    if (mainDialog === null)
	return false;
    return true;
};

App.prototype.submitData = function(element, address) {

    var form = $(element);
    var params = {};
    $.each(form.find(':input'), function(i, e) {
	params[$(e).attr('name')] = $(e).val();
    });

    var ajax = new Ajax();
    ajax.reserveAccess();
    ajax.setAddress(form.attr('action'));
    ajax.setMethod(form.attr('method'));

    ajax.setParameters(params);
    ajax.setCallback(function(ajaxObject) {

	if (typeof ajaxObject.response !== 'undefined' && ajaxObject.response !== '')
	{
	    if (ajaxObject.status) {
		app.showMessage(ajaxObject.response, 'success');
	    } else {
		app.showMessage(ajaxObject.response, 'danger');
	    }
	}
    });

    ajax.query();

    if (typeof address !== 'undefined') {
	app.loadAddress(address);
    }

    return false;
};

/**
 * Obtiene todas las operaciones del jstree entregado
 * @param {jstree} jstree
 * @returns {Array|App.prototype.getOperations.operations}
 */
App.prototype.getOperations = function() {
    var operations = [];
    var rootNode = this.getCompanyTree().get_node('#');

    for (var i = 0; i < rootNode.children.length; i++) {
	var node = this.getCompanyTree().get_node(rootNode.children[i]);
	if (node.type === 'graph') {
	    operations.push(node);
	}
    }

    return operations;
};

App.prototype.getAreasByParent = function(parentNode) {
    var areas = [];
    var jstree = this.getCompanyTree();

    for (var i = 0; i < parentNode.children.length; i++) {
	var node = jstree.get_node(parentNode.children[i]);
	if (node.type === 'area') {
	    areas.push(node);
	}
    }

    return areas;
};

App.prototype.drawAreasRankingTableSelect = function(selectData) {
    var parts = selectData.split(';');
    this.drawAreasRankingTable(parts[0], parts[1]);
};

App.prototype.drawAreasRankingTable = function(tableType, indicatorType) {

    var targetTable = null;
    if (tableType === 'top') {
	// Se limpia y dibuja el espacio a utilizar
	var topTableHtml = '<table class="top-areas-list dashboard-table"><tr><th colspan="2">';
	topTableHtml += '<select id="top-areas-list" onchange="app.drawAreasRankingTableSelect(this.value)"><option value="top;' + indicatorType + '" selected="selected">' + lang.line('label_top_js') + '</option><option value="bottom;' + indicatorType + '">' + lang.line('label_bottom_js') + '</option></select> ';
	topTableHtml += lang.line('label_in_js').toUpperCase() + ' ' + lang.line('label_chart_' + indicatorType + '_js').toUpperCase() + '</th></table>';
	if ($('table.top-areas-list').length === 0)
	    $('.operations-top-areas').append(topTableHtml);
	else
	    $('table.top-areas-list').replaceWith(topTableHtml);
	targetTable = $('table.top-areas-list');
    }
    else {
	var bottomTableHtml = '<table class="top-areas-list dashboard-table"><tr><th colspan="2">';
	bottomTableHtml += '<select id="top-areas-list" onchange="app.drawAreasRankingTableSelect(this.value)"><option value="top;' + indicatorType + '">' + lang.line('label_top_js') + '</option><option value="bottom;' + indicatorType + '" selected="selected">' + lang.line('label_bottom_js') + '</option></select> ';
	bottomTableHtml += lang.line('label_in_js').toUpperCase() + ' ' + lang.line('label_chart_' + indicatorType + '_js').toUpperCase() + '</th></table>';

	// Se dibuja el espacio a utilizar
	if ($('table.top-areas-list').length === 0)
	    $('.operations-top-areas').append(bottomTableHtml);
	else
	    $('table.top-areas-list').replaceWith(bottomTableHtml);
	targetTable = $('table.top-areas-list');
    }

    var plantsList = []

    for (var operation_key in this.generalIndicatorsData) {
	var operation = this.generalIndicatorsData[operation_key];
	for (var area_key in operation.areas) {
	    var area = operation.areas[area_key];
	    var accumArea = 0;
	    var periodsArea = 0;

	    for (var i = 0; i < area.indicators.length; i++) {
		var indicator = area.indicators[i];
		if (indicatorType === 'avail' && indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id === METRIC_TYPE_MONTH) {
		    for (var j = 0; j < indicator.indicator_data.length; j++) {
			var data = indicator.indicator_data[j];
			accumArea += data.value;
			periodsArea++;
		    }

		    var availabilityValue = roundFloat(accumArea * 100 / periodsArea);
		    plantsList.push([area.text, operation.text, availabilityValue]); // Se agrega la lista top, luego seran ordenadas
		}
		if (indicatorType === 'usage' && indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id === METRIC_TYPE_MONTH) {
		    for (var j = 0; j < indicator.indicator_data.length; j++) {
			var data = indicator.indicator_data[j];
			accumArea += data.value;
			periodsArea++;
		    }

		    var availabilityValue = roundFloat(accumArea * 100 / periodsArea);
		    plantsList.push([area.text, operation.text, availabilityValue]); // Se agrega la lista top, luego seran ordenadas
		}
		if (indicatorType === 'effective_usage' && indicator.indicator_type === INDICATOR_EFFECTIVE_UTILIZATION && indicator.metric_id === METRIC_TYPE_MONTH) {
		    for (var j = 0; j < indicator.indicator_data.length; j++) {
			var data = indicator.indicator_data[j];
			accumArea += data.value;
			periodsArea++;
		    }

		    var availabilityValue = roundFloat(accumArea * 100 / periodsArea);
		    plantsList.push([area.text, operation.text, availabilityValue]); // Se agrega la lista top, luego seran ordenadas
		}
	    }

	}
    }

    if (tableType === 'top') {
	plantsList.sort(function(a, b) {
	    return b[2] - a[2] // Descendente
	});
    }
    else {
	plantsList.sort(function(a, b) {
	    return a[2] - b[2] // Ascendente
	});
    }

    var rankingHtml = '';
    // Obtener los primeros registros y generar html
    for (var i = 0; i < Math.min(plantsList.length, 5); i++) {
	rankingHtml += '<tr><td>' + plantsList[i][0] + ' <span>' + lang.line('label_in_js') + ' ' + plantsList[i][1] + '</span></td><td>' + plantsList[i][2] + '%' + '</td></tr>';
    }
    rankingHtml += '<tr><td colspan="2" class="top-table-selector">';
    rankingHtml += '<a href="javascript:app.drawAreasRankingTable(\'' + tableType + '\', \'avail\')" data-uk-tooltip="{pos:\'bottom\'}" title="' + lang.line('label_chart_avail_js') + '" class="' + ((indicatorType === 'avail') ? 'active' : '') + '">&nbsp;</a>';
    rankingHtml += '<a href="javascript:app.drawAreasRankingTable(\'' + tableType + '\', \'usage\')" data-uk-tooltip="{pos:\'bottom\'}" title="' + lang.line('label_chart_usage_js') + '" class="' + ((indicatorType === 'usage') ? 'active' : '') + '">&nbsp;</a>';
    rankingHtml += '<a href="javascript:app.drawAreasRankingTable(\'' + tableType + '\', \'effective_usage\')" data-uk-tooltip="{pos:\'bottom\'}" title="' + lang.line('label_chart_effective_usage_js') + '" class="' + ((indicatorType === 'effective_usage') ? 'active' : '') + '">&nbsp;</a>';
    rankingHtml += '</td></tr>';
    $(targetTable).append(rankingHtml);
};

App.prototype.toggleTab = function(element, forceAction) {
    var parent = $(element).parent('.sparkline-wrapper');

    var icon = parent.find('.sparkline-title i');

    if (icon.hasClass('icon-up') && (typeof forceAction === 'undefined' || forceAction !== 'open')) {
	icon.removeClass('icon-up').addClass('icon-down');
	parent.find('.sparkline-table-wrapper').slideUp(250);
    }
    else {
	icon.removeClass('icon-down').addClass('icon-up');
	parent.find('.sparkline-table-wrapper').slideDown(250);
    }
};

App.prototype.drawAreasMiniCharts = function() {

    $('.operations-gauge-charts').append('<div class="mini-charts"></div>');


    for (var operation_key in this.generalIndicatorsData) {
	var operation = this.generalIndicatorsData[operation_key];
	var miniChartsHtml = '<div class="sparkline-wrapper"><div class="sparkline-title" onclick="app.toggleTab(this)" data-operation-id="' + operation.original.oid + '">Overview ' + operation.text + ' <i class="icon-down" style="float: right;"></i></div>';
	miniChartsHtml += '<div class="sparkline-table-wrapper" style="display: none;"><table id="table-sparkline-' + operation.text + '" class="sparkline-tables">';
	miniChartsHtml += '<thead>';
	miniChartsHtml += '    <tr>';
	miniChartsHtml += '        <th>' + lang.line('label_area_js') + '</th>';
	miniChartsHtml += '        <th colspan="2">' + lang.line('label_chart_avail_js') + '</th>';
	miniChartsHtml += '        <th colspan="2">' + lang.line('label_chart_usage_js') + '</th>';
	miniChartsHtml += '        <th colspan="2">' + lang.line('label_chart_effective_usage_js') + '</th>';
	miniChartsHtml += '        <th>' + lang.line('label_chart_production_goal_real_js') + '</th>';
	miniChartsHtml += '    </tr>';
	miniChartsHtml += '</thead>';
	miniChartsHtml += '<tbody id="tbody-sparkline-' + operation.text + '">';

	for (var area_key in operation.areas) {
	    var area = operation.areas[area_key];
	    var availData = [];
	    var availTendency = 0;
	    var availLastData = 0;
	    var availTendencyCount = 0;
	    var utilizationData = [];
	    var utilizationLastData = 0;
	    var utilizationTendency = 0;
	    var utilizationTendencyCount = 0;
	    var effectiveUtilizationData = [];
	    var effectiveUtilizationLastData = 0;
	    var productionData = [];
	    var effectiveUtilizationTendency = 0;
	    var effectiveUtilizationTendencyCount = 0;

	    miniChartsHtml += '<tr class="spark-row spark-row-' + area_key + '"><th>' + area.text + '</th>';

	    for (var i = 0; i < area.indicators.length; i++) {
		var indicator = area.indicators[i];
		if (indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id == METRIC_TYPE_MONTH) {

		    indicator.indicator_data.reverse(); // TODO: Mejorar

		    var limit = Math.min(indicator.indicator_data.length, 4);// ultimos 4 meses o menos

		    for (var j = 0; j < limit; j++) {
			var data = indicator.indicator_data[j];
			var date = new Date(latinDateToIso(data.from, '/'));
			var pointLabel = date.getTime();
			var pv = roundFloat(data.value * 100);
			availData.push(pointLabel + ':' + pv);
			availTendency += pv;
			availTendencyCount++;
			if (j === 0) {
			    availLastData = pv;
			}
		    }
		}
		if (indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id == METRIC_TYPE_MONTH) {
		    indicator.indicator_data.reverse(); // TODO: Mejorar
		    var limit = Math.min(indicator.indicator_data.length, 4);

		    for (var j = 0; j < limit; j++) {
			var data = indicator.indicator_data[j];
			var date = new Date(latinDateToIso(data.from, '/'));
			var pointLabel = date.getTime();
			var pv = roundFloat(data.value * 100);

			utilizationData.push(pointLabel + ':' + pv);
			productionData.push(pointLabel + ':' + roundFloat(((data.value - 0.9) * 100) * ((j % 2 == 1) ? -1 : 2)));
			utilizationTendency += pv;
			utilizationTendencyCount++;
			if (j === 0) {
			    utilizationLastData = pv;
			}
		    }
		}
		if (indicator.indicator_type === INDICATOR_EFFECTIVE_UTILIZATION && indicator.metric_id == METRIC_TYPE_MONTH) {
		    indicator.indicator_data.reverse(); // TODO: Mejorar
		    var limit = Math.min(indicator.indicator_data.length, 4);

		    for (var j = 0; j < limit; j++) {
			var data = indicator.indicator_data[j];
			var date = new Date(latinDateToIso(data.from, '/'));
			var pointLabel = date.getTime();
			var pv = roundFloat(data.value * 100);

			effectiveUtilizationData.push(pointLabel + ':' + pv);
			effectiveUtilizationTendency += pv;
			effectiveUtilizationTendencyCount++;
			if (j === 0) {
			    effectiveUtilizationLastData = pv;
			}
		    }
		}
	    }

	    availTendency = availLastData - (availTendency / availTendencyCount);
	    utilizationTendency = utilizationLastData - (utilizationTendency / utilizationTendencyCount);
	    effectiveUtilizationTendency = effectiveUtilizationLastData - (effectiveUtilizationTendency / effectiveUtilizationTendencyCount);

	    miniChartsHtml += '<td data-sparkline="' + availData.reverse().join(',') + '"/>';
	    miniChartsHtml += '<td class="' + ((availTendency >= 0) ? 'icon-up tendency-increase' : 'icon-down tendency-decrease') + '">' + roundFloat(availTendency) + 'pp.</td>';
	    miniChartsHtml += '<td data-sparkline="' + utilizationData.reverse().join(',') + '"/>';
	    miniChartsHtml += '<td class="' + ((utilizationTendency >= 0) ? 'icon-up tendency-increase' : 'icon-down tendency-decrease') + '">' + roundFloat(utilizationTendency) + 'pp.</td>';
	    miniChartsHtml += '<td data-sparkline="' + effectiveUtilizationData.reverse().join(',') + '"/>';
	    miniChartsHtml += '<td class="' + ((effectiveUtilizationTendency >= 0) ? 'icon-up tendency-increase' : 'icon-down tendency-decrease') + '">' + roundFloat(effectiveUtilizationTendency) + 'pp.</td>';
	    miniChartsHtml += '<td data-sparkline="' + productionData.reverse().join(',') + ';column"/>';
	    miniChartsHtml += '</tr>';
	}

	miniChartsHtml += '    </tbody>';
	miniChartsHtml += '</table></div>';
	miniChartsHtml += '</div>';

	$('.mini-charts').append(miniChartsHtml);
    }



    Highcharts.SparkLine = function(options, callback) {
	var defaultOptions = {
	    chart: {
		renderTo: (options.chart && options.chart.renderTo) || this,
		backgroundColor: null,
		borderWidth: 0,
		type: 'area',
		margin: [2, 0, 2, 0],
		width: 60,
		height: 30,
		style: {
		    overflow: 'visible'
		},
		skipClone: true
	    },
	    title: {
		text: ''
	    },
	    credits: {
		enabled: false
	    },
	    xAxis: {
		type: 'datetime',
		labels: {
		    enabled: false
		},
		title: {
		    text: null
		},
		startOnTick: false,
		endOnTick: false,
		tickPositions: []
	    },
	    yAxis: {
		endOnTick: false,
		startOnTick: false,
		labels: {
		    enabled: false
		},
		title: {
		    text: null
		},
		tickPositions: [0]
	    },
	    legend: {
		enabled: false
	    },
	    tooltip: {
		backgroundColor: null,
		borderWidth: 0,
		shadow: false,
		useHTML: true,
		hideDelay: 0,
		shared: true,
		padding: 0,
		positioner: function(w, h, point) {
		    return {x: point.plotX - w / 2, y: point.plotY - h};
		}
	    },
	    exporting: {enabled: false},
	    plotOptions: {
		series: {
		    animation: false,
		    lineWidth: 1,
		    shadow: false,
		    states: {
			hover: {
			    lineWidth: 1
			}
		    },
		    marker: {
			radius: 1,
			states: {
			    hover: {
				radius: 2
			    }
			}
		    },
		    fillOpacity: 0.2
		},
		column: {
		    negativeColor: '#910000',
		    borderColor: 'silver'
		}
	    }
	};
	options = Highcharts.merge(defaultOptions, options);

	return new Highcharts.Chart(options, callback);
    };

    var $tds = $("td[data-sparkline]");

    function doChunk() {
	var time = +new Date(),
		i,
		len = $tds.length,
		$td,
		sparkData = [],
		data = [],
		chart;

	for (i = 0; i < len; i += 1) {
	    var data = [];
	    var sparkData = [];
	    var sparkPreData = [];
	    chart = {};

	    $td = $($tds[i]);
	    sparkPreData = $td.data('sparkline').split(';');
	    sparkData = sparkPreData[0].split(',');

	    if (sparkPreData[1]) {
		chart.type = sparkPreData[1];
	    }

	    for (var j = 0; j < sparkData.length; j++) {
		var sparkElement = sparkData[j].split(':');
		data.push([sparkElement[0], parseFloat(sparkElement[1])]);
	    }



	    $td.highcharts('SparkLine', {
		series: [{
			data: data,
			pointStart: 1,
			lineWidth: 2
		    }],
		tooltip: {
		    formatter: function() {
			return  '<b>' + this.y + '%</b><br/>' +
				Highcharts.dateFormat((!parent.overlap) ? '%d-%b-%Y' : '%d-%b',
					new Date(parseInt(this.points[0].key)));
		    }
		},
		chart: chart
	    });

	    // If the process takes too much time, run a timeout to allow interaction with the browser
	    if (new Date() - time > 500) {
		$tds.splice(0, i + 1);
		setTimeout(doChunk, 0);
		break;
	    }
	}
    }
    doChunk();


};

App.prototype.drawGeneralData = function() {


    $('#tree-title').html(lang.line('label_main_process_dashboard') + '<div class="uk-button uk-button-primary" onclick="$(\'#losses-modal\').fadeIn(200);" style="position: absolute; right: 50px; top: 25px; cursor: pointer;" data-uk-tooltip="{pos:\'top\'}" title="Open Losses Diagram"><i class="uk-icon-arrows-alt uk-icon-mini uk-icon-hover"></i></div><div class="orange-separator"></div>');
    $('#right-panel-charts').html('');
    $('#right-panel-charts').append('<div class="operations-charts-wrapper"></div>');
    $('#right-panel-charts .operations-charts-wrapper').append('<div class="operations-first-row"></div>');
    $('#right-panel-charts .operations-charts-wrapper .operations-first-row').append('<div class="operations-general-wrapper"><div class="operations-general-chart"></div></div>');
    $('#right-panel-charts .operations-charts-wrapper .operations-first-row').append('<div class="operations-gauges"></div>');
    $('#right-panel-charts .operations-charts-wrapper .operations-first-row').append('<div class="clearfix"></div>');
    $('#right-panel-charts .operations-charts-wrapper').append('<div class="operations-second-row"></div>');
    $('#right-panel-charts .operations-charts-wrapper .operations-second-row').append('<div class="operations-map-block"><div class="map-block-menu"><div class="mbm-map">' + lang.line('label_map_js') + '</div><div class="mbm-production">' + lang.line('label_production_ranking_js') + '</div><div class="mbm-roic">' + lang.line('label_roic_ranking_js') + '</div><div class="mbm-security">' + lang.line('label_security_ranking_js') + '</div></div>');
    $('#right-panel-charts .operations-charts-wrapper .operations-second-row').append('<div class="operations-top-areas"></div>');
    $('#right-panel-charts').append('<div class="operations-gauge-charts"></div>');

    this.drawMainDrilldownChartNew('avail');
    //this.drawAreasRankingTable('top', 'avail');
    this.drawYtdOperationsGauge('.operations-gauges');
    this.drawLastMonthOperationsGauge('.operations-gauges');

    equalizeHeight('.operations-general-wrapper', '.operations-gauges', 'exact');

    this.drawMapBlock('.operations-map-block');
    this.drawAreasRankingTable('bottom', 'avail');

    equalizeHeight('.operations-map-block', '.operations-top-areas', 'max');

    this.drawAreasMiniCharts();

    var availableHeight = $(window).height() - $('#header').height() - $('#tree-title').outerHeight(true);

    equalizeHeight('.operations-charts-wrapper', '.operations-gauge-charts', 'exact', availableHeight, null);
    $(window).resize();
    setTimeout(function() {
	$('.sparkline-title').first().click(); // Abrir la primera operacion
    }, 1000);
};

App.prototype.drawMapBlock = function() {

    if ($('.operations-map-block #map').length === 0) {
	$('#map').detach().appendTo('.operations-map-block').css({opacity: 1, display: 'block'});
    }

    // TODO: redefinir automaticamente
    var latlng = new google.maps.LatLng(-25.555739, -70.026151);
    $(window).resize(function() {
	google.maps.event.trigger(map, "resize");
	map.setCenter(latlng);
	map.setZoom(4);
    });

    if ($('#pop-map').length == 0) {
	$('#map').append('<div id="pop-map" onclick="app.mapFindMarkerById(null)"></div>');
    }

    this.getTopAreasByAttributes();
};

App.prototype.getTopAreasByAttributes = function() {

    var nodes = this.getCompanyTree()._model.data;

    var topSecurity = [];
    var topRoic = [];
    var topProduction = [];
    for (var nodeId in nodes) {
	var node = nodes[nodeId];
	// Se rechaza el nodo raiz de jstree
	if (node.id === '#')
	    continue;

	// Se rechazan los subsystemas ya que no se deben graficar en el mapa
	if (node.original.node_type == NODE_TYPE_AREA) {

//				console.log(attribute);
	    for (var i = 0; i < node.data.attributes.length; i++) {
		var attribute = node.data.attributes[i];
		// Se descartan valores vacios
		if (attribute['valor_attr'].trim() == '' || isNaN(parseFloat(attribute['valor_attr']))) {
		    continue;
		}
		// TODO: crear index en vez de nombre
		if (attribute.nombre_attr == 'Índice de Seguridad Año Anterior') {
		    var temp = {name: node['text'], value: attribute['valor_attr'], sortable_value: parseFloat(attribute['valor_attr'])}
		    topSecurity.push(temp);
		}
		if (attribute.nombre_attr == 'Producción Año Anterior') {
		    var temp = {name: node['text'], value: attribute['valor_attr'], sortable_value: parseFloat(attribute['valor_attr'])}
		    topProduction.push(temp);
		}
		if (attribute.nombre_attr == 'ROIC Año Anterior') {
		    var temp = {name: node['text'], value: attribute['valor_attr'], sortable_value: parseFloat(attribute['valor_attr'])}
		    topRoic.push(temp);
		}
	    }
	}
    }

    // Debe existir campo sortable_value
    topSecurity.sort(sortByValue);
    topRoic.sort(sortByValue);
    topProduction.sort(sortByValue);

    var securityHtml = generateNameValueTable(topSecurity, lang.line('label_security_ranking_js'), 10, 'dashboard-table');
    var roicHtml = generateNameValueTable(topRoic, lang.line('label_roic_ranking_js'), 10, 'dashboard-table');
    var prodHtml = generateNameValueTable(topProduction, lang.line('label_production_ranking_js'), 10, 'dashboard-table');

    $('.operations-map-block').append('<div class="mbm-content mbm-production-content">' + prodHtml + '</div>');
    $('.operations-map-block').append('<div class="mbm-content mbm-roic-content">' + roicHtml + '</div>');
    $('.operations-map-block').append('<div class="mbm-content mbm-security-content">' + securityHtml + '</div>');

    $('.mbm-map').click(function() {
	$('.mbm-content').hide();
	$('#map').show();
    });

    $('.mbm-production').click(function() {
	$('#map, .mbm-content').hide();
	$('.mbm-production-content').show();
    });
    $('.mbm-roic').click(function() {
	$('#map, .mbm-content').hide();
	$('.mbm-roic-content').show();
    });
    $('.mbm-security').click(function() {
	$('#map, .mbm-content').hide();
	$('.mbm-security-content').show();
    });

};

App.prototype.drawOperationGeneralData = function(node, metricId) {

    // Recupera el mapa antes de borrar contenido
    app.recoverMap();
    var companyTree = app.getCompanyTree();

    // Si se entrego idNodo, obtener nodo
    if (typeof node === 'string') {
	node = companyTree.get_node(node);

    }
    // Si no se selecciono metrica, entonces cargar YTD
    if (typeof metricId === 'undefined' || metricId === null) {
	metricId = METRIC_TYPE_YTD;
    }

    // TODO: Validar $this
    var $this = this;
    var selector = '#right-panel-charts';

    // Borrar contenido anterior
    $(selector).html('');


    // Se obtiene la operacion con los datos de indicadores
    var operation = null;

    if (node.type === 'graph') {

	for (var operation_key in $this.generalIndicatorsData) {
	    if ($this.generalIndicatorsData[operation_key].id == node.id) {
		operation = $this.generalIndicatorsData[operation_key];
		break;
	    }
	}
    }
    else {

	var parents = node.parents;
	for (var key in parents) {
	    var parentNode = companyTree.get_node(parents[key]);
	    if (parentNode.type === 'graph') {
		operation = parentNode;
		break;
	    }
	}
    }

    // No se encontro la operacion para el nodo entregado
    if (operation === null)
	return false;

    var areasContainer = $('<div id="dashboard-areas-wrapper"></div>').appendTo(selector);
    var highestHeight = $(window).height() - $('#header').height() - $('#tree-title').outerHeight(true);

    for (var area_key in operation.areas) {

	var areaContainer = $('<div class="dashboard-area-wrapper dashboard-area-wrapper-' + area_key + ' ' + ((node.type === 'area' && node.id === area_key) ? 'selected' : '') + '"></div>').appendTo(areasContainer);

	var areaHeader = '<h4 data-tip="Ver Gráficos" title="Ver indicadores para ' + operation.areas[area_key].text + '"><span onclick="checkAndGetIndicators(\'' + area_key + '\')" ><i class="icon-graph"></i> ' + operation.areas[area_key].text + ' <span class="hidden-raquo">&raquo;</span> </span>';


	areaHeader += '<div class="area-controls">';
	areaHeader += '<form class="uk-form">';
	areaHeader += '<ul class="uk-subnav">';

	areaHeader += '<li data-uk-dropdown="{mode:\'click\'}">';

	areaHeader += '<a href="#" style="vertical-align: sub; color: #666;" class="hoverable-link"><i class="uk-icon-list uk-icon-small"></i></a>';

	areaHeader += '<div class="uk-dropdown uk-dropdown-small">';
	areaHeader += '<ul class="uk-nav uk-nav-dropdown">';
	areaHeader += '    <li><a href="#" onclick="app.mapFindMarkerById(\'' + area_key + '\')">' + lang.line('task_open_in_map') + '</a></li>';
	areaHeader += '    <li><a href="#" onclick="app.showAreaDiagram(\'' + area_key + '\')">' + lang.line('task_open_diagram') + '</a></li>';
	areaHeader += '    <li><a href="#" onclick="checkAndGetBasicIndicators(\'' + area_key + '\')">' + lang.line('task_show_basic_graphs') + '</a></li>';
	areaHeader += '    <li><a href="#" onclick="checkAndGetAdvancedIndicators(\'' + area_key + '\')">' + lang.line('task_show_graphs') + '</a></li>';
	areaHeader += '</ul>';
	areaHeader += '</div>';
	areaHeader += '</li>';

	areaHeader += '<li><select class="area-metric-selector" onchange="app.drawOperationGeneralData(\'' + node.id + '\',this.value);" title="' + lang.line('task_select_metric_period_js') + '" data-uk-tooltip="{pos:\'top\'}" ><option value="' + METRIC_TYPE_YTD + '" ' + ((metricId == METRIC_TYPE_YTD) ? 'selected="selected"' : '') + '>' + lang.line('label_metric_type_' + METRIC_TYPE_YTD + '_js') + '</option><option value="' + METRIC_TYPE_MY + '" ' + ((metricId == METRIC_TYPE_MY) ? 'selected="selected"' : '') + '>' + lang.line('label_metric_type_' + METRIC_TYPE_MY + '_js') + '</option><option value="' + METRIC_TYPE_MTD + '" ' + ((metricId == METRIC_TYPE_MTD) ? 'selected="selected"' : '') + '>' + lang.line('label_metric_type_' + METRIC_TYPE_MTD + '_js') + '</option><option value="' + METRIC_TYPE_FMTD + '" ' + ((metricId == METRIC_TYPE_FMTD) ? 'selected="selected"' : '') + '>' + lang.line('label_metric_type_' + METRIC_TYPE_FMTD + '_js') + '</option></select></li>';
	areaHeader += '<li><span class="area-slider-controls uk-buttons" style="margin-left: 15px;"><i class="uk-icon-angle-left uk-icon-medium uk-button uk-button-primary"></i><i class="uk-icon-angle-right uk-icon-medium uk-button uk-button-primary"></i></span></li>';
	areaHeader += '</ul>';
	areaHeader += '</form></div></h4>';

	$(areaContainer).append(areaHeader);

	//$this.drawLastMonthGauge(area_key, operation, $('.mini-gauges-' + area_key));
	//$this.drawYTDGauge(area_key, operation, $('.mini-gauges-' + area_key));
	//$this.drawLastMonthData(area_key, operation, areaContainer);
	var grid = $('<div class="uk-grid uk-grid-small" data-uk-grid-margin></div>').appendTo(areaContainer);
	$(grid).append('<div class="uk-width-1-4"><div class="area-block mini-gauges-' + area_key + ' mini-gauge-wrapper"></div></div>');

	$this.drawYtdData(area_key, operation, grid, metricId);
	$this.drawAreaGauges(area_key, operation, $('.mini-gauges-' + area_key), metricId);
	//$this.drawEquipmentsAUEERanking('top', 'avail', area_key, grid);
	$this.drawEquipmentsAUEERanking2(area_key, grid, 'avail', 'bot', metricId);
	$this.drawAreaProductionData(area_key, operation, grid);
	var fnpGroup = $('<div class="uk-width-3-4 fnp-group"><div class="uk-button-group" data-uk-button-radio style="position:absolute; top: 5px; left: 15px; z-index: 999;">'
		+ '<button class="uk-button uk-button-mini" onclick="app.updateAreaFnp(\'' + area_key + '\',\'' + operation.id + '\', ' + INDICATOR_FNP + ');">' + lang.line('label_chart_fpn_js') + '</button>'
		+ '<button class="uk-button uk-button-mini" onclick="app.updateAreaFnp(\'' + area_key + '\',\'' + operation.id + '\', ' + INDICATOR_OEE + ');">' + lang.line('label_chart_oee_js') + '</button></div><div class="area-block fnp-area-block"></div></div>')
		.appendTo(grid).children('.area-block');


	$this.drawFnpGraph(area_key, operation, fnpGroup, INDICATOR_FNP);
	$this.drawFnpTables(area_key, operation, fnpGroup, INDICATOR_FNP);
	$this.drawCostsRanking(area_key, grid, 'failure_cost', 'top', metricId);
	$this.drawMTBSRanking(area_key, grid, 'mttr', 'top', metricId);
	$this.drawMTBIRanking(area_key, grid, 'mtbf', 'bot', metricId);
	$this.drawReliabilityRanking(area_key, grid, 'current_reliability', 'bot');


	$(areaContainer).append('<div class="clearfix"></div>');
	if ($(areaContainer).height() > highestHeight)
	    highestHeight = $(areaContainer).height();
    }


    $('.dashboard-area-wrapper').hide();
    if ($('.dashboard-area-wrapper.selected').length === 0) {
	$('.dashboard-area-wrapper').first().addClass('selected');
    }

    $('.dashboard-area-wrapper.selected').fadeIn(250);

    $('.dashboard-area-wrapper .area-slider-controls .uk-icon-angle-left').click(function() {
	$(this).parents('.dashboard-area-wrapper').fadeOut(500).prev().fadeIn(500);
	$(window).resize();
    });
    $('.dashboard-area-wrapper:first-of-type .uk-icon-angle-left').prop('disabled', true);

    $('.dashboard-area-wrapper .area-slider-controls .uk-icon-angle-right').click(function() {
	$(this).parents('.dashboard-area-wrapper').fadeOut(250).next().fadeIn(500);
	$(window).resize();
    });
    $('.dashboard-area-wrapper:last-of-type .uk-icon-angle-right').prop('disabled', true);

    $(areasContainer).height(highestHeight);
    $('#right-panel').show();
    $(window).resize();
    //$(window).trigger('resize');

    //}).delay(300).fadeIn(500);
};

App.prototype.drawReliabilityRanking = function(area_key, parentSelector, indicatorType, order) {

    var selector = 'reliability-ranking-' + area_key;

    if (area_key != null) {

	var areaNode = app.getCompanyTree().get_node(area_key);

	// Query ajax granular para aprovechar cache de browser
	var ajax = new Ajax();
	ajax.reserveAccess();
	ajax.setType('json');
	ajax.setAddress('indicators/reliability_ranking/');
	ajax.setMethod('get');

	ajax.setCallback(function(ao) {

	    var orderSelectHtml = '<select id="top-selector-' + selector + '" onchange="app.drawReliabilityRanking(\'' + area_key + '\',\'' + parentSelector + '\',\'' + indicatorType + '\',this.value)"><option value="top" ' + ((order === 'top') ? 'selected="selected"' : '') + '>' + lang.line('label_top_js') + '</option><option value="bot" ' + ((order === 'bot') ? 'selected="selected"' : '') + '>' + lang.line('label_bottom_js') + '</option></select> ';
	    var selectHtml = lang.line('label_chart_reliability_js');

	    var title = {
		text: orderSelectHtml + ' ' + lang.line('label_in_js') + ' ' + selectHtml
	    };

	    var valueTitle = {
		text: lang.line('label_percent_js')
	    };

	    var rows = [];
	    if (typeof ao.response[areaNode.original.oid + ''] !== 'undefined') {
		var list = ao.response[areaNode.original.oid + ''][order][indicatorType];

		for (var i = 0; i < list.length; i++) {
		    rows.push({title: list[i].name, desc: '', value: roundFloat(list[i].value * 100) + ' %'});
		}
	    }

	    $('#' + selector).html(app.generateRankingTable(title, valueTitle, rows, {id: 'reliability-ranking-' + area_key, class: 'semaphore-table'}));
	});

	ajax.query();
    }

    // Se crea un nuevo espacio
    if ($('#' + selector).length === 0) {
	var containerHtml = '';
	containerHtml += '<div class="uk-width-1-4"><div id="' + selector + '" class="area-mtbs-ranking-wrapper area-block"></div></div>';
	$(parentSelector).append(containerHtml);
    }

};

App.prototype.drawMTBSRanking = function(area_key, parentSelector, indicatorType, order, metricId) {

    var selector = 'mtbs-ranking-' + area_key;
    var halfIndicators = ['mttoa', 'mtti', 'mtta', 'mttw', 'mtts', 'mttow', 'mttos', 'mttr', 'mttm'];

    if (area_key != null) {

	var areaNode = app.getCompanyTree().get_node(area_key);

	// Query ajax granular para aprovechar cache de browser
	var ajax = new Ajax();
	ajax.reserveAccess();
	ajax.setType('json');
	ajax.setAddress('indicators/ckpi_ranking/' + metricId);
	ajax.setMethod('get');

	ajax.setCallback(function(ao) {

	    var orderSelectHtml = '<select id="top-selector-' + selector + '" onchange="app.drawMTBSRanking(\'' + area_key + '\',\'' + parentSelector + '\',\'' + indicatorType + '\',this.value,\'' + metricId + '\')"><option value="top" ' + ((order === 'top') ? 'selected="selected"' : '') + '>' + lang.line('label_top_js') + '</option><option value="bot" ' + ((order === 'bot') ? 'selected="selected"' : '') + '>' + lang.line('label_bottom_js') + '</option></select> ';
	    var selectHtml = '<select id="selector-' + selector + '" onchange="app.drawMTBSRanking(\'' + area_key + '\',\'' + parentSelector + '\',this.value,\'' + order + '\',\'' + metricId + '\')">';

	    for (var i = 0; i < halfIndicators.length; i++) {
		var value = halfIndicators[i];
		selectHtml += '<option value="' + value + '" ' + ((indicatorType === value) ? 'selected="selected"' : '') + '>' + value.toUpperCase() + '</option>';
	    }
	    selectHtml += '</select>';

	    var title = {
		text: orderSelectHtml + ' ' + lang.line('label_in_js') + ' ' + selectHtml
	    };

	    var valueTitle = {
		text: lang.line('label_hours_js')
	    };

	    var list = ao.response[areaNode.original.oid + ''][order][indicatorType];
	    var rows = [];

	    for (var i = 0; i < list.length; i++) {
		rows.push({title: list[i].name, desc: '', value: roundFloat(list[i].value)});
	    }

	    $('#' + selector).html(app.generateRankingTable(title, valueTitle, rows, {id: 'mtbs-ranking-' + area_key}));
	});

	ajax.query();
    }

    // Se crea un nuevo espacio
    if ($('#' + selector).length === 0) {
	var containerHtml = '';
	containerHtml += '<div class="uk-width-1-4"><div id="' + selector + '" class="area-mtbs-ranking-wrapper area-block"></div></div>';
	$(parentSelector).append(containerHtml);
    }
};

App.prototype.drawMTBIRanking = function(area_key, parentSelector, indicatorType, order, metricId) {

    var selector = 'mtbi-ranking-' + area_key;
    var halfIndicators = ['mtboa', 'mtbi', 'mtba', 'mtbw', 'mtbs', 'mtbow', 'mtbos', 'mtbf', 'mtbm'];

    if (area_key != null) {
	var areaNode = app.getCompanyTree().get_node(area_key);
	// Query ajax granular para aprovechar cache de browser
	var ajax = new Ajax();
	ajax.reserveAccess();
	ajax.setType('json');
	ajax.setAddress('indicators/ckpi_ranking/' + metricId);
	ajax.setMethod('get');

	ajax.setCallback(function(ao) {

	    var orderSelectHtml = '<select id="top-selector-' + selector + '" onchange="app.drawMTBIRanking(\'' + area_key + '\',\'' + parentSelector + '\',\'' + indicatorType + '\',this.value,\'' + metricId + '\')"><option value="top" ' + ((order === 'top') ? 'selected="selected"' : '') + '>' + lang.line('label_top_js') + '</option><option value="bot" ' + ((order === 'bot') ? 'selected="selected"' : '') + '>' + lang.line('label_bottom_js') + '</option></select> ';
	    var selectHtml = '<select id="selector-' + selector + '" onchange="app.drawMTBIRanking(\'' + area_key + '\',\'' + parentSelector + '\',this.value,\'' + order + '\',\'' + metricId + '\')">';

	    for (var i = 0; i < halfIndicators.length; i++) {
		var value = halfIndicators[i];
		selectHtml += '<option value="' + value + '" ' + ((indicatorType === value) ? 'selected="selected"' : '') + '>' + value.toUpperCase() + '</option>';
	    }
	    selectHtml += '</select>';

	    var title = {
		text: orderSelectHtml + ' ' + lang.line('label_in_js') + ' ' + selectHtml
	    };

	    var valueTitle = {
		text: lang.line('label_hours_js')
	    };

	    var list = ao.response[areaNode.original.oid + ''][order][indicatorType];
	    var rows = [];

	    for (var i = 0; i < list.length; i++) {
		rows.push({title: list[i].name, desc: '', value: roundFloat(list[i].value)});
	    }

	    $('#' + selector).html(app.generateRankingTable(title, valueTitle, rows, {id: 'mtbi-ranking-' + area_key}));
	});

	ajax.query();
    }

    // Se crea un nuevo espacio
    if ($('#' + selector).length === 0) {
	var containerHtml = '';
	containerHtml += '<div class="uk-width-1-4"><div id="' + selector + '" class="area-mtbi-ranking-wrapper area-block"></div>';
	$(parentSelector).append(containerHtml);
    }
};

App.prototype.drawCostsRanking = function(area_key, parentSelector, indicatorType, order, metricId) {

    var selector = 'costs-ranking-' + area_key;
    var halfIndicators = ['failure_cost', 'intervention_cost', 'total_cost'];

    if (area_key != null) {

	var areaNode = app.getCompanyTree().get_node(area_key);

	// Query ajax granular para aprovechar cache de browser
	var ajax = new Ajax();
	ajax.reserveAccess();
	ajax.setType('json');
	ajax.setAddress('indicators/ckpi_ranking/' + metricId);
	ajax.setMethod('get');

	ajax.setCallback(function(ao) {

	    var orderSelectHtml = '<select id="top-selector-' + selector + '" onchange="app.drawCostsRanking(\'' + area_key + '\',\'' + parentSelector + '\',\'' + indicatorType + '\',this.value,\'' + metricId + '\')"><option value="top" ' + ((order === 'top') ? 'selected="selected"' : '') + '>' + lang.line('label_top_js') + '</option><option value="bot" ' + ((order === 'bot') ? 'selected="selected"' : '') + '>' + lang.line('label_bottom_js') + '</option></select> ';

	    var selectHtml = '<select id="selector-' + selector + '" onchange="app.drawCostsRanking(\'' + area_key + '\',\'' + parentSelector + '\',this.value,\'' + order + '\',\'' + metricId + '\')">';

	    for (var i = 0; i < halfIndicators.length; i++) {
		var value = halfIndicators[i];
		selectHtml += '<option value="' + value + '" ' + ((indicatorType === value) ? 'selected="selected"' : '') + '>' + lang.line('label_' + value + 's_js').toUpperCase() + '</option>';
	    }
	    selectHtml += '</select>';

	    var title = {
		text: orderSelectHtml + ' ' + lang.line('label_in_js') + ' ' + selectHtml
	    };

	    var valueTitle = {
		text: lang.line('label_value_js')
	    };

	    var list = ao.response[areaNode.original.oid + ''][order][indicatorType];

	    var rows = [];

	    for (var i = 0; i < list.length; i++) {
		var value = roundFloat(list[i].value, 2);
		if (value > 1000000) {
		    value = roundFloat(value / 1000, 0) + ' K'; // Reducir a miles
		}

		rows.push({title: list[i].name, desc: '', value: value});
	    }

	    $('#' + selector).html(app.generateRankingTable(title, null, rows, {id: 'costs-ranking-' + area_key}));
	});

	ajax.query();
    }

    // Se crea un nuevo espacio
    if ($('#' + selector).length === 0) {
	var containerHtml = '';
	containerHtml += '<div class="uk-width-1-4"><div id="' + selector + '" class="area-mtbs-ranking-wrapper area-block"></div></div>';
	$(parentSelector).append(containerHtml);
    }

};

App.prototype.drawEquipmentsAUEERankingTable = function(value) {

    var params = value.split(';');

    this.drawEquipmentsAUEERanking(params[0], params[1], params[2], params[3]);
};

App.prototype.drawEquipmentsAUEERanking = function(tableType, indicatorType, area_key, parentSelector) {
    var selector = 'area-auue-ranking-' + area_key;
    var titleHtml = '';

    if (tableType === 'top') {
	titleHtml = '<select id="' + selector + '" onchange="app.drawEquipmentsAUEERankingTable(this.value)"><option value="top;' + indicatorType + ';' + area_key + ';' + selector + '" selected="selected">' + lang.line('label_top_js') + '</option><option value="bottom;' + indicatorType + ';' + area_key + ';' + selector + '">' + lang.line('label_bottom_js') + '</option></select> ';
	titleHtml += ' ' + lang.line('label_in_js') + ' ';
	titleHtml += '<select id="select-' + selector + '" onchange="app.drawEquipmentsAUEERankingTable(this.value)" style="max-width: 125px;">';
	titleHtml += '	<option value="' + tableType + ';avail;' + area_key + ';' + selector + '" ' + ((indicatorType === 'avail') ? 'selected="selected"' : '') + '>' + lang.line('label_chart_avail_js').toUpperCase() + '</option>';
	titleHtml += '	<option value="' + tableType + ';usage;' + area_key + ';' + selector + '" ' + ((indicatorType === 'usage') ? 'selected="selected"' : '') + '>' + lang.line('label_chart_usage_js').toUpperCase() + '</option>';
	titleHtml += '	<option value="' + tableType + ';effective_usage;' + area_key + ';' + selector + '" ' + ((indicatorType === 'effective_usage') ? 'selected="selected"' : '') + '>' + lang.line('label_chart_effective_usage_js').toUpperCase() + '</option>';
	titleHtml += '</select>';
	titleHtml += ' <div class="auue-system-selector" data-uk-tooltip="{pos:\'top\'}" title="' + lang.line('label_show_hide_systems_js') + '"><i class="icon-tree"></i><input type="checkbox" style="vertical-align: middle;"/></div>';
    }
    else {

	titleHtml += '<select id="' + selector + '" onchange="app.drawEquipmentsAUEERankingTable(this.value)"><option value="top;' + indicatorType + ';' + area_key + ';' + selector + '">' + lang.line('label_top_js') + '</option><option value="bottom;' + indicatorType + ';' + area_key + ';' + selector + '" selected="selected">' + lang.line('label_bottom_js') + '</option></select> ';
	titleHtml += ' ' + lang.line('label_in_js') + ' ';
	titleHtml += '<select id="select-' + selector + '" onchange="app.drawEquipmentsAUEERankingTable(this.value)" style="max-width: 125px;">';
	titleHtml += '	<option value="' + tableType + ';avail;' + area_key + ';' + selector + '" ' + ((indicatorType === 'avail') ? 'selected="selected"' : '') + '>' + lang.line('label_chart_avail_js').toUpperCase() + '</option>';
	titleHtml += '	<option value="' + tableType + ';usage;' + area_key + ';' + selector + '" ' + ((indicatorType === 'usage') ? 'selected="selected"' : '') + '>' + lang.line('label_chart_usage_js').toUpperCase() + '</option>';
	titleHtml += '	<option value="' + tableType + ';effective_usage;' + area_key + ';' + selector + '" ' + ((indicatorType === 'effective_usage') ? 'selected="selected"' : '') + '>' + lang.line('label_chart_effective_usage_js').toUpperCase() + '</option>';
	titleHtml += '</select>';
	titleHtml += ' <div class="auue-system-selector" data-uk-tooltip="{pos:\'top\'}" title="' + lang.line('label_show_hide_systems_js') + '"><i class="icon-tree"></i><input type="checkbox" style="vertical-align: middle;"/></div>';
    }

    var plantsList = [];

    for (var operation_key in this.generalIndicatorsData) {
	var operation = this.generalIndicatorsData[operation_key];
	for (var area_key in operation.areas) {
	    var area = operation.areas[area_key];
	    var accumArea = 0;
	    var periodsArea = 0;

	    for (var i = 0; i < area.indicators.length; i++) {
		var indicator = area.indicators[i];
		if (indicatorType === 'avail' && indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id === METRIC_TYPE_MONTH) {

		    for (var j = 0; j < indicator.indicator_data.length; j++) {
			var data = indicator.indicator_data[j];
			accumArea += data.value;
			periodsArea++;
		    }

		    var availabilityValue = roundFloat(accumArea * 100 / periodsArea);
		    plantsList.push([area.text, operation.text, availabilityValue]); // Se agrega la lista top, luego seran ordenadas
		}
		if (indicatorType === 'usage' && indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id === METRIC_TYPE_MONTH) {
		    for (var j = 0; j < indicator.indicator_data.length; j++) {
			var data = indicator.indicator_data[j];
			accumArea += data.value;
			periodsArea++;
		    }

		    var availabilityValue = roundFloat(accumArea * 100 / periodsArea);
		    plantsList.push([area.text, operation.text, availabilityValue]); // Se agrega la lista top, luego seran ordenadas
		}
		if (indicatorType === 'effective_usage' && indicator.indicator_type === INDICATOR_EFFECTIVE_UTILIZATION && indicator.metric_id === METRIC_TYPE_MONTH) {
		    for (var j = 0; j < indicator.indicator_data.length; j++) {
			var data = indicator.indicator_data[j];
			accumArea += data.value;
			periodsArea++;
		    }

		    var availabilityValue = roundFloat(accumArea * 100 / periodsArea);
		    plantsList.push([area.text, operation.text, availabilityValue]); // Se agrega la lista top, luego seran ordenadas
		}
	    }

	}
    }

    if (tableType === 'top') {
	plantsList.sort(function(a, b) {
	    return b[2] - a[2] // Descendente
	});
    }
    else {
	plantsList.sort(function(a, b) {
	    return a[2] - b[2] // Ascendente
	});
    }

    //var rankingHtml = '';
    var rows = [];
    // Obtener los primeros registros y generar html
    for (var i = 0; i < Math.min(plantsList.length, 5); i++) {
	//rankingHtml += '<tr><td>' + plantsList[i][0] + ' <span>' + lang.line('label_in_js') + ' ' + plantsList[i][1] + '</span></td><td>' + plantsList[i][2] + '%' + '</td></tr>';
	rows.push({title: plantsList[i][0], desc: '', value: plantsList[i][2] + '%'});
    }

    //$(targetTable).append(rankingHtml);


    var title = {
	text: titleHtml
    };

    var valueTitle = {
	text: lang.line('label_value_js')
    };

    // Se crea un nuevo espacio
    var containerHtml = '<div id="' + selector + '" class="area-auue-ranking-wrapper area-block">';
    containerHtml += this.generateRankingTable(title, valueTitle, rows, {id: 'auue-ranking-table-' + area_key, class: 'semaphore-table'});
    containerHtml += '</div>';

    if (parentSelector !== selector) {
	$(parentSelector).append('<div class="uk-width-1-4">' + containerHtml + '</div>');
    }
    else {
	$('#' + selector).parent().html(containerHtml);
    }

};

App.prototype.drawEquipmentsAUEERanking2 = function(area_key, parentSelector, indicatorType, order, metricId) {

    var selector = 'auue-ranking-' + area_key;
    var halfIndicators = ['avail', 'usage', 'effective_usage'];

    if (area_key != null) {
	var areaNode = app.getCompanyTree().get_node(area_key);
	// Query ajax granular para aprovechar cache de browser
	var ajax = new Ajax();
	ajax.reserveAccess();
	ajax.setType('json');
	ajax.setAddress('indicators/ckpi_ranking/' + metricId);
	ajax.setMethod('get');

	ajax.setCallback(function(ao) {

	    var orderSelectHtml = '<select id="top-selector-' + selector + '" onchange="app.drawEquipmentsAUEERanking2(\'' + area_key + '\',\'' + parentSelector + '\',\'' + indicatorType + '\',this.value,\'' + metricId + '\')"><option value="top" ' + ((order === 'top') ? 'selected="selected"' : '') + '>' + lang.line('label_top_js') + '</option><option value="bot" ' + ((order === 'bot') ? 'selected="selected"' : '') + '>' + lang.line('label_bottom_js') + '</option></select> ';
	    var selectHtml = '<select id="selector-' + selector + '" onchange="app.drawEquipmentsAUEERanking2(\'' + area_key + '\',\'' + parentSelector + '\',this.value,\'' + order + '\',\'' + metricId + '\')">';

	    for (var i = 0; i < halfIndicators.length; i++) {
		var value = halfIndicators[i];
		selectHtml += '<option value="' + value + '" ' + ((indicatorType === value) ? 'selected="selected"' : '') + '>' + value.toUpperCase() + '</option>';
	    }
	    selectHtml += '</select>';

	    var title = {
		text: orderSelectHtml + ' ' + lang.line('label_in_js') + ' ' + selectHtml
	    };

	    var valueTitle = {
		text: lang.line('label_value_js')
	    };

	    var list = ao.response[areaNode.original.oid + ''][order][indicatorType];
	    var rows = [];

	    for (var i = 0; i < list.length; i++) {
		rows.push({title: list[i].name, desc: '', value: roundFloat(list[i].value * 100) + ' %'});
	    }

	    $('#' + selector).html(app.generateRankingTable(title, valueTitle, rows, {id: 'auue-ranking-' + area_key, class: 'semaphore-table'}));
	});

	ajax.query();
    }

    // Se crea un nuevo espacio
    if ($('#' + selector).length === 0) {
	var containerHtml = '';
	containerHtml += '<div class="uk-width-1-4"><div id="' + selector + '" class="area-mtbi-ranking-wrapper area-block"></div>';
	$(parentSelector).append(containerHtml);
    }
};

App.prototype.updateAreaFnp = function(area_key, operation_key, indicator_type) {
    var parentSelector = $('.dashboard-area-wrapper-' + area_key + ' .fnp-group .area-block');
    var operation = this.generalIndicatorsData[operation_key];

    this.drawFnpGraph(area_key, operation, parentSelector, indicator_type);
    this.drawFnpTables(area_key, operation, parentSelector, indicator_type);
};

App.prototype.drawFnpGraph = function(area_key, operation, parentSelector, indicator_type) {


    var selector = 'fnp-chart-' + area_key;
    var containerHtml = '';

    // Se crea un nuevo chart
    // TODO: Fix multiplicidad de charts
    //console.log(parentSelector);
    if ($(parentSelector).hasClass('fnp-area-block')) {
	$(parentSelector).find('.area-fnp-chart-wrapper').remove();
	var containerHtml = '<div class="area-fnp-chart-wrapper" style="height: 100%;">';
	containerHtml += '<div id="' + selector + '" style="height: 100%;"></div>';
	containerHtml += '</div>';
	$(parentSelector).append(containerHtml);
    }
    else {
	var containerHtml = '<div class="area-fnp-chart-wrapper">';
	containerHtml += '<div id="' + selector + '" style="min-height: 300px;"></div>';
	containerHtml += '</div>';
	$(parentSelector).html(containerHtml);
    }

    var chart = new Chart();
    chart = $.extend(chart, (indicator_type === INDICATOR_FNP) ? new FnpChart() : new OeeChart());
    chart.setContainer($('#' + selector));
    chart.setMetricId(null);

    var area = operation.areas[area_key];

    chart.setTitle(area.text + ' ' + lang.line('label_chart_' + ((indicator_type === INDICATOR_FNP) ? 'fpn' : 'oee') + '_js'), false);


    for (var i = 0; i < area.indicators.length; i++) {
	var indicator = area.indicators[i];
	if (indicator.indicator_type === indicator_type) {

	    chart.addIndicator(indicator, true);
	}
    }


    chart.setChartOptions({
	chart: {
	    height: $('#' + selector).height()
	},
	title: {
	    text: chart.getTitle(),
	    x: -20 //center
	}
    });

    chart.draw(false);
    return chart;
};

App.prototype.drawFnpTables = function(area_key, operation, parentSelector, indicatorType) {

    var area = operation.areas[area_key];
    //var fnpTableHtml = '';
    var rows = [];
    for (var i = 0; i < area.indicators.length; i++) {
	var indicator = area.indicators[i];
	if (indicator.indicator_type === indicatorType) {
	    for (var j = 0; j < indicator.indicator_data.length; j++) {
		var indicator_data = indicator.indicator_data[j];
		//console.log(indicator_data);
		var values = JSON.parse(indicator_data.value);
		for (var k = 0; k < values.data.length; k++) {
		    rows.push({title: lang.line('label_' + values.data[k]['alias'] + '_js'), desc: '', value: roundFloat(values.data[k]['hours']) + 'hrs'});
		    //fnpTableHtml += '<tr><td>' + lang.line('label_' + values[k]['alias'] + '_js') + '</td><td>' + parseFloat(values[k]['hours']) + '</td></tr>';
		}
	    }

	}
    }

    var title = {
	text: lang.line('label_' + ((indicatorType === INDICATOR_FNP) ? 'fnp' : 'oee') + '_items_js')
    };

    var valueTitle = {
	text: lang.line('label_value_js')
    };

    var selector = 'fpn-ranking-' + area_key;
    var containerHtml = '';

    // Se crea un nuevo espacio
    $('#' + selector + '.area-fpn-ranking-wrapper').remove();
    var containerHtml = '<div id="' + selector + '" class="area-fpn-ranking-wrapper uk-width-1-3">';
    containerHtml += this.generateRankingTable(title, valueTitle, rows, {id: 'fpn-ranking-table-' + area_key});
    containerHtml += '</div>';
    $(parentSelector).append(containerHtml);

};

App.prototype.drawAreaProductionData = function(area_key, operation, parentSelector) {

    var selector = 'mini-production-chart-' + area_key;

    var containerHtml = '<div class="uk-width-1-4"><div class="area-production-chart-wrapper area-block">';
    //containerHtml += '<div class="area-production-chart-title">' + lang.line('label_chart_production_goal_real_js') + '</div>';
    containerHtml += '<div id="' + selector + '"></div>';
    containerHtml += '</div></div>';

    // Se crea un nuevo chart
    $(parentSelector).append(containerHtml);

    //$('.operations-gauge-charts').append('<div class="operation-gauge operation-gauge-' + area_key + '"></div>');
    var productionChart = new Chart();
    productionChart = $.extend(productionChart, new RelativeProductionChart());
    productionChart.setContainer($('#' + selector));
    productionChart.setTitle(lang.line('label_chart_production_goal_real_js'), false);
    productionChart.setMetricId(null);

    var area = operation.areas[area_key];
    var seriesData = [];
    var seriesAxis = [];

    for (var i = 0; i < area.indicators.length; i++) {
	var indicator = area.indicators[i];
	if (indicator.indicator_type === INDICATOR_PRODUCTION) {
	    for (var j = 0; j < indicator.indicator_data.length; j++) {
		var goalData = $.parseJSON(indicator.indicator_data[j].from);
		var realData = $.parseJSON(indicator.indicator_data[j].to);

		for (var gKey in goalData) {
		    var goal = goalData[gKey];
		    for (var rKey in realData) {
			// Fechas son iguales
			if (rKey == gKey) {
			    var real = realData[rKey];

			    seriesAxis.push(getMonthName(latinDateToIso(rKey, '-')));
			    seriesData.push(roundFloat((real * 100 / goal) - 100));
			}
		    }
		}
	    }
	}
    }

    // Se muestra un periodo maximo de 4 meses
    if (seriesData.length > 4) {
	seriesData = seriesData.slice(-4);
	seriesAxis = seriesAxis.slice(-4);
    }

    productionChart.setSeries([{name: lang.line('label_chart_relative_production'), data: seriesData}]);
    productionChart.setXAxis(seriesAxis);
    productionChart.draw();
};

App.prototype.showAreaDiagram = function(area_key) {
    $('#top-layer').html('').css('background-color', 'rgba(255,255,255, 0.7)');
    var blockHtml = '<div class="uk-width-medium-1-2 uk-container-center" style="margin-top:20px;">';
    blockHtml += '<div><a href="#" onClick="app.close(\'#top-layer\')"  class="uk-close uk-close-alt"></a></div>';
    blockHtml += '<div><img class="uk-align-center" onClick="app.close(\'#top-layer\')" src="/rmes-portal/assets/img/diagrama.png" alt=""></div>';
    blockHtml += '</div>';
    $('#top-layer').append(blockHtml);
    this.open('#top-layer');

};

App.prototype.mapFindMarkerById = function(areaKey) {

    var jstree = app.getCompanyTree();
    jstree.deselect_all(true);
    app.close('#left-panel', false);
    app.close('#right-panel', false);
    app.open('#map', true);

    if (areaKey == null) {
	return;
    }

    var newNodes = [];
    var areaNode = jstree.get_node(areaKey);
    var parentNode = jstree.get_node(areaNode.parent);
    for (var i = 0; i < parentNode.children.length; i++) {
	newNodes.push(jstree.get_node(parentNode.children[i]));
    }

    var newNodes = [];
    var areaNode = jstree.get_node(areaKey);
    var parentNode = jstree.get_node(areaNode.parent);
    for (var i = 0; i < parentNode.children.length; i++) {
	newNodes.push(jstree.get_node(parentNode.children[i]));
    }

    if (newNodes.length > 0) {
	drawMarkers(newNodes, 2, false, true); // TODO: eliminar constante 2
    }

    jstree.select_node(areaKey);
};

App.prototype.drawYtdOperationsGauge = function(parentSelector) {

    var selector = 'operation-ytd-gauge';
    // Se crea un nuevo chart
    $(parentSelector).append('<div class="operation-ytd-gauge operations-gauge-wrapper"></div>');
    $('.operation-ytd-gauge').append('<div class="operations-gauge-title">' + lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' YTD</div>');
    $('.operation-ytd-gauge').append('<div id="' + selector + '" class="operations-gauge-chart"></div>');

    //$('.operations-gauge-charts').append('<div class="operation-gauge operation-gauge-' + area_key + '"></div>');
    var gaugeChart = new Chart();
    var gaugeChart = $.extend(gaugeChart, new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setTitle('', false);
    gaugeChart.setYTitle('', false);
    gaugeChart.setMetricId(null);
    var gaugeSeries = [];

    var availabilityValue = 0;
    var availabilityCount = 0;
    var utilizationValue = 0;
    var utilizationCount = 0;

    for (var operation_key in this.generalIndicatorsData) {
	var operation = this.generalIndicatorsData[operation_key];
	for (var area_key in operation.areas) {
	    var area = operation.areas[area_key];

	    for (var i = 0; i < area.indicators.length; i++) {
		var indicator = area.indicators[i];
		if (indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id == METRIC_TYPE_MONTH) {
		    for (var j = 0; j < indicator.indicator_data.length; j++) {
			var data = indicator.indicator_data[j];
			availabilityValue += data.value;
			availabilityCount++;
		    }
		}
		if (indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id == METRIC_TYPE_MONTH) {
		    for (var j = 0; j < indicator.indicator_data.length; j++) {
			var data = indicator.indicator_data[j];
			utilizationValue += data.value;
			utilizationCount++;
		    }
		}
	    }
	}
    }

    gaugeSeries.push([lang.line('label_chart_avail_js'), roundFloat(availabilityValue * 100 / availabilityCount)]);
    gaugeSeries.push([lang.line('label_chart_usage_js'), roundFloat(utilizationValue * 100 / utilizationCount)]);

    gaugeChart.setSeries(gaugeSeries);
    this.setYtdOperationsGaugeData(gaugeSeries);
    gaugeChart.draw();

};

App.prototype.drawLastMonthOperationsGauge = function(parentSelector) {

    var selector = 'operation-lm-gauge';
    // Se crea un nuevo chart
    $(parentSelector).append('<div class="operation-lm-gauge operations-gauge-wrapper"></div>');
    $('.operation-lm-gauge').append('<div class="operations-gauge-title">' + lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' ' + lang.line('label_last_month_js') + ' </div>');
    $('.operation-lm-gauge').append('<div id="' + selector + '" class="operations-gauge-chart"></div>');

    //$('.operations-gauge-charts').append('<div class="operation-gauge operation-gauge-' + area_key + '"></div>');
    var gaugeChart = new Chart();
    var gaugeChart = $.extend(gaugeChart, new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setTitle('', false);
    gaugeChart.setYTitle('', false);
    gaugeChart.setMetricId(null);
    var gaugeSeries = [];

    var availabilityValue = 0;
    var availabilityCount = 0;
    var utilizationValue = 0;
    var utilizationCount = 0;

    for (var operation_key in this.generalIndicatorsData) {
	var operation = this.generalIndicatorsData[operation_key];
	for (var area_key in operation.areas) {
	    var area = operation.areas[area_key];

	    for (var i = 0; i < area.indicators.length; i++) {
		var indicator = area.indicators[i];
		if (indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id == METRIC_TYPE_MONTH) {

		    var data = indicator.indicator_data[indicator.indicator_data.length - 1];
		    availabilityValue += data.value;
		    availabilityCount++;

		}
		if (indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id == METRIC_TYPE_MONTH) {

		    var data = indicator.indicator_data[indicator.indicator_data.length - 1];
		    utilizationValue += data.value;
		    utilizationCount++;

		}
	    }
	}
    }

    gaugeSeries.push([lang.line('label_chart_avail_js'), roundFloat(availabilityValue * 100 / availabilityCount)]);
    gaugeSeries.push([lang.line('label_chart_usage_js'), roundFloat(utilizationValue * 100 / utilizationCount)]);
    gaugeChart.setSeries(gaugeSeries);
    this.setLastMonthOperationsGaugeData(gaugeSeries);
    gaugeChart.draw();

};

App.prototype.drawLastMonthGauge = function(area_key, operation, parentSelector) {

    var selector = 'plant-gauge-' + area_key;
    // Se crea un nuevo chart
    $(parentSelector).append('<div class="mini-gauge-title">' + lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' ' + lang.line('label_last_month_js') + '</div>');
    $(parentSelector).append('<div id="' + selector + '" class="mini-gauge-chart"></div>');

    //$('.operations-gauge-charts').append('<div class="operation-gauge operation-gauge-' + area_key + '"></div>');
    var gaugeChart = new Chart();
    var gaugeChart = $.extend(gaugeChart, new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setTitle(lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' YTD', false);
    gaugeChart.setYTitle(lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' YTD', false);
    gaugeChart.setMetricId(null);
    var gaugeSeries = [];

    var area = operation.areas[area_key];
    var availabilityValue = 0;
    var utilizationValue = 0;

    for (var i = 0; i < area.indicators.length; i++) {
	var indicator = area.indicators[i];
	if (indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id == METRIC_TYPE_MONTH) {
	    var accum = 0;
	    for (var j = 0; j < indicator.indicator_data.length; j++) {
		var data = indicator.indicator_data[j];
		accum += data.value * 100;
	    }
	    availabilityValue = roundFloat(accum / indicator.indicator_data.length);
	}
	if (indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id == METRIC_TYPE_MONTH) {
	    var accum = 0;
	    for (var j = 0; j < indicator.indicator_data.length; j++) {
		var data = indicator.indicator_data[j];
		accum += roundFloat(data.value * 100);
	    }
	    utilizationValue = roundFloat(accum / indicator.indicator_data.length);
	}
    }
    gaugeSeries.push([lang.line('label_chart_avail_js'), availabilityValue]);
    gaugeSeries.push([lang.line('label_chart_usage_js'), utilizationValue]);
    gaugeChart.setSeries(gaugeSeries);
//	this.setLastMonthOperationsGaugeData(gaugeSeries);
    gaugeChart.draw();

};

App.prototype.drawAreaGauges = function(area_key, operation, parentSelector, metric) {


    var area = operation.areas[area_key];
    var availabilityValue = 0;
    var utilizationValue = 0;
    var effectiveUtilizationValue = 0;
    var productionValue = 0;
    var displayParams = this.getNonStandardMetricParams(metric);

    for (var i = 0; i < area.indicators.length; i++) {
	var indicator = area.indicators[i];
	if (indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id == displayParams.metric_display) {


	    var accum = 0;
	    var accumCount = 0;
	    for (var j = 0; j < indicator.indicator_data.length; j++) {
		var data = indicator.indicator_data[j];

		// Metricas especiales requieren realizar un filtro por fecha, ya que no hay indicadores
		if (displayParams.from != null && moment(data.from, "DD/MM/YYYY").isBefore(displayParams.from)) {
		    continue;
		}
		accum += data.value * 100;
		accumCount++;
	    }
	    if (accumCount > 0) {
		availabilityValue = roundFloat(accum / accumCount);
	    }
	}
	if (indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id == displayParams.metric_display) {
	    var accum = 0;
	    var accumCount = 0;
	    for (var j = 0; j < indicator.indicator_data.length; j++) {
		var data = indicator.indicator_data[j];

		// Metricas especiales requieren realizar un filtro por fecha, ya que no hay indicadores
		if (displayParams.from != null && moment(data.from, "DD/MM/YYYY").isBefore(displayParams.from)) {
		    continue;
		}
		accum += data.value * 100;
		accumCount++;
	    }
	    if (accumCount > 0) {
		utilizationValue = roundFloat(accum / accumCount);
	    }
	}
	if (indicator.indicator_type === INDICATOR_EFFECTIVE_UTILIZATION && indicator.metric_id == displayParams.metric_display) {
	    var accum = 0;
	    var accumCount = 0;
	    for (var j = 0; j < indicator.indicator_data.length; j++) {
		var data = indicator.indicator_data[j];

		// Metricas especiales requieren realizar un filtro por fecha, ya que no hay indicadores
		if (displayParams.from != null && moment(data.from, "DD/MM/YYYY").isBefore(displayParams.from)) {
		    continue;
		}
		accum += data.value * 100;
		accumCount++;
	    }

	    if (accumCount > 0) {
		effectiveUtilizationValue = roundFloat(accum / accumCount);
	    }
	}
	if (indicator.indicator_type === INDICATOR_PRODUCTION && indicator.metric_id == null) {
	    var accumTarget = 0;
	    var accumReal = 0;
	    if (indicator.indicator_data.length > 0) {
		var data = indicator.indicator_data[indicator.indicator_data.length - 1];
		var targetProduction = JSON.parse(data.from);
		var realProduction = JSON.parse(data.to);

		for (var production_key in realProduction) {
		    accumTarget += parseFloat(targetProduction[production_key]);
		    accumReal += parseFloat(realProduction[production_key]);
		}
	    }

	    if (accumTarget !== 0)
		productionValue = roundFloat(accumReal * 100 / accumTarget);
	    else
		productionValue = 0;
	}
    }


    var gaugeSeries = [];
    gaugeSeries.push([lang.line('label_chart_usage_js'), utilizationValue]);
    gaugeSeries.push([lang.line('label_chart_avail_js'), availabilityValue]);
//	this.setLastMonthOperationsGaugeData(gaugeSeries);

    // Availability
    var selector = 'plant-gauge-availability-' + area_key;
    // Se crea un nuevo chart
    $(parentSelector).append('<div id="' + selector + '" class="mini-gauge-chart"></div>');
    var gaugeChart = $.extend(new Chart(), new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setMetricId(null);
    gaugeChart.setSeries([[lang.line('label_chart_avail_js'), availabilityValue]]);
    gaugeChart.draw();
    delete gaugeChart;

    // Utilization
    var selector = 'plant-gauge-utilization-' + area_key;
    // Se crea un nuevo chart
    $(parentSelector).append('<div id="' + selector + '" class="mini-gauge-chart"></div>');
    var gaugeChart = $.extend(new Chart(), new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setMetricId(null);
    gaugeChart.setSeries([[lang.line('label_chart_usage_js'), utilizationValue]]);
    gaugeChart.draw();
    delete gaugeChart;

    // Effective Utilization
    var selector = 'plant-gauge-effective-utilization-' + area_key;
    // Se crea un nuevo chart
    $(parentSelector).append('<div id="' + selector + '" class="mini-gauge-chart"></div>');
    var gaugeChart = $.extend(new Chart(), new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setMetricId(null);
    gaugeChart.setSeries([[lang.line('label_chart_effective_usage_js'), effectiveUtilizationValue]]);
    gaugeChart.draw();
    delete gaugeChart;

    // Production
    var selector = 'plant-gauge-production-' + area_key;
    // Se crea un nuevo chart
    $(parentSelector).append('<div id="' + selector + '" class="mini-gauge-chart"></div>');
    var gaugeChart = $.extend(new Chart(), new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setMetricId(null);
    gaugeChart.setSeries([[lang.line('label_chart_production_js'), productionValue]]);
    gaugeChart.draw();
    delete gaugeChart;

};

App.prototype.drawYTDGauge = function(area_key, operation, parentSelector) {

    var selector = 'plant-gauge-ytd-' + area_key;
    // Se crea un nuevo chart
    $(parentSelector).append('<div class="mini-gauge-title">' + lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' YTD' + '</div>');
    $(parentSelector).append('<div id="' + selector + '" class="mini-gauge-chart"></div>');

    //$('.operations-gauge-charts').append('<div class="operation-gauge operation-gauge-' + area_key + '"></div>');
    var gaugeChart = new Chart();
    var gaugeChart = $.extend(gaugeChart, new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setTitle(lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' YTD', false);
    gaugeChart.setYTitle(lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' YTD', false);
    gaugeChart.setMetricId(null);
    var gaugeSeries = [];

    var area = operation.areas[area_key];
    var availabilityValue = 0;
    var utilizationValue = 0;

    for (var i = 0; i < area.indicators.length; i++) {
	var indicator = area.indicators[i];
	if (indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id == METRIC_TYPE_MONTH) {
	    var accum = 0;
	    for (var j = 0; j < indicator.indicator_data.length; j++) {
		var data = indicator.indicator_data[j];
		accum += data.value * 100;
	    }
	    availabilityValue = roundFloat(accum / indicator.indicator_data.length);
	}
	if (indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id == METRIC_TYPE_MONTH) {
	    var accum = 0;
	    for (var j = 0; j < indicator.indicator_data.length; j++) {
		var data = indicator.indicator_data[j];
		accum += roundFloat(data.value * 100);
	    }
	    utilizationValue = roundFloat(accum / indicator.indicator_data.length);
	}
    }
    gaugeSeries.push([lang.line('label_chart_avail_js'), availabilityValue]);
    gaugeSeries.push([lang.line('label_chart_usage_js'), utilizationValue]);
    gaugeChart.setSeries(gaugeSeries);
    gaugeChart.draw();

};

App.prototype.drawLastMonthData = function(area_key, operation, parentSelector) {

    var selector = 'plant-lm-' + area_key;
    // Se crea un nuevo chart
    $(parentSelector).append('<div id="' + selector + '" class="right-panel-chart plant-lm"></div>');
    var chart = new Chart();
    var generalChart = $.extend(chart, new GeneralOperationInfoChart());
    generalChart.setContainer($('#' + selector));
    generalChart.setTitle(lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' ' + lang.line('label_last_month_js'), false);
    generalChart.setYTitle(lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' %', false);
    generalChart.setMetricId(METRIC_TYPE_WEEK);

    var serie = {name: lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js'), data: []};
    var area = operation.areas[area_key];
    var rangeDataObj = {};

    for (var i = 0; i < area.indicators.length; i++) {
	var indicator = area.indicators[i];
	if (indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id == METRIC_TYPE_WEEK) {
	    for (var j = 0; j < indicator.indicator_data.length; j++) {
		var data = indicator.indicator_data[j];

		if (typeof rangeDataObj[data.from] == 'undefined') {
		    rangeDataObj[data.from] = [null, roundFloat(data.value * 100)];
		}
		else {
		    rangeDataObj[data.from][1] = roundFloat(data.value * 100);
		}
	    }
	}
	if (indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id == METRIC_TYPE_WEEK) {
	    for (var j = 0; j < indicator.indicator_data.length; j++) {
		var data = indicator.indicator_data[j];

		if (typeof rangeDataObj[data.from] == 'undefined') {
		    rangeDataObj[data.from] = [roundFloat(data.value * 100), null];
		}
		else {
		    rangeDataObj[data.from][0] = roundFloat(data.value * 100);
		}
	    }
	}
    }

    // Se obtuvieron los indicadores para realizar la comparacion
    // Preparar serie y agregar
    for (var range_key in rangeDataObj) {
	var rangeData = rangeDataObj[range_key];
	var date = new Date(range_key);
	serie.data.push([date.getTime(), rangeData[0], rangeData[1]]);
    }
    generalChart.setSeries(serie);
    generalChart.draw();

};
App.prototype.getNonStandardMetricParams = function(metricId) {

    var nonStandardMetrics = [
	[METRIC_TYPE_YTD, METRIC_TYPE_MONTH],
	[METRIC_TYPE_MTD, METRIC_TYPE_WEEK],
	[METRIC_TYPE_MY, METRIC_TYPE_MONTH],
	[METRIC_TYPE_FMTD, METRIC_TYPE_WEEK]
    ];

    for (var i = 0; i < nonStandardMetrics.length; i++) {
	if (nonStandardMetrics[i][0] == metricId) {

	    var dateFrom = moment();
	    switch (parseInt(metricId)) {
		case METRIC_TYPE_YTD:
		    dateFrom = moment().startOf('year');
		    break;
		case METRIC_TYPE_MTD:
		    dateFrom = moment().startOf('month');
		    break;
		case METRIC_TYPE_MY:
		    dateFrom = moment().subtract(1, 'year').startOf('month');
		    break;
		case METRIC_TYPE_FMTD:
		    dateFrom = moment().subtract(4, 'month').startOf('month');
		    break;
	    }
	    return {metric_display: nonStandardMetrics[i][1], from: dateFrom};
	}
    }

    return {metric_display: metricId, from: null}; //1 2 o 3
};
App.prototype.drawYtdData = function(area_key, operation, parentSelector, metric) {

    var selector = 'plant-ytd-' + area_key;
    // Se crea un nuevo chart
    $(parentSelector).append('<div class="uk-width-1-4"><div id="' + selector + '" class="area-block plant-ytd"></div></div>');
    var chart = new Chart();
    var generalChart = $.extend(chart, new GeneralOperationInfoChart());
    generalChart.setContainer($('#' + selector));
    generalChart.setTitle(lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js'), false);
    generalChart.setYTitle(lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' %', false);
    generalChart.setMetricId();

    var serie = {name: lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js'), data: []};
    var area = operation.areas[area_key];
    var rangeDataObj = {};

    //var isStandardMetric = $.inArray(metric, [METRIC_TYPE_DAY, METRIC_TYPE_WEEK, METRIC_TYPE_MONTH]) === -1;

    var displayParams = this.getNonStandardMetricParams(metric);

    for (var i = 0; i < area.indicators.length; i++) {
	var indicator = area.indicators[i];


	if (indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id == displayParams.metric_display) {

	    for (var j = 0; j < indicator.indicator_data.length; j++) {
		var data = indicator.indicator_data[j];

		// Metricas especiales requieren realizar un filtro por fecha, ya que no hay indicadores
		if (displayParams.from != null && moment(data.from, "DD/MM/YYYY").isBefore(displayParams.from)) {
		    continue;
		}

		if (typeof rangeDataObj[data.from] == 'undefined') {
		    rangeDataObj[data.from] = [null, roundFloat(data.value * 100)];

		}
		else {
		    rangeDataObj[data.from][1] = roundFloat(data.value * 100);

		}
	    }
	}
	if (indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id == displayParams.metric_display) {
	    for (var j = 0; j < indicator.indicator_data.length; j++) {
		var data = indicator.indicator_data[j];

		// Metricas especiales requieren realizar un filtro por fecha, ya que no hay indicadores
		if (displayParams.from != null && moment(data.from, "DD/MM/YYYY").isBefore(displayParams.from)) {
		    continue;
		}


		if (typeof rangeDataObj[data.from] == 'undefined') {
		    rangeDataObj[data.from] = [roundFloat(data.value * 100), null];
		}
		else {
		    rangeDataObj[data.from][0] = roundFloat(data.value * 100);
		}
	    }
	}
    }


    // Se obtuvieron los indicadores para realizar la comparacion
    // Preparar serie y agregar
    for (var range_key in rangeDataObj) {
	var rangeData = rangeDataObj[range_key];
	var date = new Date(latinDateToIso(range_key));

	serie.data.push([date.getTime(), rangeData[0], rangeData[1]]);
    }
    generalChart.setSeries(serie);
    generalChart.draw();

};

App.prototype.loadBasicView = function() {
    indicatorsWrapper.destroy();
    app.open('#left-panel', true);
    generateDashboardCompanyTree(companyStructure, '#jstree');
};

App.prototype.loadAdvancedView = function() {
    indicatorsWrapper.destroy();
    app.open('#left-panel', true);
    generateDashboardCompanyTree(companyStructure, '#jstree');
};

/**
 * Carga la informacion principal para ser mostrada en el panel derecho
 * @returns {Boolean}
 */
App.prototype.loadGeneralData = function() {


    setTimeout(function() { // Se separa la carga de datos para dejar que otras cosas carguen
	if (app.getCompanyTree() === null) {
	    return false;
	}
	var operations = app.getOperations();
	var queryGroup = [];
	var parent = app;

	for (var i = 0; i < operations.length; i++) {
	    var operation = operations[i];
	    var areas = app.getAreasByParent(operation);

	    // Se crea el objeto operacion
	    if (typeof parent.generalIndicatorsData[operations[i].id] == 'undefined') {
		parent.generalIndicatorsData[operations[i].id] = operations[i];
		parent.generalIndicatorsData[operations[i].id].areas = {};
	    }

	    for (var j = 0; j < areas.length; j++) {
		var area = areas[j];

		var requiredIndicators = [[INDICATOR_AVAILABILITY, METRIC_TYPE_MONTH],
		    [INDICATOR_AVAILABILITY, METRIC_TYPE_WEEK],
		    [INDICATOR_UTILIZATION, METRIC_TYPE_MONTH],
		    [INDICATOR_UTILIZATION, METRIC_TYPE_WEEK],
		    [INDICATOR_EFFECTIVE_UTILIZATION, METRIC_TYPE_MONTH],
		    [INDICATOR_EFFECTIVE_UTILIZATION, METRIC_TYPE_WEEK],
		    [INDICATOR_OEE, null],
		    [INDICATOR_PRODUCTION, null],
		    [INDICATOR_FNP, null]];


		// Se crea el objeto area
		if (typeof parent.generalIndicatorsData[operations[i].id].areas[areas[j].id] == 'undefined') {
		    parent.generalIndicatorsData[operations[i].id].areas[areas[j].id] = areas[j];
		    parent.generalIndicatorsData[operations[i].id].areas[areas[j].id].indicators = [];
		}

		if (area.data.subsystem_ref != null) {
		    for (var k = 0; k < requiredIndicators.length; k++) {
			// Query ajax granular para aprovechar cache de browser
			var ajax = new Ajax();
			ajax.reserveAccess();
			ajax.setAddress('indicators/get_indicator');
			ajax.setMethod('post');
			ajax.setParameters({
			    indicator_type_id: requiredIndicators[k][0],
			    metric_id: requiredIndicators[k][1],
			    report_id: null,
			    subsystem_id: area.data.subsystem_ref
			});

			// Variables Custom
			//ajax.operation_index = i;
			//ajax.area_index = j;
			ajax.parent_operation_id = operations[i].id;
			ajax.parent_area_id = areas[j].id;

			ajax.setCallback(function(ao) {
			    for (var n = 0, size = ao.response.length; n < size; n++) {
				var indicator = ao.response[n];

				// Se agrega el indicador
				parent.generalIndicatorsData[ao.parent_operation_id].areas[ao.parent_area_id].indicators.push(indicator);
			    }
			});

			var queryRef = ajax.query();
			queryGroup.push(queryRef);

		    }
		}
	    }
	}

	$.when.apply($, queryGroup).done(function() {
	    if (user.id_user == 32) {
		
		var userNode = app.getCompanyTree().get_node('j1_308');
		app.drawOperationGeneralData(userNode, null);
	    } else {
		app.drawGeneralData();
	    }
	});
    }, 500);
};

App.prototype.recoverMap = function(display) {

    if (typeof display == 'undefined')
	display = false;

    // Se trae el mapa, pues quizas fue removido para otros usos
    if ($('body > #map').length === 0) { // Mapa no esta como capa
	$("#map").detach().appendTo('body').css('display', (display) ? 'block' : 'none');

	// TODO: redefinir automaticamente
	var latlng = new google.maps.LatLng(-25.555739, -70.026151);
	$(window).resize(function() {
	    google.maps.event.trigger(map, "resize");
	    map.setCenter(latlng);
	});
    }
};


App.prototype.drawYtdAreaGauge = function(parentSelector, idArea) {

    var selector = 'operation-ytd-gauge';
    // Se crea un nuevo chart
//    $(parentSelector).append('<div class="operation-ytd-gauge operations-gauge-wrapper"></div>');
//    $('.operation-ytd-gauge').append('<div class="operations-gauge-title">' + lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' YTD</div>');
//    $('.operation-ytd-gauge').append('<div id="' + selector + '" class="operations-gauge-chart"></div>');

    //$('.operations-gauge-charts').append('<div class="operation-gauge operation-gauge-' + area_key + '"></div>');
    var gaugeChart = new Chart();
    var gaugeChart = $.extend(gaugeChart, new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setTitle('', false);
    gaugeChart.setYTitle('', false);
    gaugeChart.setMetricId(null);
    var gaugeSeries = [];

    var availabilityValue = 0;
    var availabilityCount = 0;
    var utilizationValue = 0;
    var utilizationCount = 0;

    for (var operation_key in this.generalIndicatorsData) {
	var operation = this.generalIndicatorsData[operation_key];
	for (var area_key in operation.areas) {
	    var area = operation.areas[area_key];
	    if (area.id == idArea) {
		for (var i = 0; i < area.indicators.length; i++) {
		    var indicator = area.indicators[i];
		    if (indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id == METRIC_TYPE_MONTH) {
			for (var j = 0; j < indicator.indicator_data.length; j++) {
			    var data = indicator.indicator_data[j];
			    availabilityValue += data.value;
			    availabilityCount++;
			}
		    }
		    if (indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id == METRIC_TYPE_MONTH) {
			for (var j = 0; j < indicator.indicator_data.length; j++) {
			    var data = indicator.indicator_data[j];
			    utilizationValue += data.value;
			    utilizationCount++;
			}
		    }
		}
	    }
	}
    }

    gaugeSeries.push([lang.line('label_chart_avail_js'), roundFloat(availabilityValue * 100 / availabilityCount)]);
    gaugeSeries.push([lang.line('label_chart_usage_js'), roundFloat(utilizationValue * 100 / utilizationCount)]);
    gaugeChart.setSeries(gaugeSeries);
    gaugeChart.draw();

};

App.prototype.drawLastMonthAreaGauge = function(parentSelector, idArea) {

    var selector = 'operation-lm-gauge';
    // Se crea un nuevo chart
//    $(parentSelector).append('<div class="operation-lm-gauge operations-gauge-wrapper"></div>');
//    $('.operation-lm-gauge').append('<div class="operations-gauge-title">' + lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' ' + lang.line('label_last_month_js') + ' </div>');
//    $('.operation-lm-gauge').append('<div id="' + selector + '" class="operations-gauge-chart"></div>');

    //$('.operations-gauge-charts').append('<div class="operation-gauge operation-gauge-' + area_key + '"></div>');
    var gaugeChart = new Chart();
    var gaugeChart = $.extend(gaugeChart, new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setTitle('', false);
    gaugeChart.setYTitle('', false);
    gaugeChart.setMetricId(null);
    var gaugeSeries = [];

    var availabilityValue = 0;
    var availabilityCount = 0;
    var utilizationValue = 0;
    var utilizationCount = 0;

    for (var operation_key in this.generalIndicatorsData) {
	var operation = this.generalIndicatorsData[operation_key];
	for (var area_key in operation.areas) {
	    var area = operation.areas[area_key];
	    if (area.id == idArea) {
		for (var i = 0; i < area.indicators.length; i++) {
		    var indicator = area.indicators[i];
		    if (indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id == METRIC_TYPE_MONTH) {

			var data = indicator.indicator_data[indicator.indicator_data.length - 1];
			availabilityValue += data.value;
			availabilityCount++;

		    }
		    if (indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id == METRIC_TYPE_MONTH) {

			var data = indicator.indicator_data[indicator.indicator_data.length - 1];
			utilizationValue += data.value;
			utilizationCount++;

		    }
		}
	    }
	}
    }

    gaugeSeries.push([lang.line('label_chart_avail_js'), roundFloat(availabilityValue * 100 / availabilityCount)]);
    gaugeSeries.push([lang.line('label_chart_usage_js'), roundFloat(utilizationValue * 100 / utilizationCount)]);
    gaugeChart.setSeries(gaugeSeries);
    gaugeChart.draw();

};

App.prototype.drawYtdOperationGauge = function(parentSelector, idOperation) {

    var selector = 'operation-ytd-gauge';
    // Se crea un nuevo chart
//    $(parentSelector).append('<div class="operation-ytd-gauge operations-gauge-wrapper"></div>');
//    $('.operation-ytd-gauge').append('<div class="operations-gauge-title">' + lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' YTD</div>');
//    $('.operation-ytd-gauge').append('<div id="' + selector + '" class="operations-gauge-chart"></div>');

    //$('.operations-gauge-charts').append('<div class="operation-gauge operation-gauge-' + area_key + '"></div>');
    var gaugeChart = new Chart();
    var gaugeChart = $.extend(gaugeChart, new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setTitle('', false);
    gaugeChart.setYTitle('', false);
    gaugeChart.setMetricId(null);
    var gaugeSeries = [];

    var availabilityValue = 0;
    var availabilityCount = 0;
    var utilizationValue = 0;
    var utilizationCount = 0;

    for (var operation_key in this.generalIndicatorsData) {
	var operation = this.generalIndicatorsData[operation_key];
	if (operation.id == idOperation) {
	    for (var area_key in operation.areas) {
		var area = operation.areas[area_key];

		for (var i = 0; i < area.indicators.length; i++) {
		    var indicator = area.indicators[i];
		    if (indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id == METRIC_TYPE_MONTH) {
			for (var j = 0; j < indicator.indicator_data.length; j++) {
			    var data = indicator.indicator_data[j];
			    availabilityValue += data.value;
			    availabilityCount++;
			}
		    }
		    if (indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id == METRIC_TYPE_MONTH) {
			for (var j = 0; j < indicator.indicator_data.length; j++) {
			    var data = indicator.indicator_data[j];
			    utilizationValue += data.value;
			    utilizationCount++;
			}
		    }
		}
	    }
	}
    }

    gaugeSeries.push([lang.line('label_chart_avail_js'), roundFloat(availabilityValue * 100 / availabilityCount)]);
    gaugeSeries.push([lang.line('label_chart_usage_js'), roundFloat(utilizationValue * 100 / utilizationCount)]);
    gaugeChart.setSeries(gaugeSeries);
    gaugeChart.draw();

};

App.prototype.drawLastMonthOperationGauge = function(parentSelector, idOperation) {

    var selector = 'operation-lm-gauge';
    // Se crea un nuevo chart
//    $(parentSelector).append('<div class="operation-lm-gauge operations-gauge-wrapper"></div>');
//    $('.operation-lm-gauge').append('<div class="operations-gauge-title">' + lang.line('label_chart_usage_js') + ' vs ' + lang.line('label_chart_avail_js') + ' ' + lang.line('label_last_month_js') + ' </div>');
//    $('.operation-lm-gauge').append('<div id="' + selector + '" class="operations-gauge-chart"></div>');

    //$('.operations-gauge-charts').append('<div class="operation-gauge operation-gauge-' + area_key + '"></div>');
    var gaugeChart = new Chart();
    var gaugeChart = $.extend(gaugeChart, new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setTitle('', false);
    gaugeChart.setYTitle('', false);
    gaugeChart.setMetricId(null);
    var gaugeSeries = [];

    var availabilityValue = 0;
    var availabilityCount = 0;
    var utilizationValue = 0;
    var utilizationCount = 0;

    for (var operation_key in this.generalIndicatorsData) {
	var operation = this.generalIndicatorsData[operation_key];
	if (operation.id == idOperation) {
	    for (var area_key in operation.areas) {
		var area = operation.areas[area_key];

		for (var i = 0; i < area.indicators.length; i++) {
		    var indicator = area.indicators[i];
		    if (indicator.indicator_type === INDICATOR_AVAILABILITY && indicator.metric_id == METRIC_TYPE_MONTH) {

			var data = indicator.indicator_data[indicator.indicator_data.length - 1];
			availabilityValue += data.value;
			availabilityCount++;

		    }
		    if (indicator.indicator_type === INDICATOR_UTILIZATION && indicator.metric_id == METRIC_TYPE_MONTH) {

			var data = indicator.indicator_data[indicator.indicator_data.length - 1];
			utilizationValue += data.value;
			utilizationCount++;

		    }
		}
	    }
	}
    }

    gaugeSeries.push([lang.line('label_chart_avail_js'), roundFloat(availabilityValue * 100 / availabilityCount)]);
    gaugeSeries.push([lang.line('label_chart_usage_js'), roundFloat(utilizationValue * 100 / utilizationCount)]);
    gaugeChart.setSeries(gaugeSeries);
    gaugeChart.draw();

};



App.prototype.updateDashboard = function(actualNode) {

    var tree = app.getCompanyTree();
    var level = -1;
    var areaNode = null;
    var newNodes = [];

    if (typeof actualNode !== 'undefined' && actualNode !== null) {
	areaNode = tree.get_node(actualNode);
	level = parseInt(areaNode.original.node_type);
    }

    switch (level) {
	case 0: // Llamado desde operacion
	    this.drawYtdOperationGauge('.operations-gauges', actualNode);
	    this.drawLastMonthOperationGauge('.operations-gauges', actualNode);

	    $('.sparkline-title').each(function() {
		if ($(this).attr('data-operation-id') == areaNode.original.oid) {
		    app.toggleTab(this, 'open');
		}
		else {
		    app.toggleTabHide(this);
		}
	    });
	    $('.spark-row').removeClass('active');

	    // si entro en una operación se muestran todas sus plantas
	    if (areaNode.original.node_type == 0) {
		for (var i = 0; i < areaNode.children.length; i++) {
		    newNodes.push(tree.get_node(areaNode.children[i]));
		}
		if (newNodes.length > 0) {
		    drawMarkers(newNodes, 2, false, true); // TODO: eliminar constante 2
		}
	    }

	    break;
	case 1: // Llamado desde planta
	    this.drawYtdAreaGauge('.operations-gauges', actualNode);
	    this.drawLastMonthAreaGauge('.operations-gauges', actualNode);
	    $('.spark-row-' + areaNode.id).addClass('active');

	    break;
	default: // Llamado desde cualquier lugar
	    var jsTreeNodes = app.getCompanyTree()._model.data;
	    drawMarkers(jsTreeNodes, 1, false, false);
	    this.drawYtdOperationsGaugeNew();
	    this.drawLastMonthOperationsGaugeNew();
	    $('.sparkline-title').each(function(i, e) {

		if (i !== 0) {
		    app.toggleTabHide(this);
		}
	    });
	    $('.spark-row').removeClass('active');

	    break;
    }

};

App.prototype.toggleTabHide = function(element) {
    var parent = $(element).parent('.sparkline-wrapper');
    var icon = parent.find('.sparkline-title i');

    if (icon.hasClass('icon-up')) {
	icon.removeClass('icon-up').addClass('icon-down');
	parent.find('.sparkline-table-wrapper').toggle(250);
    }

};

App.prototype.toggleTabShow = function(element) {
    var parent = $(element).parent('.sparkline-wrapper');
    var icon = parent.find('.sparkline-title i');

    if (icon.hasClass('icon-down')) {
	icon.removeClass('icon-down').addClass('icon-up');
	parent.find('.sparkline-table-wrapper').toggle(250);
    }

};

App.prototype.drawYtdOperationsGaugeNew = function() {

    var selector = 'operation-ytd-gauge';
    var gaugeChart = new Chart();
    var gaugeChart = $.extend(gaugeChart, new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setTitle('', false);
    gaugeChart.setYTitle('', false);
    gaugeChart.setMetricId(null);
    gaugeChart.setSeries(this.getYtdOperationsGaugeData());
    gaugeChart.draw();

};

App.prototype.drawLastMonthOperationsGaugeNew = function() {

    var selector = 'operation-lm-gauge';
    var gaugeChart = new Chart();
    var gaugeChart = $.extend(gaugeChart, new GaugeChart());
    gaugeChart.setContainer($('#' + selector));
    gaugeChart.setTitle('', false);
    gaugeChart.setYTitle('', false);
    gaugeChart.setMetricId(null);
    gaugeChart.setSeries(this.getLastMonthOperationsGaugeData());
    gaugeChart.draw();

};

App.prototype.setYtdOperationsGaugeData = function(data) {
    this.ytdOperationGaugeData = data;
};

App.prototype.setLastMonthOperationsGaugeData = function(data) {
    this.lastMonthOperationsGaugeData = data;
};

App.prototype.getYtdOperationsGaugeData = function() {
    return this.ytdOperationGaugeData;
};

App.prototype.getLastMonthOperationsGaugeData = function() {
    return this.lastMonthOperationsGaugeData;
};

App.prototype.loadDashboard = function() {
    this.clearUi(true, false);
    this.open('#right-panel', true);

    this.drawGeneralData();
};

// TODO: Unfelipy
App.prototype.drawMainDrilldownChartNew = function(indicatorType) {

    // Se crea un nuevo chart
    var generalChart = $.extend(new Chart(), new GeneralInfoChart());

    // Se elimina grafico fpn general
    $('.operations-general-chart-fpn').detach();

    generalChart.setContainer($('.operations-general-chart').show());
    generalChart.setTitle(lang.line('label_chart_' + indicatorType + '_js') + ' ' + lang.line('label_in_js') + ' ' + lang.line('label_main_process_features'), false);
    generalChart.setYTitle(lang.line('label_chart_' + indicatorType + '_js') + ' YTD %', false);
    generalChart.setMetricId(METRIC_TYPE_MONTH);

    var series = [];
    var drilldownSeries = [];
    var operationCategories = [];

    for (var operation_key in this.generalIndicatorsData) {
	var operation = this.generalIndicatorsData[operation_key];

	var accumOperation = 0;
	var periodsOperation = 0;
	var drilldownData = [];
	var categories = [];
	var datal = [];
	var areaName = [];
	var areaIds = [];

	for (var area_key in operation.areas) {

	    var area = operation.areas[area_key];
	    categories.push(area.text);
	    var accumArea = 0;
	    var periodsArea = 0;

	    for (var i = 0; i < area.indicators.length; i++) {
		var indicator = area.indicators[i];
		var auxDictionary = {avail: [INDICATOR_AVAILABILITY, METRIC_TYPE_MONTH], usage: [INDICATOR_UTILIZATION, METRIC_TYPE_MONTH], effective_usage: [INDICATOR_EFFECTIVE_UTILIZATION, METRIC_TYPE_MONTH]};

		if (indicator.indicator_type === auxDictionary[indicatorType][0] && indicator.metric_id === auxDictionary[indicatorType][1]) {

		    for (var j = 0; j < indicator.indicator_data.length; j++) {
			var data = indicator.indicator_data[j];
			accumOperation += data.value;
			accumArea += data.value;
			periodsOperation++;
			periodsArea++;
		    }

		    drilldownData.push({
			name: area.text,
			y: roundFloat(accumArea * 100 / periodsArea),
			drilldown: 'second_drilldown_' + area.text
		    });
		}

	    }
	    datal.push(roundFloat(accumArea * 100 / periodsArea));
	    areaName.push(area.text);
	    areaIds.push(area.id);
	}

	var newDrilldownSerie = {
	    id: 'drill_' + operation.text,
	    name: lang.line('label_areas_js') + ' ' + operation.text,
	    data: drilldownData
	};
	drilldownSeries.push(newDrilldownSerie);

	var drilldownData = {
	    name: areaName,
	    area_ids: areaIds,
	    categories: categories,
	    data: datal
	};

	var newSerieDataTest = {
	    name: operation.text,
	    y: roundFloat(accumOperation * 100 / periodsOperation),
	    drilldown: drilldownData,
	    id_tree: operation.id
	};

	series.push(newSerieDataTest);
	operationCategories.push(operation.text);

    }

    var serieNew = {data: series, colorByPoint: true};

    generalChart.setSeries(serieNew);
    generalChart.setDrilldownSeries(operationCategories);
    generalChart.draw();

    var indicatorSelectorHtml = '<div class="top-table-selector">';
    indicatorSelectorHtml += '<a href="javascript:app.drawMainDrilldownChartNew(\'avail\')" data-uk-tooltip="{pos:\'bottom\'}" title="' + lang.line('label_chart_avail_js') + '" class="' + ((indicatorType === 'avail') ? 'active' : '') + '">&nbsp;</a>';
    indicatorSelectorHtml += '<a href="javascript:app.drawMainDrilldownChartNew(\'usage\')" data-uk-tooltip="{pos:\'bottom\'}" title="' + lang.line('label_chart_usage_js') + '" class="' + ((indicatorType === 'usage') ? 'active' : '') + '">&nbsp;</a>';
    indicatorSelectorHtml += '<a href="javascript:app.drawMainDrilldownChartNew(\'effective_usage\')" data-uk-tooltip="{pos:\'bottom\'}" title="' + lang.line('label_chart_effective_usage_js') + '" class="' + ((indicatorType === 'effective_usage') ? 'active' : '') + '">&nbsp;</a>';
    indicatorSelectorHtml += '</div>';

    // Se agrega el selector inferior
    $('.operations-general-wrapper .top-table-selector').remove();
    $('.operations-general-wrapper').append(indicatorSelectorHtml);

    // Si no es la primera vez que se ejecuta, entonces se debe refrescar el dashboard
    if (this.started) {
	this.updateDashboard(null);
    }

    this.started = true;
};

App.prototype.generateRankingTable = function(title, valueTitle, rows, tableAttrs) {

    var tableHtml = '<div ' + ((typeof tableAttrs.id !== 'undefined') ? 'id=' + tableAttrs.id : '') + ' class="ranking-table default-table ' + ((typeof tableAttrs.class !== 'undefined') ? tableAttrs.class : '') + '">';

    tableHtml += '<div class="ranking-header">';

    if (valueTitle !== null) {
	tableHtml += '	<div class="ranking-title">' + title.text + '</div>'
	tableHtml += '	<div class="ranking-value-title">' + valueTitle.text + '</div>';
    }
    else {
	tableHtml += '	<div class="ranking-title" style="width: 100%;">' + title.text + '</div>'
    }
    tableHtml += '</div>';

    tableHtml += '<div class="ranking-body">';

    if (rows.length > 0) {
	for (var i = 0; i < rows.length; i++) {
	    var row = rows[i];

	    var statusClass = '';
	    if (row.value >= GOAL_SUCCESS) {
		statusClass = 'success';
	    }
	    else {
		if (row.value >= GOAL_WARNING) {
		    statusClass = 'warning';
		}
		else {
		    statusClass = 'danger';
		}
	    }

	    tableHtml += '<div class="ranking-row ' + statusClass + '">';
	    tableHtml += '	<div class="ranking-row-title">' + row.title + '<span class="ranking-row-desc">' + row.desc + '</span></div>';
	    tableHtml += '	<div class="ranking-row-value">' + row.value + '</div>';
	    tableHtml += '</div>';
	}
    }
    else {
	tableHtml += '<div class="ranking-empty>' + lang.line('label_no_results_found_js') + '</div>';
    }

    tableHtml += '</div>'; // fin .ranking-body
    tableHtml += '</div>'; // fin .ranking-table

    return tableHtml;

};
