
function Block() {
	this.type = 0;
	this.element = null;
	this.parent = null;
	this.chart = null;
	this.content = '';
	this.customClass = 'uk-width-large-1-2';
	this.availableWidths = ['uk-width-large-1-3', 'uk-width-large-1-2', 'uk-width-large-1-1'];
	this.title = '';
	this.icon = '';
	this.options = null;
}

Block.prototype.setType = function(type) {
	this.type = type;
};
Block.prototype.getType = function() {
	return this.type;
};
Block.prototype.setElement = function(element) {
	this.element = element;
};
Block.prototype.getElement = function() {
	return this.element;
};
Block.prototype.getContentElement = function() {
	return $(this.element).children('.block-content');
};
Block.prototype.setParent = function(parent) {
	this.parent = parent;
};
Block.prototype.getParent = function() {
	return this.parent;
};
Block.prototype.setChart = function(chart) {
	this.chart = chart;
	if (this.getContentElement() > 0)
		this.chart.setContainer(this.getContentElement());
};
Block.prototype.getChart = function() {
	return this.chart;
};
Block.prototype.setContent = function(content) {
	this.content = content;
	this.getContentElement().html(content);
};
Block.prototype.getContent = function() {
	return this.chart;
};
Block.prototype.getChartByDom = function() {
	return $(this.element).highCharts();
};
Block.prototype.setParent = function(parent) {
	this.parent = parent;
};
Block.prototype.getParent = function() {
	return this.parent;
};
Block.prototype.setCustomClass = function(customClass) {
	this.customClass = customClass;
	$(this.element).attr('class', 'block ' + this.customClass);
};
Block.prototype.getCustomClass = function() {
	return this.customClass;
};
Block.prototype.setTitle = function(title) {
	this.title = title;
};
Block.prototype.getTitle = function() {
	return this.title;
};
Block.prototype.setIcon = function(icon) {
	this.icon = icon;
};
Block.prototype.getIcon = function() {
	return this.icon;
};
Block.prototype.setOptions = function(options) {
	this.options = options;
};
Block.prototype.getOptions = function() {
	return this.options;
};
Block.prototype.updateWidth = function(changeValue) {
	var currentCustomClass = this.getCustomClass();
	var widthClass = this.getWidthClass();
	var classIndex = $.inArray(widthClass, this.availableWidths);
	if (classIndex === -1 || widthClass === '') { // Si la clase no se encuentra, entonces se agrega la clase por defecto
		this.setCustomClass(currentCustomClass + ' ' + this.availableWidths[1]); // 1 es 50% ancho
	}
	else {
		if (classIndex < this.availableWidths.length) {
			$(this.element).removeClass(widthClass);
			var newCustomClass = currentCustomClass.replace(widthClass, this.availableWidths[classIndex + changeValue]);
			this.setCustomClass(newCustomClass);
			setTimeout(function() {
				$(window).resize(); // Refresh de los graficos
			}, 200)
		}
	}
};
Block.prototype.increaseWidth = function() {
	this.updateWidth(+1);
};
Block.prototype.decreaseWidth = function() {
	this.updateWidth(-1);
};
Block.prototype.getWidthClass = function() {
	var currentCustomClasses = this.getCustomClass().split(' ');
	for (var i = 0; i < currentCustomClasses.length; i++) {
		if (currentCustomClasses[i].indexOf('width') > -1) {
			return currentCustomClasses[i];
		}
	}
	return '';
};
Block.prototype.draw = function() {

	if (this.chart !== null && this.chart.getIndicatorTypeId(0) === INDICATOR_PRODUCTION) {
		this.setCustomClass('uk-width-large-1-1');
	}

	var blockHtml = '<div class="block ' + this.customClass + '"><form class="uk-form chart-date-handler"></form><div class="block-content"></div></div>';
	var index = this.parent.getBlockIndex(this);
	var $selector = $(this.parent.getSelector() + ' .block');

	if (index !== -1 && $selector.length > index) {
		this.element = $(blockHtml).insertBefore($selector.eq(index)); // Se agrega antes del elemento seleccionado
	}
	else {
		this.element = $(blockHtml).appendTo(this.parent.getSelector()); // Se agrega al final de los graficos
	}

	if (this.title !== '' && this.element !== null) {
		this.drawTitleBar();
	}

	if (this.chart !== null) {

		var metricDictionary = ['', 'DAY', 'WEEK', 'MONTH'];
		if ($.inArray(this.chart.metricId, [METRIC_TYPE_DAY, METRIC_TYPE_WEEK, METRIC_TYPE_MONTH]) !== -1) {
			// Se agregan los controles de seleccion de fecha
			var controlsHtml = '<div class="uk-form-icon"><i class="uk-icon-calendar"></i><input type="text" name="date-from" placeholder="Desde" class="uk-form-small date-from" readonly/></div>';
			controlsHtml += '<div class="uk-form-icon"><i class="uk-icon-calendar"></i><input type="text" name="date-to" placeholder="Hasta" class="uk-form-small date-to" readonly/><br/></div>';
			controlsHtml += '<ul class="uk-subnav uk-subnav-pill date-options" data-uk-switcher="{connect:\'#subnav-pill-content\', active: ' + (this.chart.metricId - 1) + '}">';
			controlsHtml += '<li><a href="javascript:void(\'daily\');" onclick="indicatorsWrapper.updateChartBlockMetrics(indicatorsWrapper.findParentBlockIndex(this), ' + METRIC_TYPE_DAY + ');">' + lang.line('label_daily_js') + '</a></li>';
			controlsHtml += '<li><a href="javascript:void(\'weekly\');" onclick="javascript:indicatorsWrapper.updateChartBlockMetrics(indicatorsWrapper.findParentBlockIndex(this), ' + METRIC_TYPE_WEEK + ');">' + lang.line('label_weekly_js') + '</a></li>';
			controlsHtml += '<li><a href="javascript:void(\'monthly\');" onclick="javascript:indicatorsWrapper.updateChartBlockMetrics(indicatorsWrapper.findParentBlockIndex(this), ' + METRIC_TYPE_MONTH + ');">' + lang.line('label_monthly_js') + '</a></li>';
			controlsHtml += '</ul>';
			$(this.element).children('.chart-date-handler').append(controlsHtml);
			// Agregado de funcionalidad datepicker
			var objectConfig = {weekstart: 1, format: 'DD-MM-YYYY', i18n: {months: lang.monthList, weekdays: lang.weekList}};
			var fromElement = $(this.element).find('.chart-date-handler .date-from');
			var toElement = $(this.element).find('.chart-date-handler .date-to');

			var parent = this;
			var nameDictionary = {
				avail: 'disponibilidad',
				usage: 'utilizacion',
				efective_usage: 'utilizacionefectiva',
				mttr: 'mttr',
				mtbf: 'mtbf',
				mtbs: 'mtbs'
			};
			var token = 'test';

			fromElement.change(function() {
				var fromChart = parent.getChart().chart;
				var chartAxis = fromChart.xAxis[0];
				var newDate = Date.parse(latinDateToIso($(this).val(), '-')) + 14400000; // Ajuste horario
				var queryDate = millisToDate(newDate);
				var selectedNodes = app.getCompanyTree().get_selected(true);

				chartAxis.setExtremes(newDate, chartAxis.getExtremes().max);
				fromChart.showResetZoom();
				if (newDate < fromChart.xAxis[0].getExtremes().dataMin && selectedNodes.length === 1) {
					if (confirm(lang.line('msg_dialog_query_for_calculations_js'))) {
						var ajax = new Ajax();
						ajax.reserveAccess();
						ajax.setAddress(BASE_URL + 'indicators/online_indicators');
						ajax.setParameters({
							id: 3,
							from: queryDate,
							to: millisToDate((toElement.val() !== '' ? Date.parse(latinDateToIso(toElement.val(), '-')) + 14400000 : fromChart.xAxis[0].getExtremes().dataMax)),
							token: token,
							metric: metricDictionary[parent.getChart().getMetricId()],
							name: nameDictionary[parent.title],
							metricsbyperiod: 1
						});
						ajax.setCallback(function(ajaxObject) {
							if (ajaxObject.response === null || ajaxObject.response == '') {
								app.showMessage(lang.line('msg_error_loading_server_response_js'), 'error');
								return;
							}

							var response = JSON.parse(ajaxObject.response);
							if (response.status !== true) {
								app.showMessage(lang.line('msg_error_loading_server_response_js'), 'error');
								return;
							}

							//TODO: invertir fors, probablemente el interior es mas pequeño
							for (var i = 0; i < response.list.length; i++) {
								var indicator = response.list[i];

								for (var j = 0; j < selectedNodes.length; j++) {
									if (indicator.nickName === selectedNodes[j].text) {
										parent.getChart().replaceSeriesFromWs(indicator, parent);
									}
								}
							}
							parent.getChart().draw();
						});
						ajax.setMethod('get');
						ajax.setType('html');
						ajax.query();
					}
				}
			});
			toElement.change(function() {
				var toChart = parent.getChart().chart;
				var chartAxis = toChart.xAxis[0];
				var newDate = Date.parse(latinDateToIso($(this).val(), '-'));
				var queryDate = millisToDate(newDate);
				var selectedNodes = app.getCompanyTree().get_selected(true);

				chartAxis.setExtremes(chartAxis.getExtremes().min, newDate);
				toChart.showResetZoom();
				if (newDate > toChart.xAxis[0].getExtremes().dataMax && selectedNodes.length === 1) {
					if (confirm(lang.line('msg_dialog_query_for_calculations_js'))) {
						var ajax = new Ajax();
						ajax.reserveAccess();
						ajax.setAddress(BASE_URL + 'indicators/online_indicators');
						ajax.setParameters({
							id: 3,
							from: millisToDate((fromElement.val() !== '' ? Date.parse(latinDateToIso(fromElement.val(), '-')) + 14400000 : toChart.xAxis[0].getExtremes().dataMin)),
							to: queryDate,
							token: token,
							metric: metricDictionary[parent.getChart().getMetricId()],
							name: nameDictionary[parent.title],
							metricsbyperiod: 1
						});
						ajax.setCallback(function(ajaxObject) {
							if (ajaxObject.response === null || ajaxObject.response == '') {
								app.showMessage(lang.line('msg_error_loading_server_response_js'), 'error');
								return;
							}

							var response = JSON.parse(ajaxObject.response);
							if (response.status !== true) {
								app.showMessage(lang.line('msg_error_loading_server_response_js'), 'error');
								return;
							}
							console.log('invertir fors')
							//TODO: invertir fors, probablemente el interior es mas pequeño
							for (var i = 0; i < response.list.length; i++) {
								var indicator = response.list[i];

								for (var j = 0; j < selectedNodes.length; j++) {
									if (indicator.nickName === selectedNodes[j].text) {
										console.log('indicator.nickName');
										console.log(indicator.nickName);
										parent.getChart().replaceSeriesFromWs(indicator, parent);
									}
								}
							}
							parent.getChart().draw();
						});
						ajax.setMethod('get');
						ajax.setType('html');
						ajax.query();
					}
				}
			});
			$.UIkit.datepicker(fromElement, objectConfig);
			$.UIkit.datepicker(toElement, objectConfig);
		}

		this.chart.setContainer(this.getContentElement());
		this.chart.draw();
	}

	if (this.content !== '') {
		$(this.element).children('.block-content').html(this.content);
	}
};
Block.prototype.drawTitleBar = function() {

	var titleBar = $(this.element).children('.block-title-bar');
	if (titleBar.length > 0) {
		titleBar.find('.block-title h3')
				.html(((this.icon !== '') ? '<i class="' + this.icon + '"></i> ' + ' ' : '') + lang.line('label_chart_' + this.title + '_js'));
	}
	else {
		var titleHtml = '<div class="block-title-bar">';
		titleHtml += '<div class="block-title uk-width-8-10"><h3>';
		titleHtml += ((this.icon !== '') ? '<i class="' + this.icon + '"></i> ' : '') + lang.line('label_chart_' + this.title + '_js');
		titleHtml += '</h3></div>';
		titleHtml += '<div class="block-options uk-button-group uk-width-2-10">';
		titleHtml += '	<div data-uk-dropdown>';
		titleHtml += '    <a href="#" class="uk-button uk-button-primary"><i class="uk-icon-bars"></i></a>';
		titleHtml += '		<div class="uk-dropdown uk-dropdown-flip">';
		titleHtml += '                <ul class="uk-nav uk-nav-dropdown">';
		titleHtml += '                    <li><a href="javascript:void(0)" onclick="indicatorsWrapper.printChartBlock(indicatorsWrapper.findParentBlockIndex(this));">' + lang.line('task_chart_print_js') + '</a></li>';
		titleHtml += '                    <li class="uk-nav-divider"></li>';
		titleHtml += '                    <li><a href="javascript:void(0)" onclick="indicatorsWrapper.exportChartBlock(indicatorsWrapper.findParentBlockIndex(this), \'image/png\');">' + lang.line('task_chart_export_png_js') + '</a></li>';
		titleHtml += '                    <li><a href="javascript:void(0)" onclick="indicatorsWrapper.exportChartBlock(indicatorsWrapper.findParentBlockIndex(this), \'image/jpeg\');">' + lang.line('task_chart_export_jpeg_js') + '</a></li>';
		titleHtml += '                    <li><a href="javascript:void(0)" onclick="indicatorsWrapper.exportChartBlock(indicatorsWrapper.findParentBlockIndex(this), \'application/pdf\');">' + lang.line('task_chart_export_pdf_js') + '</a></li>';
		titleHtml += '                    <li><a href="javascript:void(0)" onclick="indicatorsWrapper.exportChartBlock(indicatorsWrapper.findParentBlockIndex(this), \'image/svg+xml\');">' + lang.line('task_chart_export_svg_xml_js') + '</a></li>';
		titleHtml += '                    <li class="uk-nav-divider"></li>';
		titleHtml += '                    <li><a href="javascript:void(0)" onclick="indicatorsWrapper.decreaseWidth(indicatorsWrapper.findParentBlockIndex(this));">' + lang.line('task_chart_decrease_width_js') + '</a></li>';
		titleHtml += '                    <li><a href="javascript:void(0)" onclick="indicatorsWrapper.increaseWidth(indicatorsWrapper.findParentBlockIndex(this));">' + lang.line('task_chart_increase_width_js') + '</a></li>';
		titleHtml += '                </ul>';
		titleHtml += '          </div>';
		titleHtml += '  </div>';
		titleHtml += '  <a class="panel-option keep-panel uk-button uk-button-primary" onclick="indicatorsWrapper.removeBlock(indicatorsWrapper.findParentBlockIndex(this));">';
		titleHtml += '       <i class="icon-close"></i>';
		titleHtml += '  </a>';
		titleHtml += '	</div>';
		titleHtml += '</div>';
		$(this.element).prepend(titleHtml);
	}
};
Block.prototype.addHandlers = function() {
	var container = this.element;
	$(container).children('.block-title-bar').on("mouseover", function() {
		var sortableDragging = ($('html.uk-sortable-moving').length > 0);
		if (!sortableDragging) {
			$.UIkit.sortable('#main-panel-content').options.warp = false;
			$.UIkit.sortable('#main-panel-content').options.handleClass = 'block-title';
		}
	});
	$(container).find('.highcharts-title').on("mouseover", function() {
		var sortableDragging = ($('html.uk-sortable-moving').length > 0);
		if (!sortableDragging) {
			$.UIkit.sortable('#main-panel-content').options.warp = true;
			$.UIkit.sortable('#main-panel-content').options.handleClass = 'highcharts-title';
		}
	});
};
Block.prototype.destroy = function() {
	this.type = 0;
	if (this.chart !== null) {
		this.chart.destroy();
		this.chart = null;
	}

	if (this.element !== null) {
		$(this.element).remove();
		this.element = null;
	}

	this.parent = null;
	this.customClass = null;
	this.title = '';
	this.icon = '';
	this.options = null;
};