
function BlockFactory() {
    this.id = 0;
    this.subsystems = [];
    this.selectedIndicators = [];
    this.isInitialized = false;
    this.target = '';
}

BlockFactory.prototype.getInitialized = function() {
    return this.isInitialized;
};

BlockFactory.prototype.setTarget = function(target) {
    this.target = target;
};

BlockFactory.prototype.getTarget = function() {
    return this.target;
};

/**
 * Se extraen los datos de los nodos seleccionados y 
 * se obtienen los reportes e indicadores disponibles
 * @returns {Boolean}
 */
BlockFactory.prototype.initialize = function() {
    var parent = this;
    var selected_nodes = app.getCompanyTree().get_selected(true);
    var ajaxQueryGroup = [];

    for (var i = 0; i < selected_nodes.length; i++) {
	var node = selected_nodes[i];

	var subsystem = {
	    id: node.data.subsystem_ref,
	    name: node.text,
	    reports: []
	};

	parent.subsystems.push(subsystem);

	if (subsystem.id !== null && subsystem.id > 0) {
	    var ajax = new Ajax();

	    if (ajax.reserveAccess()) {
		ajax.setAddress(BASE_URL + 'indicators/get_reports/' + subsystem.id);
		ajax.setExtras({index: i});
		ajax.setCallback(function(a) {
		    if (a.status) {
			var reports = a.response;
			parent.subsystems[a.getExtras().index].reports = reports;
		    }
		});
		var ajaxQuery = ajax.query();

		// Se agrega la referencia del objeto (jquery) de consulta ajax a un arreglo
		ajaxQueryGroup.push(ajaxQuery);
	    }
	}
    }

    // Se espera a que todas las consultas ajax se hayan ejecutado
    $.when.apply($, ajaxQueryGroup).done(function() {

	// TODO: Verificar si el usuario ya ha seleccionado un subsystem y mostrar sus opciones (reportes) si selecciono un subsystema antes de que cargaran los reportes
	parent.isInitialized = true;
    });
};

BlockFactory.prototype.getContent = function() {

    var bfHtml = '<div class="block-factory">';
    bfHtml += '<div class="uk-grid uk-grid-small">';
    bfHtml += '<div class="uk-width-large-1-3"><h3>' + lang.line('label_select_system_js') + '</h3></div>';
    bfHtml += '<div class="uk-width-large-1-3"><h3>' + lang.line('label_select_report_js') + '</h3></div>';
    bfHtml += '<div class="uk-width-large-1-3"><h3>' + lang.line('label_select_indicator_js') + '</h3></div>';
    bfHtml += '</div>';
    bfHtml += '<div class="block-factory-selection uk-grid uk-grid-small" data-uk-grid-match="{target:\'.selection-panel\'}"><div class="uk-width-large-1-3">';
    bfHtml += '<div class="subsystems-selection selection-panel"><ul>';

    for (var i = 0; i < this.subsystems.length; i++) {
	var subsystem = this.subsystems[i];

	if (subsystem.id !== null) {
	    bfHtml += '<li><a href="javascript:void(0)" onclick="indicatorsWrapper.getBlockFactory().selectSubsystem(' + subsystem.id + ', this);">' + subsystem.name + '</a></li>';
	}
	else {
	    bfHtml += '<li class="uk-text-muted" title="' + lang.line('label_no_rmes_file_js') + '">' + subsystem.name + '</li>';
	}
    }

    bfHtml += '</ul></div></div>';
    bfHtml += '<div class="uk-width-large-1-3"><div class="reports-selection selection-panel"><ul></ul></div></div>';
    bfHtml += '<div class="uk-width-large-1-3"><div class="indicators-selection selection-panel"><ul></ul></div></div>';
    bfHtml += '</div>';
    bfHtml += '<div class="block-factory-list"><br/><h3 class="uk-panel-title">' + lang.line('label_selected_indicators_js') + '</h3><div class="selection-panel"><dl class="uk-description-list-line"><dt>' + lang.line('msg_indicator_not_yet_selected_js') + '</dt><dd>' + lang.line('msg_please_select_subsystem_js') + '</dd></dl></div></div>';

    bfHtml += '<div class="buttons uk-form">';
    bfHtml += '<span class="enable-overlap"><input type="checkbox" id="enable-overlap"/> ' + lang.line('label_chart_enable_overlap_js') + '</span>';
    bfHtml += '<button onclick="javascript:indicatorsWrapper.getBlockFactory().clearSelections()" class="generate-graph-button uk-button" disabled="disabled">' + lang.line('label_reset_js') + '</button>';
    bfHtml += '<button onclick="javascript:indicatorsWrapper.getBlockFactory().generateNewGraph()" class="generate-graph-button uk-button uk-button-primary" disabled="disabled">' + lang.line('label_create_graph_js') + '</button>';
    bfHtml += '</div>';

    return bfHtml;
};

BlockFactory.prototype.getBlock = function() {
    var block = new Block();

    // TODO: Indexar tipo block factory
    block.setType(99);
    block.setCustomClass('uk-width-large-1-2');
    block.setParent(indicatorsWrapper);
    // TODO: Internacionalizar
    block.setTitle('Crear Nuevo Bloque');
    block.setContent(this.getContent());
    return block;
};

BlockFactory.prototype.selectSubsystem = function(subsystemId, parentSelection) {
    if (!this.getInitialized()) {
	return;
    }

    if (typeof parentSelection !== 'undefined') {
	$('.subsystems-selection a.selected').removeClass('selected');
	$(parentSelection).addClass('selected');
    }

    var reportsHtml = '';
    for (var i = 0; i < this.subsystems.length; i++) {
	if (this.subsystems[i].id == subsystemId) {
	    var subsystem = this.subsystems[i];
	    if (subsystem.reports.length > 0) {
		for (var j = 0; j < subsystem.reports.length; j++) {
		    var report = subsystem.reports[j];
		    reportsHtml += '<li><a href="javascript:void(0)" onclick="javascript:indicatorsWrapper.getBlockFactory().selectReport(' + subsystemId + ', ' + report.id_report + ', this);">' + report.name + ' <span class="small-note">&raquo; ' + report.desde + ' ' + report.hasta + '</span></a></li>';
		}
	    }
	    else {
		reportsHtml = '' + lang.line('msg_selection_without_indicators_js') + '';
	    }

	    break;
	}
    }



    if (ANIMATIONS_ENABLED) {
	$('.block-factory .reports-selection ul').stop(true).fadeOut('fast').html(reportsHtml).fadeIn('fast');
	$('.block-factory .indicators-selection ul').stop(true).fadeOut('fast').html('');
    }
    else {
	$('.block-factory .reports-selection ul').html(reportsHtml).show();
	$('.block-factory .indicators-selection ul').html('').hide();
    }
};

BlockFactory.prototype.selectReport = function(subsystemId, reportId, parentSelection) {
    if (!this.getInitialized()) {
	return;
    }

    if (typeof parentSelection !== 'undefined') {
	$('.reports-selection li a.selected').removeClass('selected');
	$(parentSelection).addClass('selected');
    }

    var indicatorsHtml = '';
    for (var i = 0; i < this.subsystems.length; i++) {
	if (this.subsystems[i].id == subsystemId) {
	    var subsystem = this.subsystems[i];
	    if (subsystem.reports.length > 0) {
		for (var j = 0; j < subsystem.reports.length; j++) {
		    var report = subsystem.reports[j];

		    if (report.id_report == reportId) {

			for (var k = 0; k < report.indicators.length; k++) {
			    var indicator = report.indicators[k];
			    
			    if(indicator.id_indicator_type == 25){
				continue;
			    }

			    indicatorsHtml += '<li><a href="javascript:void(0)" onclick="javascript:indicatorsWrapper.getBlockFactory().selectIndicator(' + subsystemId + ', ' + reportId + ', ' + indicator.id_indicator_type + ', ' + indicator.id_metrica + ', \'' + indicator.it_name + '\', \'' + report.name + '\', \'' + subsystem.name + '\');">' + indicator.it_name + '</a></li>';

			}
			break;
		    }
		}
	    }

	    break;
	}
    }

    if (ANIMATIONS_ENABLED)
	$('.block-factory .indicators-selection ul').stop(true).fadeOut('fast').html(indicatorsHtml).fadeIn('fast');
    else
	$('.block-factory .indicators-selection ul').html(indicatorsHtml).show();
};

BlockFactory.prototype.selectIndicator = function(subsystemId, reportId, indicatorTypeId, metricId, indicatorName, reportName, subsystemName) {

    var newIndicator = {
	subsystem_id: subsystemId,
	report_id: reportId,
	indicator_type_id: indicatorTypeId,
	metric_id: metricId
    };

    var bfList = $('.block-factory .block-factory-list dl');

    if (this.selectedIndicators.length > 0) {
	bfList.append('<dt>' + indicatorName + '</dt><dd>' + subsystemName + ' &raquo; ' + reportName + '</dd>');
    }
    else {
	bfList.html('<dt>' + indicatorName + '</dt><dd>' + subsystemName + ' &raquo; ' + reportName + '</dd>');
    }

    this.selectedIndicators.push(newIndicator);
    $('.block-factory .generate-graph-button').prop('disabled', false);
};

BlockFactory.prototype.generateNewGraph = function() {
    indicatorsWrapper.loadIndicatorBySelection(this.selectedIndicators, $('#enable-overlap').prop('checked'));
    this.clearSelections();
};

BlockFactory.prototype.clearSelections = function() {
    this.selectedIndicators = [];

    $('.block-factory .block-factory-list dl').html('');
    $('.reports-selection ul').html('');
    $('.indicators-selection ul').html('');
    $('.block-factory .generate-graph-button').prop('disabled', true);
};