
function Chart() {
	this.chart = null;
	this.chartOptions = {};
	this.container = null;
	this.title = '';
	this.yTitle = '';
	this.metricId = 0;
	this.metricSize = 0;
	this.series = [];
	this.overlap = false;
}

Chart.prototype.isInitialized = function() {
	if (this.chart === null) {
		return false;
	}

	return true;
};

Chart.prototype.setChartOptions = function(chartOptions) {
	this.chartOptions = chartOptions;
};

Chart.prototype.getChartOptions = function() {
	return this.chartOptions;
};

Chart.prototype.setTitle = function(title, updateChart) {
	this.title = title;

//   No usar titulos por defecto
//     if (updateChart && this.isInitialized) {
//	this.chart.setTitle({text: this.title});
//    }
};

Chart.prototype.getTitle = function() {
	return this.title;
};

Chart.prototype.setContainer = function(container) {
	this.container = container;
};
Chart.prototype.getContainer = function() {
	return this.container;
};

Chart.prototype.setYTitle = function(yTitle, updateChart) {
	this.yTitle = yTitle;

	if (updateChart && this.isInitialized) {
		this.chart.yAxis[0].update({title: {text: this.yTitle}});
	}
};

Chart.prototype.setMetricId = function(metricId) {
	this.metricId = metricId;
};

Chart.prototype.getMetricId = function() {
	return this.metricId;
};

Chart.prototype.isCompatible = function(compatibilityHash) {
	return this.getCompatibilityHash() === compatibilityHash;
};

Chart.prototype.getCompatibilityHash = function() {
	return this.metricId + '-' + this.metricSize;
};

Chart.prototype.setOverlap = function(overlap) {
	this.overlap = overlap;
};

Chart.prototype.getOverlap = function() {
	return this.overlap;
};

Chart.prototype.getIndicatorTypeId = function(seriesIndex) {
	if (seriesIndex + 1 > this.getSeries().length)
		return 0; // No hay serie para seriesIndex
	return this.getSeries()[seriesIndex].extras.indicator_type_id;
};

Chart.prototype.replaceSeriesFromWs = function(indicator, parentBlock) {
	// TODO: replace indexOf!
	for (var i = 0; i < this.series.length; i++) {
		if (this.series[i].name.indexOf(indicator.nickName) > -1) {
			// Se encontro la serie a reemplazar
			var newData = [];
			for (var j = 0; j < indicator.indicatorsList.length; j++) {
				var pointDate = indicator.indicatorsList[j].desde.replace('/', '-');
				var date = new Date(pointDate);
				var point_label = date.getTime();
				newData.push([point_label, indicator.indicatorsList[j].indicatorValue * 100]);
			}
			this.series[i].data = newData;
			break;
		}
	}

};

Chart.prototype.addIndicator = function(indicator, redraw) {

	if (typeof indicator.subsystem_id === 'undefined')
		return;

	var serie = this.getFormattedSerie(indicator);

	this.series.push(serie);

	if (this.isInitialized()) {
		if (typeof redraw === 'undefined') {
			redraw = true;
		}

		this.chart.addSeries(serie, redraw);
	}
};

Chart.prototype.addIndicatorAndUpdate = function(indicator) {
	this.addIndicator(indicator, true);
};

// TODO: continue
Chart.prototype.importFormattedSerie = function(formattedSerie, redraw) {
	this.series.push(formattedSerie);

	if (typeof redraw === 'undefined') {
		redraw = true;
	}

	delete formattedSerie._colorIndex;
	delete formattedSerie._symbolIndex;

	this.chart.addSeries(formattedSerie, redraw);
};

Chart.prototype.getSeries = function() {
	return this.series;
};

/**
 * 
 * @returns {undefined}
 */
// TODO: undanify
Chart.prototype.draw = function() {
	var container = this.container;
	var title = this.title;
	var yTitle = lang.line('label_chart_' + this.yTitle + '_js');
	var series = this.series;
	var parent = this;

	var mmSeries = [{name: 'MC', data: []}, {name: 'MP', data: []}];
	var categories = [];

	var paretoDetected = [];
	var paretoCount = 0;

	var overridingType = 0;
	var xJkPlotLine = 0;
	var yJkPlotLine = 0;
	var jkCounter = 0;
	for (var i = 0; i < series.length; i++) {
		var indicatorType = series[i].extras.indicator_type_id;

		if (indicatorType === INDICATOR_MIX_MAINTENANCE_HRS || indicatorType === INDICATOR_MIX_MAINTENANCE_QTY) {
			mmSeries[0].data.push(series[i].data[0]);
			mmSeries[1].data.push(series[i].data[1]);
			categories.push(series[i].name);
		}

		if (indicatorType === INDICATOR_JACK_KNIFE) {
			overridingType = INDICATOR_JACK_KNIFE;

			for (var j = 0; j < series[i].data.length; j++) {
				var point = series[i].data[j];

				xJkPlotLine += point[0];
				yJkPlotLine += point[1];
				jkCounter++;
			}
		}

		if (indicatorType === INDICATOR_CRITICALITY || indicatorType === INDICATOR_RELIABILITY) {
			overridingType = indicatorType;
		}

		if (indicatorType === INDICATOR_PARETO || indicatorType === INDICATOR_FAILURE_MODE) {
			overridingType = indicatorType;

			// Se agregan todos los modos de falla
			var failureModes = series[i].data[0];
			series[i]['failureCount'] = 0;
			for (var key in failureModes) {
				var k = key.toUpperCase();
				if ($.inArray(k, paretoDetected) === -1) {
					paretoDetected.push(k.toUpperCase());
				}
				series[i]['failureCount'] += failureModes[key];
			}
			paretoCount += series[i]['failureCount']; // Conteo general para suma al 100%
		}
	}

	if (this.chart !== null) {
		this.chart.destroy();
	}

	Highcharts.setOptions({
		title: {
			text: ''
		},
		series: [{
				dataLabels: {
					style: {
						fontWeight: 'normal'
					}
				}
			}],
		chart: {
			events: {
				load: function() {
					//this.renderer.image(IMG_URL + '/watermark_logo.gif', 450, 15, 185, 37).add(); // x=900px to move image off chart (approx div width minus right margin) adjust as needed
				}
			}
		},
		credits: {
			enabled: false
		},
		exporting: {
			enabled: false
		}
	});

	var chartOptions = {
		chart: {
			renderTo: container[0],
			type: 'line',
			height: 390,
			zoomType: 'xy',
			plotBorderColor: '#999',
			plotBorderWidth: 1
		},
		title: {
			text: '', //'',//lang.line('label_chart_' + title + '_js'),
			x: -20 //center
		},
		subtitle: {
			text: '', //'Reliability & Maintenance Engineering System',
			x: -20
		},
		xAxis: {
			//type: 'category',
			type: 'datetime',
			//dateTimeLabelFormats: {// don't display the dummy year
			//	month: '%e. %b',
			//year: '%b'
			//},
			labels: {
				rotation: -60, // para que el texto del eje se vea ladeado
				formatter: function() {
					return  Highcharts.dateFormat((!parent.overlap) ? '%d-%b-%y' : '%d-%b',
							new Date(this.value));
				}
			}
		},
		yAxis: {
			title: {
				text: yTitle + ' (%)'
			},
			labels: {
				formatter: function() {
					return this.value + '%';
				}
			}
		},
		legend: {
			//layout: 'vertical',
			borderWidth: 0,
			maxHeight: 60
		},
		plotOptions: {
			spline: {
				marker: {
					enable: false
				}
			}
		},
		tooltip: {
			formatter: function() {
				return  '<b>' + this.series.name + ': ' + this.y + '</b><br/>' +
						Highcharts.dateFormat((!parent.overlap) ? '%d-%b-%Y' : '%d-%b',
								new Date(this.x));
			}
		},
		series: series
	};



	if (categories.length > 0) {
		chartOptions.chart.type = 'column';

		chartOptions.xAxis = {
			categories: categories
		};

		chartOptions.plotOptions = {
			column: {
				stacking: 'percent',
				dataLabels: {
					enabled: true,
					color: (Highcharts.theme && Highcharts.theme.dataLabelsColor) || 'white',
					style: {
						textShadow: '0 0 3px black, 0 0 3px black'
					}
				}

			}
		};

		chartOptions.tooltip = {
			pointFormat: '<span style="color:{series.color}">{series.name}</span>: <b>{point.y}</b> ({point.percentage:.0f}%)<br/>',
			shared: true
		};

		chartOptions.series = mmSeries;
	}

	if (overridingType === INDICATOR_JACK_KNIFE) {
		if (jkCounter > 0) {
			xJkPlotLine = xJkPlotLine / jkCounter;
			yJkPlotLine = yJkPlotLine / jkCounter;
		}
		chartOptions.xAxis = {
			gridLineWidth: 0,
			minorGridLineWidth: 0,
			title: {
				text: 'ln(n)'
			},
			plotLines: [{
					value: xJkPlotLine,
					color: '#ff0000',
					width: 1,
					zIndex: 4,
					label: {text: xJkPlotLine.toFixed(2),
						style: {
							color: '#ccc',
							fontSize: '10px'
						}}
				}]
		},
		chartOptions.yAxis = {
			gridLineWidth: 0,
			minorGridLineWidth: 0,
			title: {
				text: 'ln(MTTR)'
			},
			labels: {
				formatter: function() {
					return this.value;
				}
			},
			plotLines: [{
					value: yJkPlotLine,
					color: '#ff0000',
					width: 1,
					zIndex: 4,
					label: {text: yJkPlotLine.toFixed(2),
						style: {
							color: '#ccc',
							fontSize: '10px'
						}}
				}]
		};

		chartOptions.tooltip = {
			headerFormat: '<b>{series.name}</b><br/>',
			pointFormat: '({point.x}, {point.y})'
		};

		chartOptions.labels = {
			items: [{
					html: 'Agudo',
					style: {
						left: '25px',
						top: '25px'
					}
				}, {
					html: 'Agudo Crónico',
					style: {
						left: '475px',
						top: '25px'
					}
				}, {
					html: 'Bajo Control',
					style: {
						left: '25px',
						top: '250px'
					}
				}, {
					html: 'Crónico',
					style: {
						left: '475px',
						top: '250px'
					}
				}]};

	}

	if (overridingType === INDICATOR_CRITICALITY) {
		chartOptions.chart.type = 'scatter';
		chartOptions.xAxis = {
			type: 'linear',
			title: {
				text: 'Indisponibilidad'
			},
			labels: {
				formatter: function() {
					return this.value + '%';
				}
			}
		};

		chartOptions.yAxis = {
			title: {
				text: 'Consecuencia'
			},
			labels: {
				formatter: function() {
					return this.value + '%';
				}
			}
		};
	}

	if (overridingType === INDICATOR_PARETO) {

		var paretoSeries = [];
		var paretoCategories = [];
		var paretoAccumulatedSeries = [];

		paretoDetected.sort();
		series.sort(function(a, b) {
			return b.failureCount - a.failureCount;
		});

		// Se generan las barras pareto
		for (var i = 0; i < paretoDetected.length; i++) {
			var failureMode = {name: paretoDetected[i], data: []};

			for (var j = 0; j < series.length; j++) {
				var failureModes = series[j].data[0];
				var failureFound = false;

				paretoCategories.push(series[j].name);

				for (var k in failureModes) {
					if (k.toUpperCase() === failureMode.name) {
						failureMode.data.push(roundFloat(failureModes[k]));
						failureFound = true;
						break;
					}
				}

				if (!failureFound) {
					failureMode.data.push(null);
				}
			}

			paretoSeries.push(failureMode);
		}

		// Se genera el indicador de acumulado
		var accumulated = 0;
		for (var i = 0; i < series.length; i++) {
			var serie = series[i];
			accumulated += serie.failureCount * 100 / paretoCount;
			paretoAccumulatedSeries.push(roundFloat(accumulated));
		}
		var accumulatedSerie = {
			type: 'line',
			name: '% Acumulado',
			color: '#333',
			yAxis: 1,
			data: paretoAccumulatedSeries
		};

		chartOptions.chart.type = 'column';

		chartOptions.xAxis = {
			title: {
				text: 'Equipos / Configuraciones'
			},
			categories: paretoCategories,
			labels: {
				rotation: -60, // para que el texto del eje se vea ladeado
				formatter: function() {
					return truncateString(this.value, 12, true);
				},
				useHTML: true
			}
		};


		chartOptions.yAxis = [
			{
				title: {
					text: 'Horas'
				},
				labels: {
					formatter: function() {
						return this.value;
					}
				}
			},
			{
				title: {text: ''},
				max: 100,
				lineColor: '#333',
				opposite: true,
				labels: {
					formatter: function() {
						return this.value + '%';
					}
				}
			}

		];

		chartOptions.plotOptions = {
			column: {
				stacking: 'normal',
				dataLabels: {
					enabled: true,
					color: (Highcharts.theme && Highcharts.theme.dataLabelsColor) || 'white',
					style: {
						textShadow: '0 0 3px black, 0 0 3px black'
					},
					formatter: function() {
						if (this.y != 0) {
							return this.y;
						}
					}
				}

			}
		};

		chartOptions.tooltip = {
			pointFormat: '<span style="color:{series.color}">{series.name}</span>: <b>{point.y}</b> ({point.percentage:.0f}%)<br/>',
			shared: true
		};
		chartOptions.series = paretoSeries;
		chartOptions.series.push(accumulatedSerie); // Porcentaje
	}

	if (overridingType === INDICATOR_FAILURE_MODE) {

		var paretoSeries = [];
		var paretoCategories = [];
		var paretoAccumulatedSeries = [];

		paretoDetected.sort();
		series.sort(function(a, b) {
			return b.failureCount - a.failureCount;
		});

		// Se generan las barras pareto
		for (var i = 0; i < paretoDetected.length; i++) {
			var failureMode = [paretoDetected[i], 0];

			for (var j = 0; j < series.length; j++) {
				var failureModes = series[j].data[0];
				var failureFound = false;

				for (var k in failureModes) {
					if (k.toUpperCase() === failureMode[0]) {
						failureMode[1] += failureModes[k];
						failureFound = true;
						break;
					}
				}

//		if (!failureFound) {
//		    failureMode.data.push(null);
//		}
			}

			paretoSeries.push(failureMode);
		}

		paretoSeries.sort(function(a, b) {
			return b[1] - a[1];
		});

		// Se genera el indicador de acumulado
		var accumulated = 0;
		var relativeAccum = [];

		for (var i = 0; i < paretoSeries.length; i++) {
			var serie = paretoSeries[i];
			accumulated += serie[1] * 100 / paretoCount;
			relativeAccum.push(roundFloat(serie[1] * 100 / paretoCount));
			paretoAccumulatedSeries.push(roundFloat(accumulated));

			paretoCategories.push(ucFirst(serie[0].toLowerCase()));
		}

		var accumulatedSerie = {
			type: 'line',
			name: 'Impacto Acumulado (%)',
			color: '#009900',
			yAxis: 1,
			data: paretoAccumulatedSeries
		};
		var individualImpactSerie = {
			type: 'line',
			name: 'Impacto Individual (%)',
			color: '#990000',
			yAxis: 1,
			data: relativeAccum
		};

		chartOptions.chart.type = 'column';

		chartOptions.xAxis = {
			title: {
				text: 'Descripción'
			},
			categories: paretoCategories,
			labels: {
				rotation: -60, // para que el texto del eje se vea ladeado
				formatter: function() {
					return truncateString(this.value, 12, true);
				},
				useHTML: true
			}
		};


		chartOptions.yAxis = [
			{
				title: {
					text: 'Número'
				},
				labels: {
					formatter: function() {
						return this.value;
					}
				}
			},
			{
				title: {text: ''},
				max: 100,
				min: 0,
				lineColor: '#333',
				opposite: true,
				labels: {
					formatter: function() {
						return this.value + '%';
					}
				}
			}

		];

		chartOptions.plotOptions = {
			column: {
				dataLabels: {
					enabled: true,
					color: (Highcharts.theme && Highcharts.theme.dataLabelsColor) || 'white',
					style: {
						textShadow: '0 0 3px black, 0 0 3px black'
					},
					formatter: function() {
						if (this.y != 0) {
							return this.y;
						}
					}
				}

			}
		};

		chartOptions.tooltip = {
			pointFormat: '<span style="color:{series.color}">{series.name}</span>: <b>{point.y}</b> ({point.percentage:.0f}%)<br/>',
			shared: true
		};

		chartOptions.series = [{name: 'Modo de falla', data: paretoSeries}];
		chartOptions.series.push(accumulatedSerie); // Porcentaje
		chartOptions.series.push(individualImpactSerie);
	}

	if (overridingType === INDICATOR_RELIABILITY) {
		series = [series[0]];
		var tempCategories = [];
		var realSerie = [];
		var first = 0;
		var firstValue = 0;
		var last = 0;
		var lastValue = 0;

		for (var i = 0; i < series.length; i++) {

			var seriesData = series[i].data;
			seriesData.sort(function(a, b) {
				return a[0] - b[0];
			})


			for (var j = 0; j < seriesData.length; j++) {
				tempCategories.push(seriesData[j][0]);
				realSerie.push(seriesData[j][1]);
				if (j === 0) {
					first = seriesData[j][0];
					firstValue = seriesData[j][1];
				}
				if (j === seriesData.length - 1) {
					last = j;
					lastValue = seriesData[j][0];
				}
			}

			series[i].data = realSerie;
		}

		chartOptions.xAxis = {
			type: 'category',
			labels: {
				rotation: -60 // para que el texto del eje se vea ladeado
			},
			categories: tempCategories,
			plotLines: [{
					value: first,
					color: '#ff0000',
					width: 1,
					zIndex: 4,
					label: {text: 'Puesta en Marcha}: ' + firstValue,
						style: {
							color: '#ccc',
							fontSize: '10px'
						}}
				}, {
					value: last,
					color: '#ff0000',
					width: 1,
					zIndex: 4,
					label: {text: 'Tiempo Corrido: ' + lastValue,
						style: {
							color: '#ccc',
							fontSize: '10px'
						}}
				}]
		};

		chartOptions.yAxis.min = 0;
		chartOptions.plotOptions = {
			series: {
				marker: {
					enabled: false
				}
			}
		}
		chartOptions.tooltip = {
			headerFormat: '<b>{series.name}</b><br/>',
			pointFormat: '({point.x}, {point.y})'
		};
		chartOptions.series = series;
	}

	//chartOptions.series.dataLabels.style.fontWeight = 'normal';

	this.chart = new Highcharts.Chart(chartOptions);
};

Chart.prototype.getFormattedSerie = function(indicator) {
	var points = [];
	for (var i = 0; i < indicator.indicator_data.length; i++) {

		var point_data = indicator.indicator_data[i];

		// Se obtiene el label a utilizar (Eje X)
		var point_label = null;

		switch (indicator.metric_id) {
			case 1:
				// TODO: rethink date replace
				if (this.overlap === true) {
					var temp_from = point_data.from.split('/');
					temp_from[2] = '2000';
					point_data.from = temp_from.join('/');
				}
				var date = new Date(latinDateToIso(point_data.from));
				point_label = date.getTime();
				break;
			case 2:
				// TODO: rethink date replace
				if (this.overlap === true) {
					var temp_from = point_data.from.split('/');
					temp_from[2] = '2000';
					point_data.from = temp_from.join('/');
				}
				var date = new Date(latinDateToIso(point_data.from));
				point_label = date.getTime();
				break;
			case 3:
				// TODO: rethink date replace
				if (this.overlap === true) {
					var temp_from = point_data.from.split('/');
					temp_from[2] = '2000';
					point_data.from = temp_from.join('/');
				}

				var date = new Date(latinDateToIso(point_data.from));
				point_label = point_label = date.getTime();
				break;
			case 4:
				var date = new Date(latinDateToIso(point_data.from));
				point_label = getMonthName(date) + '\'' + date.getFullYear().toString().substr(2, 2);
				break;
			default:

				if (indicator.indicator_type === INDICATOR_MIX_MAINTENANCE_HRS || indicator.indicator_type === INDICATOR_MIX_MAINTENANCE_QTY) {
					point_label = indicator.name;
					break;
				}

				if (indicator.indicator_type === INDICATOR_PARETO) {
					point_label = indicator.name;
					break;
				}

				if (indicator.indicator_type === INDICATOR_FAILURE_MODE) {
					point_label = null;
					break;
				}

				if (indicator.indicator_type === INDICATOR_RELIABILITY) {
					point_label = null;
					break;
				}

				var date = new Date(latinDateToIso(point_data.from));
				//point_label = ('0' + date.getDate()).slice(-2) + '-' + getMonthName(date) + '-' + date.getFullYear().toString().substr(2, 2);
				point_label = date.getTime();
		}


		// Se crea el punto con el label (Eje X) y el valor (Eje Y)
		switch (indicator.indicator_type) {
			case 7:
				points.push([point_label, point_data.value]);
				break;
			case 8:
				points.push([point_label, point_data.value]);
				break;
			case 9:
				points.push([point_label, point_data.value]);
				break;
			case INDICATOR_JACK_KNIFE:
				points.push([roundFloat(point_data.value), roundFloat(point_data.from)]);
				break;
			case INDICATOR_MIX_MAINTENANCE_HRS:
				points.push(roundFloat(point_data.from));
				points.push(roundFloat(point_data.to));
				break;
			case INDICATOR_MIX_MAINTENANCE_QTY:
				points.push(roundFloat(point_data.from));
				points.push(roundFloat(point_data.to));
				break;
			case INDICATOR_CRITICALITY:
				points.push([roundFloat(point_data.from * 100), roundFloat(point_data.value * 100)]);
				break;
			case INDICATOR_PARETO:
				points.push(JSON.parse(point_data.from));
				break;
			case INDICATOR_FAILURE_MODE:
				points.push(JSON.parse(point_data.from));
				break;
			case INDICATOR_RELIABILITY:
				points.push([point_data.from, point_data.to * 100]);
				break;
			default:
				points.push([point_label, point_data.value * 100]);
		}
	}

	var seriesName = indicator.name + ' (' + indicator.alias + ')';

	// Extras son utilizados para guardar preferencias
	var seriesExtras = {
		indicator_type_id: indicator.indicator_type,
		report_id: indicator.report_id,
		metric_id: indicator.metric_id,
		subsystem_id: indicator.subsystem_id
	};

	return {name: seriesName, data: points, extras: seriesExtras};
};

/**
 * Actualiza todos los datos en base a la metrica entregada.
 * Los datos son recuperados nuevamente desde desde la db.
 * @param {int} metric
 * @returns {Boolean} updated
 */
Chart.prototype.updateMetrics = function(metric, blockIndex) {

	if (this.metricId === metric)
		return;

	// Se prepara el objeto para consultar
	var preference = {block: null, indicators: []};
	var response = {originalMetric: 0, status: false};

	for (var i = 0; i < this.series.length; i++) {
		var serie = this.series[i];
		preference.indicators.push(serie.extras);
	}

	// Se realiza la consulta
	var parent = this;
	var queryGroup = [];
	var groupedIndicators = {};

	for (var i = 0; i < preference.indicators.length; i++) {
		var indicator = preference.indicators[i];

		// Query ajax granular para aprovechar cache de browser
		var ajax = new Ajax();
		ajax.reserveAccess();
		ajax.setAddress('indicators/get_indicator_by_metric');
		ajax.setMethod('post');
		response.originalMetric = indicator.metric_id;

		ajax.setParameters({
			indicator_type_id: indicator.indicator_type_id,
			metric_id: metric,
			report_id: indicator.report_id,
			subsystem_id: indicator.subsystem_id
		});

		ajax.setCallback(function(ajaxObject) {
			for (var k = 0, size = ajaxObject.response.length; k < size; k++) {
				var indicator = ajaxObject.response[k];

				// La preferencia indica la agrupacion del block
				var compatibilityHash = parent.getCompatibilityHash();
				if (typeof groupedIndicators[compatibilityHash] === 'undefined') {
					groupedIndicators[compatibilityHash] = [];
				}

				// Se agrega el indicador en el grupo correspondiente segun el hash de compatibilidad
				groupedIndicators[compatibilityHash].push(indicator);
			}
		});

		var queryRef = ajax.query();
		queryGroup.push(queryRef);
	}

	$.when.apply($, queryGroup).done(function() {
		if (getObjectSize(groupedIndicators) === 0) {
			app.showMessage(lang.line('msg_selection_without_indicators_js'), 'warning');
			status = false;
		}
		else {
			for (var chartGroupKey in groupedIndicators) {

				var chartGroup = [];
				for (i = 0; i < groupedIndicators[chartGroupKey].length; i++) {
					chartGroup.push(groupedIndicators[chartGroupKey][i]);
				}

				if (chartGroup.length > 0) {
					// Se destruye el chart
					if (parent.chart !== null) {
						parent.removeAllSeries();
					}

					// Se preparan las series
					for (var i = 0; i < chartGroup.length; i++) {
						var indicator = chartGroup[i];

						// Solo se dibuja cuando se ha agregado el ultimo indicador
						parent.addIndicator(indicator, chartGroup.length - 1 === i);
					}

					response.status = true;
					parent.metricId = metric;
				}
			}
		}

		if (response.status) {
			app.showMessage(lang.line('msg_chart_updated_js'), 'success');
		}
		else {
			var block = indicatorsWrapper.getBlock(blockIndex);
			$(block.getElement()).find('.date-options .uk-active').removeClass('uk-active');
			var dateOptions = $(block.getElement()).find('.date-options li');
			$(dateOptions[response.originalMetric - 1]).addClass('uk-active');
		}
	});
};


/**
 * Remueve todas las series del chart
 * @returns void
 */
Chart.prototype.removeAllSeries = function() {
	while (this.chart.series.length > 0)
		this.chart.series[0].remove(true);

	// TODO: Check-> Se reinician los colores e iconos
	this.chart.colorCounter = 0;
	this.chart.symbolCounter = 0;

	this.series = [];
};

/**
 * Se remueve la serie segun subsystem, no se realiza redraw
 * @param {type} subsystemId
 * @returns {Boolean}
 */
Chart.prototype.removeSerie = function(subsystemId) {

	for (var i = 0; i < this.series.length; i++) {
		var serie = this.series[0];

		if (serie.subsystemId === subsystemId) {
			this.series.splice(i, 1);
			return true;
		}
	}

	return false;
};

Chart.prototype.destroy = function() {
	if (this.chart !== null) {
		this.chart.destroy();
		this.chart = null;
	}

	this.chartOptions = {};
	this.container = null; // Container debe ser removido de forma externa
	this.title = '';
	this.yTitle = '';
	this.metricId = 0;
	this.metricSize = 0;
	this.series = [];
	this.overlap = false;
};