
function FailureModeChart() {
	this.chart = null;
	this.container = null;
	this.title = '';
	this.yTitle = '';
	this.metricId = 0;
	this.metricSize = 0;
	this.series = [];
}

/**
 * 
 * @returns {undefined}
 */
// TODO: undanify
FailureModeChart.prototype.draw = function() {
	var container = this.container;
	var title = this.title;
	var yTitle = this.yTitle;
	var series = this.series;

	var paretoDetected = [];
	var paretoCount = 0;

	var overridingType = 0;

	for (var i = 0; i < series.length; i++) {
		var indicatorType = series[i].extras.indicator_type_id;

		if (indicatorType === INDICATOR_FAILURE_MODE) {
			overridingType = indicatorType;

			// Se agregan todos los modos de falla
			var failureModes = series[i].data[0];
			series[i]['failureCount'] = 0;
			for (var key in failureModes) {
				var k = key.toUpperCase();
				if ($.inArray(k, paretoDetected) === -1) {
					paretoDetected.push(k.toUpperCase());
				}

				series[i]['failureCount'] += failureModes[key].q;
			}
			paretoCount += series[i]['failureCount']; // Conteo general para suma al 100%
		}
	}

	if (this.chart !== null) {
		this.chart.destroy();
	}

	Highcharts.setOptions({
		series: [{
				dataLabels: {
					style: {
						fontWeight: 'normal'
					}
				}
			}],
		chart: {
			events: {
				load: function() {
					this.renderer.image(IMG_URL + '/watermark_logo.gif', 450, 15, 185, 37).add(); // x=900px to move image off chart (approx div width minus right margin) adjust as needed
				}
			}
		}});

	var chartOptions = {
		chart: {
			renderTo: container[0],
			type: 'line',
			height: 450,
			zoomType: 'xy'
		},
		title: {
			text: title,
			x: -20 //center
		},
		subtitle: {
			text: '', //'Reliability & Maintenance Engineering System',
			x: -20
		},
		xAxis: {
			//type: 'category',
			type: 'datetime',
			//dateTimeLabelFormats: {// don't display the dummy year
			//	month: '%e. %b',
			//year: '%b'
			//},
			labels: {
				rotation: -60 // para que el texto del eje se vea ladeado
			}
//	    //reversed: false,
//	    gridLineWidth: 1,
//	    maxPadding: 0.05,
//	    tickInterval: 100
			//showLastLabel: true
		},
		yAxis: {
			title: {
				text: yTitle + ' (%)'
			},
			labels: {
				formatter: function() {
					return this.value + '%';
				}
			}
		},
		legend: {
			//layout: 'vertical',
			borderWidth: 0,
			maxHeight: 60
		},
		tooltip: {
			headerFormat: '<b>{series.name}</b><br/>',
			pointFormat: '{point.x} : {point.y}%'
		},
		plotOptions: {
			spline: {
				marker: {
					enable: false
				}
			}
		},
		series: series
	};

	if (overridingType === INDICATOR_FAILURE_MODE) {

		var paretoSeries = [];
		var paretoCategories = [];
		var paretoAccumulatedSeries = [];

		paretoDetected.sort();
		series.sort(function(a, b) {
			return b.failureCount - a.failureCount;
		});

		// Se generan las barras pareto
		for (var i = 0; i < paretoDetected.length; i++) {
			var failureMode = [paretoDetected[i], 0];

			for (var j = 0; j < series.length; j++) {
				var failureModes = series[j].data[0];
				var failureFound = false;

				for (var k in failureModes) {
					if (k.toUpperCase() === failureMode[0]) {
						failureMode[1] += failureModes[k].h;
						failureFound = true;
						break;
					}
				}

//		if (!failureFound) {
//		    failureMode.data.push(null);
//		}
			}

			paretoSeries.push(failureMode);
		}

		paretoSeries.sort(function(a, b) {
			return b[1] - a[1];
		});

		// Se genera el indicador de acumulado
		var accumulated = 0;
		var relativeAccum = [];

		for (var i = 0; i < paretoSeries.length; i++) {
			var serie = paretoSeries[i];
			accumulated += serie[1] * 100 / paretoCount;
			relativeAccum.push(roundFloat(serie[1] * 100 / paretoCount));
			paretoAccumulatedSeries.push(roundFloat(accumulated));

			paretoCategories.push(ucFirst(serie[0].toLowerCase()));
		}

		var accumulatedSerie = {
			type: 'line',
			name: 'Impacto Acumulado (%)',
			color: '#009900',
			yAxis: 1,
			data: paretoAccumulatedSeries
		};
		var individualImpactSerie = {
			type: 'line',
			name: 'Impacto Individual (%)',
			color: '#990000',
			yAxis: 1,
			data: relativeAccum
		};

		chartOptions.chart.type = 'column';

		chartOptions.xAxis = {
			title: {
				text: 'Descripción'
			},
			categories: paretoCategories,
			labels: {
				rotation: -60, // para que el texto del eje se vea ladeado
				formatter: function() {
					return truncateString(this.value, 12, true);
				},
				useHTML: true
			}
		};


		chartOptions.yAxis = [
			{
				title: {
					text: 'Número'
				},
				labels: {
					formatter: function() {
						return this.value;
					}
				}
			},
			{
				title: {text: ''},
				max: 100,
				min: 0,
				lineColor: '#333',
				opposite: true,
				labels: {
					formatter: function() {
						return this.value + '%';
					}
				}
			}

		];

		chartOptions.plotOptions = {
			column: {
				dataLabels: {
					enabled: true,
					color: '333',
					formatter: function() {
						if (this.y != 0) {
							return this.y;
						}
					}
				}

			}
		};

		chartOptions.tooltip = {
			pointFormat: '<span style="color:{series.color}">{series.name}</span>: <b>{point.y}</b><br/>',
			shared: true
		};

		chartOptions.series = [{name: 'Modo de falla', data: paretoSeries}];
		chartOptions.series.push(accumulatedSerie); // Porcentaje
		chartOptions.series.push(individualImpactSerie);
	}

	this.chart = new Highcharts.Chart(chartOptions);
};

FailureModeChart.prototype.getFormattedSerie = function(indicator) {
	var points = [];
	for (var i = 0; i < indicator.indicator_data.length; i++) {

		var point_data = indicator.indicator_data[i];


		// Se obtiene el label a utilizar (Eje X)
		var point_label = null;

		switch (indicator.metric_id) {
			case 1:
				var date = new Date(point_data.from);
				//point_label = ('0' + date.getDate()).slice(-2) + '-' + getMonthName(date) + '-' + date.getFullYear().toString().substr(2, 2);
				point_label = date.getTime();
				break;
			case 2:
				var date = new Date(point_data.from);
				//point_label = ('0' + date.getDate()).slice(-2) + '-' + getMonthName(date) + '-' + date.getFullYear().toString().substr(2, 2);
				point_label = date.getTime();
				break;
			case 3:
				var date = new Date(point_data.from);
				point_label = getMonthName(date) + '\'' + date.getFullYear().toString().substr(2, 2);
				break;
			case 10:
				point_label = point_data.value;
				break;
			case 11:
				point_label = point_data.value;
				break;
			case 12:
				point_label = point_data.value;
				break;
			default:

				if (indicator.indicator_type === INDICATOR_MIX_MAINTENANCE_HRS || indicator.indicator_type === INDICATOR_MIX_MAINTENANCE_QTY) {
					point_label = indicator.name;
					break;
				}

				if (indicator.indicator_type === INDICATOR_PARETO) {
					point_label = indicator.name;
					break;
				}

				if (indicator.indicator_type === INDICATOR_FAILURE_MODE) {
					point_label = null;
					break;
				}

				if (indicator.indicator_type === INDICATOR_RELIABILITY) {
					point_label = null;
					break;
				}

				var date = new Date(point_data.from);
				var date = new Date(point_data.from);
				//point_label = ('0' + date.getDate()).slice(-2) + '-' + getMonthName(date) + '-' + date.getFullYear().toString().substr(2, 2);
				point_label = date.getTime();
		}


		// Se crea el punto con el label (Eje X) y el valor (Eje Y)
		switch (indicator.indicator_type) {
			case 7:
				points.push([point_label, point_data.value]);
				break;
			case 8:
				points.push([point_label, point_data.value]);
				break;
			case 9:
				points.push([point_label, point_data.value]);
				break;
			case INDICATOR_JACK_KNIFE:
				points.push([roundFloat(point_data.value), roundFloat(point_data.from)]);
				break;
			case INDICATOR_MIX_MAINTENANCE_HRS:
				points.push(roundFloat(point_data.from));
				points.push(roundFloat(point_data.to));
				break;
			case INDICATOR_MIX_MAINTENANCE_QTY:
				points.push(roundFloat(point_data.from));
				points.push(roundFloat(point_data.to));
				break;
			case INDICATOR_CRITICALITY:
				points.push([roundFloat(point_data.from * 100), roundFloat(point_data.value * 100)]);
				break;
			case INDICATOR_PARETO:
				points.push(JSON.parse(point_data.from));
				break;
			case INDICATOR_FAILURE_MODE:
				points.push(JSON.parse(point_data.value));
				break;
			case INDICATOR_RELIABILITY:
				points.push([point_data.from, point_data.to * 100]);
				break;
			default:
				points.push([point_label, point_data.value * 100]);
		}
	}

	var seriesName = indicator.name + ' (' + indicator.alias + ')';

	// Extras son utilizados para guardar preferencias
	var seriesExtras = {
		indicator_type_id: indicator.indicator_type,
		report_id: indicator.report_id,
		metric_id: indicator.metric_id,
		subsystem_id: indicator.subsystem_id
	};

	return {name: seriesName, data: points, extras: seriesExtras};
};