
function GeneralInfoChart() {
	this.chart = null;
	this.container = null;
	this.title = '';
	this.yTitle = '';
	this.metricId = 0;
	this.metricSize = 0;
	this.series = [];
	this.drilldownSeries = [];
	this.currentOperation = 0;
	this.currentArea = 0;
}

GeneralInfoChart.prototype.setSeries = function(series) {
	this.series = series;
};

GeneralInfoChart.prototype.setDrilldownSeries = function(drilldownSeries) {
	this.drilldownSeries = drilldownSeries;
};

/**
 * 
 * @returns {undefined}
 */
GeneralInfoChart.prototype.draw = function() {
	var container = this.container;
	var title = this.title;
	var yTitle = this.yTitle;
	var series = this.series;
	var drilldownSeries = this.drilldownSeries;
	var $this = this;

	if (this.chart !== null) {
		this.chart.destroy();
	}

	// Create the chart
	this.chart = new Highcharts.Chart({
		chart: {
			renderTo: container[0],
			type: 'column',
			borderWidth: 0,
			backgroundColor: 'transparent'
		},
		credits: {
			enabled: false
		},
		title: {
			text: title
		},
		yAxis: {
			title: {text: yTitle},
			max: 100,
			tickInterval: 25
		},
		xAxis: {
			type: 'category',
			title: {text: lang.line('label_main_operations_js')}
		},
		legend: {
			enabled: false
		},
		tooltip: {
			formatter: function() {
				var name = (typeof this.point.name !== 'undefined') ? this.point.name : this.x;
				return '<b>' + name + '</b>: ' + this.y + '%';
			}
		},
		plotOptions: {
			column: {
				cursor: 'pointer',
				point: {
					events: {
						click: function() {
							var drilldown = this.drilldown;
							if (typeof drilldown !== 'undefined') { // drill down de segundo nivel 
								app.updateDashboard(this.id_tree);
								$this.currentOperation = this.id_tree;

								$this.setChart(drilldown.categories, drilldown.categories, drilldown.data, drilldown.area_ids);
								$this.drawBackButton('operations');
							} else { // FPN
								$this.drawBackButton('areas');
								var companyTree = app.getCompanyTree();
								var operation = companyTree.get_node($this.currentOperation);

								var area_node_id = this.area_id;
								$this.currentArea = area_node_id;
								$('.operations-general-chart-fpn').detach();
								$('.operations-general-chart').hide().after('<div class="operations-general-chart-fpn"></div>');
								var fnp = app.drawFnpGraph(area_node_id, operation, '.operations-general-chart-fpn', INDICATOR_FNP);

								fnp.chart.setTitle({text: fnp.getTitle()});
								app.updateDashboard(area_node_id);
								$(window).resize();
							}
						}
					}
				},
				dataLabels: {
					enabled: true,
					rotation: -90,
					align: 'right',
					color: '#ffffff',
					shadow: false,
					y: 10, // 10 pixels down from the top
					style: {
						fontWeight: 'bold'
					},
					formatter: function() {
						return this.y + '%'; // +'%';
					}
				}
			}
		},
		series: [series],
		drilldown: {
			activeAxisLabelStyle: {
				color: '#0d233a',
				fontWeight: 'normal',
				textDecoration: 'none'
			},
			activeDataLabelStyle: {
				color: '#ffffff',
				fontWeight: 'bold',
				textDecoration: 'none'
			},
			series: drilldownSeries
		}
	});
};

GeneralInfoChart.prototype.setChart = function(name, categories, data, areaIds) {

	var chart = this.chart;

	chart.xAxis[0].setCategories(categories);
	while (chart.series.length > 0) {
		chart.series[0].remove(true);
	}
	var serie = [];
	for (var i = 0; i < data.length; i++) {

		serie.push({
			name: name[i],
			y: data[i],
			area_id: areaIds[i]
		});
	}

	//this.chart.series.setData(serie);
	chart.addSeries({
		data: serie,
		colorByPoint: true
	}, false);
	this.chart.redraw();

	$.each(chart.series[0].data, function(i, point) {

		if (point.y < 50) {
			point.dataLabel.attr({y: 10, align: 'right', shadow: false});
		}
	});
};

GeneralInfoChart.prototype.drawBackButton = function(show) {

	var $this = this;
	var container = this.container;
	$(container).parent().children('.back-button').detach();

	var buttonHtml = '<div class="back-button uk-button uk-button-small" title="' + lang.line('task_button_back_js') + '" data-uk-tooltip="{pos:\'top\'}">';
	buttonHtml += '<i class="uk-icon-angle-left uk-icon-small"></i>';
	buttonHtml += '</div>';

	var $button = $(container).parent().prepend(buttonHtml).off('click');


	switch (show) {
		case 'operations': // Si se solicita desplegar operaciones, entonces se resetea el dashboard

			$button.on('click', function() {
				$this.draw();
				$(container).parent().children('.back-button').detach();
				$this.currentOperation = 0;
				$this.currentArea = 0;
				app.updateDashboard(null);
				$('.uk-tooltip').hide();
			});
			break;
		case 'areas': // Si son areas, se muestra el ultimo grafico de barras desplegado

			$button.on('click', function() {
				$('.operations-general-chart-fpn').detach();
				$('.operations-general-chart').show();
				$this.drawBackButton('operations');
				app.updateDashboard($this.currentArea);
				$('.uk-tooltip').hide();
				$('.spark-row').removeClass('active');
			});
			break;
	}
};

GeneralInfoChart.prototype.destroy = function() {
	if (this.chart !== null) {
		this.chart.destroy();
		this.chart = null;
	}

	this.container = null; // Container debe ser removido de forma externa
	this.title = '';
	this.yTitle = '';
	this.metricId = 0;
	this.metricSize = 0;
	this.series = [];
	this.drilldownSeries = [];
	this.overlap = false;
};
