
function GaugeChart() {
	this.chart = null;
	this.container = null;
	this.title = '';
	this.yTitle = '';
	this.metricId = 0;
	this.metricSize = 0;
	this.series = [];
	this.drilldownSeries = [];
}

GaugeChart.prototype.setSeries = function(series) {
	this.series = series;
};

GaugeChart.prototype.setDrilldownSeries = function(drilldownSeries) {
	this.drilldownSeries = drilldownSeries;
};

/**
 * 
 * @returns {undefined}
 */
GaugeChart.prototype.draw = function() {
	var container = this.container;
	var title = this.title;
	var series = this.series;
	var drilldownSeries = this.drilldownSeries;

	if (this.chart !== null) {
		this.chart.destroy();
	}

	var gaugeSeries = [];
	for (var i = 0; i < series.length; i++) {
		var row = series[i];

		gaugeSeries.push(
				{
					name: row[0],
					data: [row[1]],
					dataLabels: {
						format: '<div style="display: none;"></div>'
					},
					tooltip: {
						valueSuffix: '%'
					}
				}

		);
	}

	var gaugeOptions = {
		chart: {
			type: 'solidgauge',
			borderWidth: 0,
			backgroundColor: 'transparent',
			events: {
				load: function(event) {
					//When is chart ready?

					var smallGraph = this.chartWidth < 200;
					var fractionHtml = '';

					if (this.series.length === 1) {
						fractionHtml = '<div class="gauge-fraction-wrapper" style="bottom: 28px;">';
						fractionHtml += '<span class="gauge-fraction-numerator" style="color: ' + this.series[0].points[0].color + ';">' + this.series[0].dataMax + '%</span>';
						fractionHtml += '</div>';
					}
					if (this.series.length === 2) {
						fractionHtml = '<div class="gauge-fraction-wrapper">';
						fractionHtml += '<span class="gauge-fraction-numerator" style="color: ' + this.series[1].points[0].color + '">' + this.series[1].dataMax + '</span>';
						fractionHtml += '<span class="gauge-fraction-divisor">/</span>';
						fractionHtml += '<span class="gauge-fraction-denominator" style="color: ' + this.series[0].points[0].color + '">' + this.series[0].dataMax + '</span>';
						fractionHtml += '</div>';
					}

					$(this.container).append(fractionHtml);


					var legendHtml = '<ul class="gauge-legend-wrapper">';
					for (var i = 0; i < this.series.length; i++) {
						//console.log(this.series);

						legendHtml += '<li>' + ((smallGraph) ? this.series[i].name : '<div class="gauge-legend-color" style="background-color: ' + this.series[i].points[0].color + '"></div> ' + this.series[i].name) + '</li>';
					}

					legendHtml += '</ul>';
					$(this.container).append(legendHtml);
				}
			}
		},
		title: null,
		exporting: {enabled: false},
		credits: {enabled: false},
		pane: {
			center: ['50%', '85%'],
			size: '140%',
			startAngle: -90,
			endAngle: 90,
			background: {
				backgroundColor: (Highcharts.theme && Highcharts.theme.background) || '#EEE',
				innerRadius: '60%',
				outerRadius: '100%',
				shape: 'arc'
			}
		},
		tooltip: {
			enabled: false
		},
		// the value axis
		yAxis: {
			stops: [
				[0.75, '#EE1111'], // red
				[0.8, '#DF5353'], // red
				[0.85, '#DDDF0D'], // yellow
				[0.9, '#55BF3B'] // green
			],
			lineWidth: 0,
			minorTickInterval: null,
			tickPixelInterval: 400,
			tickWidth: 0,
			title: {
				y: -70
			},
			labels: {
				y: 16
			}
		},
		plotOptions: {
			solidgauge: {
				dataLabels: {
					y: 5,
					borderWidth: 0,
					useHTML: true
				}
			}
		}
	};

	// The speed gauge
	$(container).highcharts(Highcharts.merge(gaugeOptions, {
		yAxis: {
			min: 0,
			max: 100,
			title: {
				text: title
			}
		},
		credits: {
			enabled: false
		},
		series: gaugeSeries

	}));





};

GaugeChart.prototype.destroy = function() {
	if (this.chart !== null) {
		this.chart.destroy();
		this.chart = null;
	}

	this.container = null; // Container debe ser removido de forma externa
	this.title = '';
	this.yTitle = '';
	this.metricId = 0;
	this.metricSize = 0;
	this.series = [];
	this.drilldownSeries = [];
	this.overlap = false;
};