
function JackKnifeChart() {
	this.chart = null;
	this.container = null;
	this.title = '';
	this.yTitle = '';
	this.metricId = 0;
	this.metricSize = 0;
	this.series = [];
}

/**
 * 
 * @returns {undefined}
 */
// TODO: undanify
JackKnifeChart.prototype.draw = function() {
	var container = this.container;
	var title = this.title;
	var yTitle = lang.line('label_chart_' + this.yTitle + '_js');
	var series = this.series;

	var xJkPlotLine = 0;
	var yJkPlotLine = 0;
	var jkCounter = 0;

	for (var i = 0; i < series.length; i++) {
		var indicatorType = parseInt(series[i].extras.indicator_type_id);

		if (indicatorType === INDICATOR_JACK_KNIFE) {
			overridingType = INDICATOR_JACK_KNIFE;
			for (var j = 0; j < series[i].data.length; j++) {
				var point = series[i].data[j];

				xJkPlotLine += point[0];
				yJkPlotLine += point[1];
				jkCounter++;
			}
		}
	}

	if (this.chart !== null) {
		this.chart.destroy();
	}

	Highcharts.setOptions({
		series: [{
				dataLabels: {
					style: {
						fontWeight: 'normal'
					}
				}
			}],
		chart: {
			events: {
				load: function() {
					this.renderer.image(IMG_URL + '/watermark_logo.gif', 450, 15, 185, 37).add(); // x=900px to move image off chart (approx div width minus right margin) adjust as needed
				}
			}
		}});

	var chartOptions = {
		chart: {
			renderTo: container[0],
			type: 'line',
			height: 450,
			zoomType: 'xy',
			plotBorderColor: '#999',
			plotBorderWidth: 1
		},
		title: {
			text: '', //lang.line('label_chart_' + title + '_js'),
			x: -20 //center
		},
		subtitle: {
			text: '', //'Reliability & Maintenance Engineering System',
			x: -20
		},
		xAxis: {
			//type: 'category',
			type: 'datetime',
			//dateTimeLabelFormats: {// don't display the dummy year
			//	month: '%e. %b',
			//year: '%b'
			//},
			labels: {
				rotation: -60 // para que el texto del eje se vea ladeado
			}
//	    //reversed: false,
//	    gridLineWidth: 1,
//	    maxPadding: 0.05,
//	    tickInterval: 100
			//showLastLabel: true
		},
		yAxis: {
			title: {
				text: yTitle + ' (%)'
			},
			labels: {
				formatter: function() {
					return this.value + '%';
				}
			}
		},
		legend: {
			//layout: 'vertical',
			borderWidth: 0,
			maxHeight: 60
		},
		tooltip: {
			headerFormat: '<b>{series.name}</b><br/>',
			pointFormat: '{point.x} : {point.y}%'
		},
		plotOptions: {
			spline: {
				marker: {
					enable: false
				}
			}
		},
		series: series
	};

	if (jkCounter > 0) {

		xJkPlotLine = xJkPlotLine / jkCounter;
		yJkPlotLine = yJkPlotLine / jkCounter;
	}

	chartOptions.xAxis = {
		gridLineWidth: 0,
		minorGridLineWidth: 0,
		title: {
			text: 'ln(n)'
		},
		plotLines: [{
				value: xJkPlotLine,
				color: '#ff0000',
				width: 1,
				zIndex: 4,
				label: {text: xJkPlotLine.toFixed(2),
					style: {
						color: '#ccc',
						fontSize: '10px'
					}}
			}]
	},
	chartOptions.yAxis = {
		gridLineWidth: 0,
		minorGridLineWidth: 0,
		title: {
			text: 'ln(MTTR)'
		},
		labels: {
			formatter: function() {
				return this.value;
			}
		},
		plotLines: [{
				value: yJkPlotLine,
				color: '#ff0000',
				width: 1,
				zIndex: 4,
				label: {text: yJkPlotLine.toFixed(2),
					style: {
						color: '#ccc',
						fontSize: '10px'
					}}
			}]
	};

	chartOptions.tooltip = {
		headerFormat: '<b>{series.name}</b><br/>',
		pointFormat: '({point.x}, {point.y})'
	};

	chartOptions.labels = {
		items: [{
				html: 'Agudo',
				style: {
					left: '25px',
					top: '25px'
				}
			}, {
				html: 'Agudo Crónico',
				style: {
					left: '375px',
					top: '25px'
				}
			}, {
				html: 'Bajo Control',
				style: {
					left: '25px',
					top: '225px'
				}
			}, {
				html: 'Crónico',
				style: {
					left: '375px',
					top: '225px'
				}
			}]};



	this.chart = new Highcharts.Chart(chartOptions);
};

JackKnifeChart.prototype.getFormattedSerie = function(indicator) {
	var points = [];
	for (var i = 0; i < indicator.indicator_data.length; i++) {

		var point_data = indicator.indicator_data[i];
		
		var jsonObj = JSON.parse(point_data.value);
		
		points.push([roundFloat(jsonObj.n), roundFloat(jsonObj.mttr)]);
	}

	var seriesName = indicator.name;

	// Extras son utilizados para guardar preferencias
	var seriesExtras = {
		indicator_type_id: indicator.indicator_type,
		report_id: indicator.report_id,
		metric_id: indicator.metric_id,
		subsystem_id: indicator.subsystem_id
	};

	return {name: seriesName, data: points, extras: seriesExtras};
};