
function MeanTimeChart() {
    this.chart = null;
    this.container = null;
    this.title = '';
    this.yTitle = '';
    this.metricId = 0;
    this.metricSize = 0;
    this.series = [];
}

/**
 * 
 * @returns {undefined}
 */
MeanTimeChart.prototype.draw = function() {
    var container = this.container;
    var title = this.title;
    var yTitle = lang.line('label_chart_' + this.yTitle + '_js');
    var series = this.series;
    var parent = this;

    if (this.chart !== null) {
	this.chart.destroy();
    }

    Highcharts.setOptions({
	series: [{
		dataLabels: {
		    style: {
			fontWeight: 'normal'
		    }
		}
	    }],
	chart: {
	    events: {
		load: function() {
		    this.renderer.image(IMG_URL + '/watermark_logo.gif', 450, 15, 185, 37).add(); // x=900px to move image off chart (approx div width minus right margin) adjust as needed
		}
	    }
	},
	credits: {
	    enabled: false
	},
	exporting: {
	    enabled: false
	}
    });

    var chartOptions = {
	chart: {
	    renderTo: container[0],
	    type: 'line',
	    height: 390,
	    zoomType: 'xy',
	    plotBorderColor: '#999',
	    plotBorderWidth: 1
	},
	title: {
	    text: '', //lang.line('label_chart_' + title + '_js'),
	    x: -20 //center
	},
	subtitle: {
	    text: '', //'Reliability & Maintenance Engineering System',
	    x: -20
	},
	xAxis: {
	    type: 'datetime',
	    labels: {
		rotation: -60, // para que el texto del eje se vea ladeado
		formatter: function() {
		    return  Highcharts.dateFormat((!parent.overlap) ? '%d-%b-%y' : '%d-%b',
				    new Date(this.value));
		}
	    }
	},
	yAxis: {
	    min: 0,
	    title: {
		text: yTitle.toUpperCase()
	    },
	    labels: {
		formatter: function() {
		    return this.value;
		}
	    }
	},
	legend: {
	    //layout: 'vertical',
	    borderWidth: 0,
	    maxHeight: 60
	},
	plotOptions: {
	    spline: {
		marker: {
		    enable: false
		}
	    }
	},
	tooltip: {
	    formatter: function() {
		return  '<b>' + this.series.name + ': ' + this.y + '</b><br/>' +
			Highcharts.dateFormat((!parent.overlap) ? '%d-%b-%Y' : '%d-%b',
				new Date(this.x));
	    }
	},
	series: series
    };

    this.chart = new Highcharts.Chart(chartOptions);
};

MeanTimeChart.prototype.getFormattedSerie = function(indicator) {
    var points = [];
    var indicatorYear = '';

    for (var i = 0; i < indicator.indicator_data.length; i++) {

	var point_data = indicator.indicator_data[i];

	// Se obtiene el label a utilizar (Eje X)
	// TODO: rethink date replace
	if (this.overlap === true) {
	    var temp_from = point_data.from.split('/');
	    if (indicatorYear === '')
		indicatorYear = temp_from[2];
	    temp_from[2] = '2000';
	    point_data.from = temp_from.join('/');
	}

	var date = new Date(latinDateToIso(point_data.from));
	var point_label = date.getTime();

	points.push([point_label, point_data.value * 100]);
    }

    var seriesName = indicator.name + ' (' + indicator.alias + (indicatorYear !== '' ? ' ' + indicatorYear : '') + ')';

    // Extras son utilizados para guardar preferencias
    var seriesExtras = {
	indicator_type_id: indicator.indicator_type,
	report_id: indicator.report_id,
	metric_id: indicator.metric_id,
	subsystem_id: indicator.subsystem_id
    };

    return {name: seriesName, data: points, extras: seriesExtras};
};