
function MixMaintenanceChart() {
	this.chart = null;
	this.container = null;
	this.title = '';
	this.yTitle = '';
	this.metricId = 0;
	this.metricSize = 0;
	this.series = [];
}

/**
 * 
 * @returns {undefined}
 */
// TODO: undanify
MixMaintenanceChart.prototype.draw = function() {
	var container = this.container;
	var title = this.title;
	var yTitle = lang.line('label_chart_' + this.yTitle + '_js');
	var series = this.series;



	var mmSeries = [{name: 'MC', data: []}, {name: 'MP', data: []}];
	var categories = [];

	for (var i = 0; i < series.length; i++) {
		var indicatorType = series[i].extras.indicator_type_id;

		if (indicatorType === INDICATOR_MIX_MAINTENANCE_HRS || indicatorType === INDICATOR_MIX_MAINTENANCE_QTY) {
			mmSeries[0].data.push(series[i].data[0]);
			mmSeries[1].data.push(series[i].data[1]);
			categories.push(series[i].name);
		}

	}

	if (this.chart !== null) {
		this.chart.destroy();
	}

	Highcharts.setOptions({
		series: [{
				dataLabels: {
					style: {
						fontWeight: 'normal'
					}
				}
			}],
		chart: {
			events: {
				load: function() {
					this.renderer.image(IMG_URL + '/watermark_logo.gif', 450, 15, 185, 37).add(); // x=900px to move image off chart (approx div width minus right margin) adjust as needed
				}
			}
		}});

	var chartOptions = {
		chart: {
			renderTo: container[0],
			type: 'line',
			height: 450,
			zoomType: 'xy',
			plotBorderColor: '#999',
			plotBorderWidth: 1
		},
		title: {
			text: '', //lang.line('label_chart_' + title + '_js'),
			x: -20 //center
		},
		subtitle: {
			text: '', //'Reliability & Maintenance Engineering System',
			x: -20
		},
		xAxis: {
			//type: 'category',
			type: 'datetime',
			//dateTimeLabelFormats: {// don't display the dummy year
			//	month: '%e. %b',
			//year: '%b'
			//},
			labels: {
				rotation: -60 // para que el texto del eje se vea ladeado
			}
//	    //reversed: false,
//	    gridLineWidth: 1,
//	    maxPadding: 0.05,
//	    tickInterval: 100
			//showLastLabel: true
		},
		yAxis: {
			title: {
				text: yTitle
			},
			labels: {
				formatter: function() {
					return this.value;
				}
			}
		},
		legend: {
			//layout: 'vertical',
			borderWidth: 0,
			maxHeight: 60
		},
		tooltip: {
			headerFormat: '<b>{series.name}</b><br/>',
			pointFormat: '{point.x} : {point.y}'
		},
		plotOptions: {
			spline: {
				marker: {
					enable: false
				}
			}
		},
		series: series
	};



	if (categories.length > 0) {
		chartOptions.chart.type = 'column';

		chartOptions.xAxis = {
			categories: categories
		};

		chartOptions.plotOptions = {
			column: {
				stacking: 'percent',
				dataLabels: {
					enabled: true,
					color: (Highcharts.theme && Highcharts.theme.dataLabelsColor) || 'white',
					style: {
						textShadow: '0 0 3px black, 0 0 3px black'
					}
				}

			}
		};

		chartOptions.tooltip = {
			pointFormat: '<span style="color:{series.color}">{series.name}</span>: <b>{point.y}</b> ({point.percentage:.0f}%)<br/>',
			shared: true
		};

		chartOptions.series = mmSeries;
	}

	this.chart = new Highcharts.Chart(chartOptions);
};

MixMaintenanceChart.prototype.getFormattedSerie = function(indicator) {
	var points = [];
	for (var i = 0; i < indicator.indicator_data.length; i++) {

		var point_data = indicator.indicator_data[i];
		
		
		var jsonObj = JSON.parse(point_data.value);

		points.push(roundFloat(jsonObj.mc));
		points.push(roundFloat(jsonObj.mp));
	}

	var seriesName = indicator.name;

	// Extras son utilizados para guardar preferencias
	var seriesExtras = {
		indicator_type_id: indicator.indicator_type,
		report_id: indicator.report_id,
		metric_id: indicator.metric_id,
		subsystem_id: indicator.subsystem_id
	};

	return {name: seriesName, data: points, extras: seriesExtras};
};