
function OeeChart() {
	this.chart = null;
	this.chartOptions = {};
	this.container = null;
	this.title = '';
	this.yTitle = '';
	this.metricId = 0;
	this.metricSize = 0;
	this.series = [];
}

OeeChart.prototype.addIndicator = function(indicator, redraw) {

	if (typeof indicator.subsystem_id === 'undefined')
		return;

	var serie = this.getFormattedSerie(indicator);

	this.series.push(serie);

	if (this.isInitialized()) {
		if (typeof redraw === 'undefined') {
			redraw = true;
		}
		// TODO: Check redraw para evitar multiples cargas
		this.chart.addSeries(serie, redraw);
	}
};

/**
 * 
 * @returns {undefined}
 */
OeeChart.prototype.draw = function(showLegend) {
	var container = this.container;
	var title = this.title;
	var yTitle = lang.line('label_chart_' + this.yTitle + '_js');
	var series = this.series;

	if (this.chart !== null) {
		this.chart.destroy();
	}

	if (typeof showLegend === 'undefined') {
		showLegend = true;
	}

	var categories = [];
	var posibleTime = 0;
	var timeBarsIndexes = [0, 3, 7, 10];

	// Se definen los colores por barra;
	var timeColor = '#7CB5EC';
	var detentionColor = '#F15C80';
	var barsColors = [];

	// Se preparan los valores para ser utilizados en columnas de rango y se obtienen las categorias
	for (var i = 0; i < series.length; i++) {
		var newSerieData = [];
		for (var j = 0; j < series[i].data.length; j++) {
			var serieData = series[i].data[j];

			// Se almacena la categoria para ser utilizada en el grafico
			var catParts = serieData[0].split(' ');
			var catTitle = '';
			for (var c = 0; c < catParts.length; c++) {
				catTitle += catParts[c][0];
			}
			categories.push(catTitle.toUpperCase());

			// columnas de tiempo comienzan desde 0, el resto es un rango calculado
			if (timeBarsIndexes.indexOf(j) >= 0) {
				if (j === 0) {
					posibleTime = serieData[1]; // Se registra el tiempo posible
				}
				newSerieData.push([0, serieData[1]]);
				barsColors.push(timeColor);
			}
			else {
				// Se realiza el calculo para la cascada a partir del tiempo posible
				newSerieData.push([posibleTime - serieData[1], posibleTime]);
				posibleTime -= serieData[1]; // Se resta al tiempo posible para la cascada con la proxima columna
				barsColors.push(detentionColor);
			}
		}
		series[i].data = newSerieData;
	}

	Highcharts.setOptions({
		series: [{
				dataLabels: {
					style: {
						fontWeight: 'normal'
					}
				}
			}]/*,
			 chart: {
			 events: {
			 load: function() {
			 this.renderer.image(IMG_URL + '/watermark_logo.gif', 450, 15, 185, 37).add(); // x=900px to move image off chart (approx div width minus right margin) adjust as needed
			 }
			 }
			 }*/});



	var chartOptions = {
		chart: {
			renderTo: container[0],
			type: 'columnrange',
			height: 450,
			zoomType: 'xy'
		},
		title: {
			text: '', //lang.line('label_chart_' + title + '_js'),
			x: -20 //center
		},
		subtitle: {
			text: '', //'Reliability & Maintenance Engineering System',
			x: -20
		},
		xAxis: {
			categories: categories
		},
		yAxis: {
			title: {
				text: 'Horas'
			},
			min: 0
		},
		legend: {
			borderWidth: 0,
			maxHeight: 60
		},
		plotOptions: {
			series: {
				colorByPoint: series.length === 1
			}
		},
		exporting: {enabled: false},
		credits: {
			enabled: false
		},
		series: series
	};

	if (series.length === 1) {
		chartOptions.colors = barsColors;
	}

	if (showLegend === false) {
		chartOptions.legend = {enabled: false}
	}

	chartOptions = $.extend(true, chartOptions, this.chartOptions);

	this.chart = new Highcharts.Chart(chartOptions);
};

OeeChart.prototype.getFormattedSerie = function(indicator) {
	var points = [];
	for (var i = 0; i < indicator.indicator_data.length; i++) {

		var point_data = indicator.indicator_data[i];
		var failures = JSON.parse(point_data.value).data;
		//console.log(failures);
		for (var failure_key in failures) {
			points.push([failures[failure_key]['name'], roundFloat(failures[failure_key]['hours']), failures[failure_key]['alias']]);
		}
	}

	var seriesName = indicator.name;

	// Extras son utilizados para guardar preferencias
	var seriesExtras = {
		indicator_type_id: indicator.indicator_type,
		report_id: indicator.report_id,
		metric_id: indicator.metric_id,
		subsystem_id: indicator.subsystem_id
	};

	return {name: seriesName, data: points, extras: seriesExtras};
};