
function ParetoChart() {
	this.chart = null;
	this.container = null;
	this.title = '';
	this.yTitle = '';
	this.metricId = 0;
	this.metricSize = 0;
	this.series = [];
}

/**
 * 
 * @returns {undefined}
 */
// TODO: undanify
ParetoChart.prototype.draw = function() {
	var container = this.container;
	var title = this.title;
	var yTitle = lang.line('label_chart_' + this.yTitle + '_js');
	var series = this.series;

	var mmSeries = [{name: 'MC', data: []}, {name: 'MP', data: []}];
	var categories = [];

	var paretoDetected = [];
	var paretoCount = 0;

	for (var i = 0; i < series.length; i++) {
		var indicatorType = series[i].extras.indicator_type_id;

		if (indicatorType === INDICATOR_PARETO || indicatorType === INDICATOR_FAILURE_MODE) {
			overridingType = indicatorType;

			// Se agregan todos los modos de falla
			var failureModes = series[i].data[0];
			series[i]['failureCount'] = 0;
			for (var key in failureModes) {
				var k = key.toUpperCase();
				if ($.inArray(k, paretoDetected) === -1) {
					paretoDetected.push(k.toUpperCase());
				}
				series[i]['failureCount'] += failureModes[key].q;
			}
			paretoCount += series[i]['failureCount']; // Conteo general para suma al 100%
		}
	}

	if (this.chart !== null) {
		this.chart.destroy();
	}

	Highcharts.setOptions({
		series: [{
				dataLabels: {
					style: {
						fontWeight: 'normal'
					}
				}
			}],
		chart: {
			events: {
				load: function() {
					this.renderer.image(IMG_URL + '/watermark_logo.gif', 450, 15, 185, 37).add(); // x=900px to move image off chart (approx div width minus right margin) adjust as needed
				}
			}
		}});

	var chartOptions = {
		chart: {
			renderTo: container[0],
			type: 'line',
			height: 450,
			zoomType: 'xy',
			plotBorderColor: '#999',
			plotBorderWidth: 1
		},
		title: {
			text: '', //lang.line('label_chart_' + title + '_js'),
			x: -20 //center
		},
		subtitle: {
			text: '', //'Reliability & Maintenance Engineering System',
			x: -20
		},
		xAxis: {
			type: 'datetime'
		},
		yAxis: {
			title: {
				text: yTitle + ' (%)'
			},
			labels: {
				formatter: function() {
					return this.value + '%';
				}
			}
		},
		legend: {
			//layout: 'vertical',
			borderWidth: 0,
			maxHeight: 60
		},
		tooltip: {
			headerFormat: '<b>{series.name}</b><br/>',
			pointFormat: '{point.x} : {point.y}%'
		},
		plotOptions: {
			spline: {
				marker: {
					enable: false
				}
			}
		},
		series: series
	};



	if (categories.length > 0) {
		chartOptions.chart.type = 'column';

		chartOptions.xAxis = {
			categories: categories
		};

		chartOptions.plotOptions = {
			column: {
				stacking: 'percent',
				dataLabels: {
					enabled: true,
					color: (Highcharts.theme && Highcharts.theme.dataLabelsColor) || 'white',
					style: {
						textShadow: '0 0 3px black, 0 0 3px black'
					}
				}

			}
		};

		chartOptions.tooltip = {
			pointFormat: '<span style="color:{series.color}">{series.name}</span>: <b>{point.y}</b> ({point.percentage:.0f}%)<br/>',
			shared: true
		};

		chartOptions.series = mmSeries;
	}


	var paretoSeries = [];
	var paretoCategories = [];
	var paretoAccumulatedSeries = [];

	paretoDetected.sort();
	series.sort(function(a, b) {
		return b.failureCount - a.failureCount;
	});

	// Se generan las barras pareto
	for (var i = 0; i < paretoDetected.length; i++) {
		var failureMode = {name: paretoDetected[i], data: []};

		for (var j = 0; j < series.length; j++) {
			var failureModes = series[j].data[0];
			var failureFound = false;

			paretoCategories.push(series[j].name);

			for (var k in failureModes) {
				if (k.toUpperCase() === failureMode.name) {
					failureMode.data.push(roundFloat(failureModes[k].h));
					failureFound = true;
					break;
				}
			}

			if (!failureFound) {
				failureMode.data.push(null);
			}
		}

		paretoSeries.push(failureMode);
	}

	// Se genera el indicador de acumulado
	var accumulated = 0;
	for (var i = 0; i < series.length; i++) {
		var serie = series[i];
		accumulated += serie.failureCount * 100 / paretoCount;
		paretoAccumulatedSeries.push(roundFloat(accumulated));
	}
	var accumulatedSerie = {
		type: 'line',
		name: '% Acumulado',
		color: '#990000',
		yAxis: 1,
		data: paretoAccumulatedSeries
	};

	chartOptions.chart.type = 'column';

	chartOptions.xAxis = {
		title: {
			text: 'Equipos / Configuraciones'
		},
		categories: paretoCategories,
		labels: {
			//rotation: -60, // para que el texto del eje se vea ladeado
			formatter: function() {
				return truncateString(this.value, 12, true);
			},
			useHTML: true
		}
	};


	chartOptions.yAxis = [
		{
			title: {
				text: 'Horas'
			},
			labels: {
				formatter: function() {
					return this.value;
				}
			}
		},
		{
			title: {text: ''},
			max: 100,
			lineColor: '#333',
			opposite: true,
			labels: {
				formatter: function() {
					return this.value + '%';
				}
			}
		}

	];

	chartOptions.plotOptions = {
		column: {
			stacking: 'normal',
			dataLabels: {
				enabled: true,
				color: (Highcharts.theme && Highcharts.theme.dataLabelsColor) || 'white',
				style: {
					textShadow: '0 0 3px black, 0 0 3px black'
				},
				formatter: function() {
					if (this.y != 0) {
						return this.y;
					}
				}
			}

		}
	};

	chartOptions.tooltip = {
		pointFormat: '<span style="color:{series.color}">{series.name}</span>: <b>{point.y}</b> ({point.percentage:.0f}%)<br/>',
		shared: true
	};
	chartOptions.series = paretoSeries;
	chartOptions.series.push(accumulatedSerie); // Porcentaje


	//chartOptions.series.dataLabels.style.fontWeight = 'normal';

	this.chart = new Highcharts.Chart(chartOptions);
};

ParetoChart.prototype.getFormattedSerie = function(indicator) {
	var points = [];
	for (var i = 0; i < indicator.indicator_data.length; i++) {

		var point_data = indicator.indicator_data[i];

		points.push(JSON.parse(point_data.value));
	}

	var seriesName = indicator.name;

	// Extras son utilizados para guardar preferencias
	var seriesExtras = {
		indicator_type_id: indicator.indicator_type,
		report_id: indicator.report_id,
		metric_id: indicator.metric_id,
		subsystem_id: indicator.subsystem_id
	};

	return {name: seriesName, data: points, extras: seriesExtras};
};