
function ProductionChart() {
	this.chart = null;
	this.container = null;
	this.title = '';
	this.yTitle = '';
	this.metricId = 0;
	this.metricSize = 0;
	this.series = [];
}

ProductionChart.prototype.addIndicator = function(indicator, redraw) {
	if (typeof indicator.subsystem_id === 'undefined')
		return;

	for (var i = 0; i < indicator.indicator_data.length; i++) {

		var indicatorTarget = {};
		indicatorTarget.indicator_type = indicator.indicator_type;
		indicatorTarget.report_id = indicator.report_id;
		indicatorTarget.metric_id = indicator.metric_id;
		indicatorTarget.subsystem_id = indicator.subsystem_id;
		indicatorTarget.name = lang.line('label_target_production_js') + ' ' + indicator.indicator_data[i].value;
		indicatorTarget.indicator_data = [{
				value: indicator.indicator_data[i].value,
				from: indicator.indicator_data[i].from,
				to: []
			}];
		var serieTarget = this.getFormattedSerie(indicatorTarget);
		this.series.push(serieTarget);

		var indicatorActual = {};
		indicatorActual.indicator_type = indicator.indicator_type;
		indicatorActual.report_id = indicator.report_id;
		indicatorActual.metric_id = indicator.metric_id;
		indicatorActual.subsystem_id = indicator.subsystem_id;
		indicatorActual.name = lang.line('label_actual_production_js') + ' ' + indicator.indicator_data[i].value;
		indicatorActual.indicator_data = [{
				value: indicator.indicator_data[i].value,
				from: indicator.indicator_data[i].to,
				to: []
			}];
		var serieActual = this.getFormattedSerie(indicatorActual);

		this.series.push(serieActual);

		if (this.isInitialized()) {
			if (typeof redraw === 'undefined') {
				redraw = true;
			}

			this.chart.addSeries(serieActual, redraw);
		}
	}


};

/**
 * 
 * @returns {undefined}
 */
ProductionChart.prototype.draw = function() {
	var container = this.container;
	var title = this.title;
	var yTitle = lang.line('label_chart_' + this.yTitle + '_js');
	var series = this.series;

	var categories = [lang.line('label_jan_js'),
		lang.line('label_feb_js'),
		lang.line('label_mar_js'),
		lang.line('label_apr_js'),
		lang.line('label_may_js'),
		lang.line('label_jun_js'),
		lang.line('label_jul_js'),
		lang.line('label_aug_js'),
		lang.line('label_sep_js'),
		lang.line('label_oct_js'),
		lang.line('label_nov_js'),
		lang.line('label_dec_js')
	];

	if (this.chart !== null) {
		this.chart.destroy();
	}

	Highcharts.setOptions({
		series: [{
				dataLabels: {
					style: {
						fontWeight: 'normal'
					}
				}
			}],
		chart: {
			events: {
				load: function() {
					this.renderer.image(IMG_URL + '/watermark_logo.gif', 450, 15, 185, 37).add(); // x=900px to move image off chart (approx div width minus right margin) adjust as needed
				}
			}
		}});

	var chartOptions = {
		chart: {
			renderTo: container[0],
			type: 'column',
			height: 450,
			zoomType: 'xy',
			plotBorderColor: '#999',
			plotBorderWidth: 1
		},
		title: {
			text: '', //lang.line('label_chart_' + title + '_js'),
			x: -20 //center
		},
		subtitle: {
			text: '', //'Reliability & Maintenance Engineering System',
			x: -20
		},
		xAxis: {
			categories: categories
//	    //reversed: false,
//	    gridLineWidth: 1,
//	    maxPadding: 0.05,
//	    tickInterval: 100
					//showLastLabel: true
		},
		yAxis: {
			title: {
				text: yTitle
			},
			labels: {
				formatter: function() {
					return this.value;
				}
			}
		},
		legend: {
			//layout: 'vertical',
			borderWidth: 0,
			maxHeight: 60
		},
		tooltip: {
			shared: true
		},
		plotOptions: {
			spline: {
				marker: {
					enable: false
				}
			}
		},
		credits: {
			enabled: false
		},
		exporting: {
			enabled: false
		},
		series: series
	};

	this.chart = new Highcharts.Chart(chartOptions);
};

ProductionChart.prototype.getFormattedSerie = function(indicator) {
	var points = [];
	for (var i = 0; i < indicator.indicator_data.length; i++) {

		var point_data = indicator.indicator_data[i];
		var production = JSON.parse(point_data.from);
		for (var key in production) {
			points.push(parseFloat(production[key]));
		}
	}

	var seriesName = indicator.name;

	// Extras son utilizados para guardar preferencias
	var seriesExtras = {
		indicator_type_id: indicator.indicator_type,
		report_id: indicator.report_id,
		metric_id: indicator.metric_id,
		subsystem_id: indicator.subsystem_id
	};
	
	console.log({name: seriesName, data: points, extras: seriesExtras});
	return {name: seriesName, data: points, extras: seriesExtras};
};