
function Indicators() {
    this.id = 0;
    this.blocks = [];
    this.selector = '';
    this.type = 2;
    this.selectedElements = [];
    this.blockFactory = null;
}

Indicators.prototype.isInitialized = function() {

    if (this.blocks.length === 0) {
	return false;
    }

    return true;
};


Indicators.prototype.setSelector = function(selector) {
    this.selector = selector;
};
Indicators.prototype.getSelector = function() {
    return this.selector;
};

Indicators.prototype.setType = function(type) {
    this.type = type;
};
Indicators.prototype.getType = function() {
    return this.type;
};

Indicators.prototype.setSelectedElements = function(selectedElements) {
    this.selectedElements = [];

    for (var i = 0; i < selectedElements.length; i++)
    {
	var selectedElement = selectedElements[i];
	var subsystemRef = ((selectedElement.data.subsystem_ref !== null) ? selectedElement.data.subsystem_ref : 0);
	this.selectedElements.push({
	    id: selectedElement.original.oid,
	    node_type: selectedElement.original.node_type,
	    is_leaf: (selectedElement.children.length === 0),
	    subsystem_ref: subsystemRef});
    }
};
Indicators.prototype.getSelectedElements = function() {
    return this.selectedElements;
};

Indicators.prototype.addBlock = function(block, index) {

    if (this.blocks.length <= index) {
	this.blocks.push(block);
    }
    else {
	this.blocks.splice(index, 0, block);
    }
};
Indicators.prototype.removeBlock = function(index) {

    if (this.blocks.length <= index) {
	return;
    }

    this.blocks[index].destroy();
    this.blocks.splice(index, 1);

    if (this.blocks.length === 0) {
	app.closeReport();
	$('#map').removeClass('blurry');
    }
};

Indicators.prototype.updateChartBlockMetrics = function(blockIndex, metric) {
    var block = this.getBlock(blockIndex);
    var chart = block.getChart();
    chart.updateMetrics(metric, blockIndex);
};

Indicators.prototype.printChartBlock = function(blockIndex) {

    var block = this.getBlock(blockIndex);
    var chart = block.getChart();
    chart.chart.print();
};

Indicators.prototype.exportChartBlock = function(blockIndex, type) {
    var block = this.getBlock(blockIndex);
    var chart = block.getChart();
    chart.chart.exportChart(
	    {
		filename: lang.line('label_chart_' + block.getTitle() + '_js'),
		type: type,
		width: 900,
		marginRight: 50
	    },
    {
	title: {
	    text: lang.line('label_chart_' + block.getTitle() + '_js')
	}
    }
    );
};

Indicators.prototype.increaseWidth = function(blockIndex) {
    var block = this.getBlock(blockIndex);
    block.increaseWidth();
};

Indicators.prototype.decreaseWidth = function(blockIndex) {
    var block = this.getBlock(blockIndex);
    block.decreaseWidth();
};

Indicators.prototype.getBlock = function(index) {
    return (this.blocks.length <= index) ? null : this.blocks[index];
};
Indicators.prototype.getBlockIndex = function(block) {

    for (var i = 0; i < this.blocks.length; i++) {
	if (this.blocks[i] === block) {
	    return i;
	}
    }

    return -1;
};
Indicators.prototype.getChart = function(index) {
    return this.charts[index];
};
Indicators.prototype.getSize = function() {
    return this.blocks.length;
};

Indicators.prototype.getConfig = function() {
    var config = [];

    for (var i = 0; i < this.blocks.length; i++)
    {
	var currentBlock = this.blocks[i];

	//TODO: Indexar tipo block factory
	if (currentBlock.getType() === 99) {
	    continue;
	}

	var blockChartSeries = currentBlock.getChart().getSeries();
	var block = {type: currentBlock.getType(), position: i, class: currentBlock.getCustomClass()};


	var indicators = [];
	for (var j = 0; j < blockChartSeries.length; j++) {
	    var indicatorsData = blockChartSeries[j].extras;
	    indicators.push(indicatorsData);
	}

	config.push({block: block, indicators: indicators});
    }

    return config;

};

Indicators.prototype.saveConfig = function(type) {
    var parent = this;

    var data = $.param({
	preferences_config: this.getConfig(),
	preferences_type_id: typeof type === 'undefined' ? this.type : type,
	preferences_selected_elements: JSON.stringify(this.selectedElements),
	preferences_is_active: 1
    });

    $.ajax({
	type: 'POST',
	url: BASE_URL + 'indicators/save_preferences',
	data: data,
	dataType: 'json',
	success: function(data, textStatus, jqXHR) {

	    if (data.status) {
		parent.showMessage(data.msg, 'success');
	    }
	    else {
		parent.showMessage(data.msg, 'error');
	    }
	}
    }).done(function() {
    }).fail(function() {
	parent.showMessage('Ocurri&oacute; un error ejecutando este comando', 'error');
    }).always(function() {
	parent.loadPreferences();
    });

};

Indicators.prototype.updateConfig = function() {
    var parent = this;
    var data = $.param({
	preferences_id: this.id,
	preferences_config: this.getConfig(),
	preferences_type_id: this.type,
	preferences_selected_elements: JSON.stringify(this.selectedElements),
	preferences_is_active: 1
    });

    $.ajax({
	type: 'POST',
	url: BASE_URL + 'indicators/update_preferences',
	data: data,
	dataType: 'json',
	success: function(data, textStatus, jqXHR) {

	    if (data.status) {
		parent.showMessage(data.msg, 'success');
	    }
	    else {
		parent.showMessage(data.msg, 'error');
	    }
	}
    }).done(function() {
    }).fail(function() {
	parent.showMessage('Ocurri&oacute; un error ejecutando este comando', 'error');
    }).always(function() {
    });

};

Indicators.prototype.loadPreferences = function() {
    var ajax = new Ajax();

    ajax.reserveAccess();
    ajax.setAddress('indicators/get_preferences');
    ajax.setElementRef('#indicators-available-preferences');
    ajax.setMethod('post');
    ajax.setParameters({
	selected_elements: JSON.stringify(this.selectedElements),
	target_element: '#indicators-available-preferences'
    });
    ajax.setCallback(function(ajaxObject) {
	ajaxObject.loadElements();
    });
    ajax.query();
};

Indicators.prototype.loadIndicatorByPreference = function(preferenceObject, title) {

    this.destroy();
    var parent = this;
    parent.destroy();
    parent.setSelector('#main-panel-content');
    parent.setType(SETTINGS_TYPE_GENERAL_INDICATORS);
    var queryGroup = [];

    var groupedIndicators = {};
    for (var i = 0; i < preferenceObject.length; i++)
    {
	var preference = preferenceObject[i];
	for (var j = 0; j < preference.indicators.length; j++)
	{
	    var indicator = preference.indicators[j];

	    // Query ajax granular para aprovechar cache de browser
	    var ajax = new Ajax();
	    ajax.reserveAccess();
	    ajax.setAddress('indicators/get_indicator');
	    ajax.setMethod('post');
	    ajax.setParameters({
		indicator_type_id: indicator.indicator_type_id,
		metric_id: indicator.metric_id,
		report_id: indicator.report_id,
		subsystem_id: indicator.subsystem_id
	    });

	    // Variable Custom
	    ajax.block = preference.block;

	    ajax.setCallback(function(ajaxObject) {
		for (var k = 0, size = ajaxObject.response.length; k < size; k++) {
		    var indicator = ajaxObject.response[k];

		    // La preferencia indica la agrupacion del block
		    var compatibilityHash = ajaxObject.block.position + '';
		    if (typeof groupedIndicators[compatibilityHash] === 'undefined') {
			groupedIndicators[compatibilityHash] = [];
		    }

		    // Se agrega el indicador en el grupo correspondiente segun el hash de compatibilidad
		    groupedIndicators[compatibilityHash].push([indicator, ajaxObject.block]);
		}
	    });

	    var queryRef = ajax.query();
	    queryGroup.push(queryRef);
	}
    }

    $.when.apply($, queryGroup).done(function() {
	for (var chartGroupKey in groupedIndicators) {

	    var chartGroup = [];
	    var blockPreference = null;
	    for (i = 0; i < groupedIndicators[chartGroupKey].length; i++) {
		chartGroup.push(groupedIndicators[chartGroupKey][i][0]);

		if (blockPreference === null)
		    blockPreference = groupedIndicators[chartGroupKey][i][1];

	    }

	    parent.createIndicatorBlock(chartGroup, blockPreference);
	    $('#indicators-preferences > span').html(title);
	}
    });
};

Indicators.prototype.loadIndicatorBySelection = function(selectionObject, enableOverlap) {

    var parent = this;

    if (this.selector == '')
	parent.setSelector('#main-panel-content');

    var queryGroup = [];

    var groupedIndicators = {};

    for (var i = 0; i < selectionObject.length; i++)
    {
	var indicator = selectionObject[i];
	// Query ajax granular para aprovechar cache de browser
	var ajax = new Ajax();

	ajax.reserveAccess();
	ajax.setAddress('indicators/get_indicator');
	ajax.setMethod('post');
	ajax.setParameters(indicator);

	ajax.setCallback(function(ajaxObject) {
	    for (var j = 0, size = ajaxObject.response.length; j < size; j++) {
		var indicator = ajaxObject.response[j];

		// Si no existe la key en el objeto, se crea 'factory' es para combinar todo
		var compatibilityHash = 'factory';

		// Si no existe la key en el objeto, se crea
		if (typeof groupedIndicators[compatibilityHash] === 'undefined') {
		    groupedIndicators[compatibilityHash] = [];
		}

		// Se agrega el indicador en el grupo correspondiente segun el hash de compatibilidad
		groupedIndicators[compatibilityHash].push(indicator);
	    }
	});

	var queryRef = ajax.query();
	queryGroup.push(queryRef);
    }

    $.when.apply($, queryGroup).done(function() {
	for (var chartGroupKey in groupedIndicators) {
	    var chartGroup = groupedIndicators[chartGroupKey];

	    var blockPreference = {
		class: 'uk-width-large-1-2',
		position: indicatorsWrapper.getSize() - ((indicatorsWrapper.getBlockFactory() !== null) ? 1 : 0),
		type: 0,
		enableOverlap: enableOverlap
	    };
	    parent.createIndicatorBlock(chartGroup, blockPreference);
	}
    });
};

Indicators.prototype.loadIndicatorsByReportDate = function(dateFrom, dateTo) {
    var parent = this;
    parent.destroy();
    parent.setSelector('#main-panel-content');
    parent.setType(SETTINGS_TYPE_GENERAL_INDICATORS);
    parent.setSelectedElements($("#jstree").jstree().get_selected(true));

    var ajax = new Ajax();

    if (ajax.reserveAccess()) {
	ajax.setAddress('indicators/by_report_date');

	// Se consulta con fecha desde / hasta para los elementos seleccionados
	ajax.setParameters({
	    date_from: dateFrom,
	    date_to: dateTo,
	    selected_elements: JSON.stringify(this.selectedElements)
	});

	var groupedIndicators = {};
	ajax.setCallback(function(ajaxObject) {

	    for (var k = 0, size = ajaxObject.response.length; k < size; k++) {
		var indicator = ajaxObject.response[k];
		// Si no existe la key en el objeto, se crea
		if (typeof groupedIndicators[indicator.compatibility_hash] === 'undefined') {
		    groupedIndicators[indicator.compatibility_hash] = [];
		}

		// Se agrega el indicador en el grupo correspondiente segun el hash de compatibilidad
		groupedIndicators[indicator.compatibility_hash].push(indicator);
	    }

	    if (!ajaxObject.status) {
		app.showMessage(lang.line('msg_selection_without_indicators_js'), 'warning');
	    }
	});

	var queryObject = ajax.query();

	$.when(queryObject).done(function() {
	    parent.createReport(groupedIndicators, parent, 'Reporte ' + dateFrom + ' a ' + dateTo);
	});
    }
};

Indicators.prototype.createReport = function(groupedIndicators, blockSettings, reportTitle) {
    for (var chartGroupKey in groupedIndicators) {
	var chartGroup = groupedIndicators[chartGroupKey];
	
	this.createIndicatorBlock(chartGroup, blockSettings[chartGroupKey]);
	$('.report-title').html(reportTitle);
    }

    //this.addBlockFactory();
};

// TODO: Vincular a sistema de mensajeria final
Indicators.prototype.showMessage = function(msg, type) {
    $.UIkit.notify(msg, {status: type, pos: 'bottom-right'});
};

/**
 * Dado un elemento, encuentra el index del block parent
 * @param {element} element
 * @returns {int} index
 */
Indicators.prototype.findParentBlockIndex = function(element) {

    var blockElement = $(element).parents('.block');

    var blocksMainPanel = $('#main-panel-content .block');

    for (var i = 0; i < blocksMainPanel.length; i++)
    {
	var blockToCompare = blocksMainPanel[i];
	if (blockToCompare === blockElement[0]) {
	    return i;
	}
    }

    return -1;
};

/**
 * Dado un elemento DOM, encuentra el indice del bloque 
 * @param {type} element
 * @returns {Number}
 */
Indicators.prototype.findBlockByElement = function(element) {
    var lookup = $(element)[0];
    for (var i = 0; i < this.blocks.length; i++) {
	if (this.blocks[i].getElement()[0] === lookup) {
	    return i;
	}
    }

    return -1;
};

Indicators.prototype.addBlockFactory = function() {
    var blockFactory = new BlockFactory();
    blockFactory.initialize();
    var block = blockFactory.getBlock();
    this.addBlock(block, 99999);// Agregar al final
    block.draw();
    this.blockFactory = blockFactory;
};

Indicators.prototype.getBlockFactory = function() {
    return this.blockFactory;
};

Indicators.prototype.createIndicatorBlock = function(chartGroup, blockSettings) {



    var indicatorName = null;
    var indicatorMetric = 0;

    // Se crea un nuevo chart
    var chart = new Chart();

    var indicatorType = 0;
    if (chartGroup.length > 0) {
	indicatorType = chartGroup[0].indicator_type;
    }

    // TODO: Recuperar graficos
    if (indicatorType == INDICATOR_JACK_KNIFE_FAILURE_MODE || indicatorType == INDICATOR_DISTRIBUTION
	    || indicatorType == INDICATOR_FAILURE_MODE) {
	return;

    }

    chart = this.extendChart(chart, indicatorType);


    if (typeof blockSettings !== 'undefined' && blockSettings !== null) {
	if (typeof blockSettings.enableOverlap !== 'undefined'
		&& blockSettings.enableOverlap === true) {
	    chart.setOverlap(true);
	}
    }

    // Se preparan las series
    for (var i = 0; i < chartGroup.length; i++) {
	var indicator = chartGroup[i];
	if (indicatorName === null) {
	    indicatorName = indicator.indicator_type_name;
	    indicatorMetric = indicator.metric_id;
	}

	chart.addIndicator(indicator, false);
    }

    //chart.setTitle(indicatorName, false);
    chart.setYTitle(indicatorName, false);
    chart.setMetricId(indicatorMetric);

    // Se dibuja el nuevo block
    var block = new Block();
    block.setParent(this);

    if (typeof blockSettings !== 'undefined' && blockSettings !== null) {

	block.setType(blockSettings.type);
	if (blockSettings.class !== '')
	    block.setCustomClass(blockSettings.class);
	else
	    block.setCustomClass('uk-width-large-1-2');
	block.getParent().addBlock(block, blockSettings.position);
    }
    else {
	block.setType(1);
	block.setCustomClass('uk-width-large-1-2');
	block.getParent().addBlock(block, 9999);// Agregar al final
    }

    block.setTitle(indicatorName);
    block.setIcon('icon-bars');
    block.setOptions({});
    block.drawTitleBar();
    block.setChart(chart);
    block.draw();
    block.addHandlers();
};

Indicators.prototype.extendChart = function(chart, indicatorType) {

    var extensionChart = {};

    switch (parseInt(indicatorType)) {
	case INDICATOR_MTBS:
	case INDICATOR_MTTR:
	case INDICATOR_MTBF:
	case INDICATOR_MTTI:

	    extensionChart = new MeanTimeChart();
	    break;
	case INDICATOR_CRITICALITY:
	    extensionChart = new CriticalityChart();
	    break;
	case INDICATOR_JACK_KNIFE:
	    extensionChart = new JackKnifeChart();
	    break;
	case INDICATOR_JACK_KNIFE_FAILURE_MODE:
	    extensionChart = new JackKnifeFailureChart();
	    break;
	case INDICATOR_PARETO:
	    extensionChart = new ParetoChart();
	    break;
	case INDICATOR_FAILURE_MODE:
	    extensionChart = new FailureModeChart();
	    break;
	case INDICATOR_RELIABILITY:
	    extensionChart = new ReliabilityChart();
	    break;
	case INDICATOR_MIX_MAINTENANCE_HRS:
	case INDICATOR_MIX_MAINTENANCE_QTY:

	    extensionChart = new MixMaintenanceChart();
	    break;
	case INDICATOR_DISTRIBUTION:
	    extensionChart = new DistributionChart();
	    break;
	case INDICATOR_OEE:
	    extensionChart = new OeeChart();
	    break;
	case INDICATOR_PRODUCTION:
	    extensionChart = new ProductionChart();
	    break;
	case INDICATOR_FNP:
	    extensionChart = new FnpChart();
	    break;
    }

    return $.extend(chart, extensionChart);
};

Indicators.prototype.loadDashboard = function(node) {
    var parent = this;
    parent.destroy();
    parent.setSelector('#main-panel-content');
    parent.setType(SETTINGS_TYPE_DASHBOARD_INDICATORS);
    //parent.setSelectedElements($("#jstree").jstree().get_selected(true));
    parent.setSelectedElements([node]);

    app.close('#right-panel', true);

    var ajax = new Ajax();

    if (ajax.reserveAccess()) {
	ajax.setAddress('indicators/get_dashboard');

	// Se consulta con fecha desde / hasta para los elementos seleccionados
	ajax.setParameters({
	    selected_elements: JSON.stringify(parent.getSelectedElements())
	});

	var groupedIndicators = {};
	var blockSettings = {};
	ajax.setCallback(function(ajaxObject) {

	    if (ajaxObject.response) {

		var blocks = ajaxObject.response.blocks;
		for (var i = 0; i < blocks.length; i++) {
		    var block = blocks[i];
		    for (var j = 0; j < block.indicator_data.length; j++) {
			var indicatorData = block.indicator_data[j];
			for (var k = 0; k < indicatorData.length; k++) {

			    // Si no existe la key en el objeto, se crea
			    var compatibilityHash = block.block.position + '';
			    if (typeof groupedIndicators[compatibilityHash] === 'undefined') {
				groupedIndicators[compatibilityHash] = [];
			    }
			    if (typeof blockSettings[compatibilityHash] === 'undefined') {
				blockSettings[compatibilityHash] = block.block;
			    }

			    // Se agrega el indicador en el grupo correspondiente segun el hash de compatibilidad
			    groupedIndicators[compatibilityHash].push(indicatorData[k]);
			}
		    }
		}
	    }
	});

	var queryObject = ajax.query();

	$.when(queryObject).done(function() {
	    parent.createReport(groupedIndicators, blockSettings, 'Dashboard ' + node.text);
	});
    }
};

Indicators.prototype.destroy = function() {
    this.id = 0;
    for (var i = 0; i < this.blocks.length; i++) {
	var block = this.blocks[i];
	block.destroy();
	block = null;
    }

    this.blocks = [];
    this.selector = '';
    this.type = 1;
    this.selectedElements = [];
    this.blockFactory = null;
};
