
function Mixer() {
    this.blocksToMix = [];
    this.titleSeparator = '/';
    this.maxTitleLength = 60;
    this.moreElementsTitle = '...';
}

Mixer.prototype.isReady = function() {
    return this.blocksToMix.length > 1;
};

Mixer.prototype.isMixing = function() {
    return this.blocksToMix.length > 0;
};

Mixer.prototype.startMixingBlocks = function(block) {
    this.blocksToMix = [];
    this.blocksToMix.push(block);
};

Mixer.prototype.finishMixingBlocks = function(block) {
    this.blocksToMix.push(block);
    this.mix();
};

/**
 * Se revisa la compatibilidad de los charts contenidos en los blocks
 * @returns {Boolean} chartsCompatibles
 */
Mixer.prototype.checkCompatibility = function() {

    var compatibilityHash = null;

    for (var i = 0; i < this.blocksToMix.length; i++) {

	var block = this.blocksToMix[i];
	var blockChart = block.getChart();

	// El block que se intenta mezclar no tiene chart
	if (blockChart === null) {
	    return false;
	}

	// Aun no se define un hash para comparar
	if (compatibilityHash === null) {
	    compatibilityHash = blockChart.getCompatibilityHash();
	    continue;
	}

	if (blockChart.getCompatibilityHash() !== compatibilityHash) {
	    return false;
	}
    }

    return true;
};

Mixer.prototype.mix = function() {

    if (!this.isReady())
	return;

    if (this.checkCompatibility()) {

	var newSeries = [];

	// Las series se mezclan en el ultimo bloque
	var targetBlock = this.blocksToMix.pop();
	var newTitleFragments = targetBlock.getTitle().split(' ' + this.titleSeparator + ' ');
	for (var i = 0; i < this.blocksToMix.length; i++) {

	    var block = this.blocksToMix[i];
	    newTitleFragments.push(block.getTitle().trim());
	    var blockChart = block.getChart();
	    newSeries = newSeries.concat(blockChart.getSeries());
	}

	if (newSeries.length > 0) {

	    // Se combinan las series
	    var targetChart = targetBlock.getChart();
	    for (var i = 0; i < newSeries.length; i++) {
		targetChart.importFormattedSerie(newSeries[i], newSeries.length - 1 === i); // Si ultima serie, dibujar
	    }

	    // Se actualizan los titulos
	    newTitleFragments = $.unique(newTitleFragments);
	    var newTitle = newTitleFragments.join(' ' + this.titleSeparator + ' ');

	    // Si el titulo es muy largo, se reconstruye por partes
	    if (newTitle.length > this.maxTitleLength) {
		newTitle = '';
		for (var i = 0; i < newTitleFragments.length; i++) {
		    var fragment = newTitleFragments[i].trim();
		    var testTitle = newTitle + fragment + ' ' + this.titleSeparator + ' ';
		    if (fragment === this.moreElementsTitle || testTitle.length > this.maxTitleLength) {
			newTitle += this.moreElementsTitle;
			break;
		    }

		    newTitle += testTitle;
		}
	    }

	    //targetChart.setTitle(newTitle, true); No usar titulos por defecto
	    targetBlock.setTitle(newTitle);
	    targetBlock.drawTitleBar();
	}

    }
    else {
	app.showMessage('Los graficos seleccionados no son compatibles.', 'warning');
    }

    this.destroy();
};

Mixer.prototype.destroy = function() {
    this.blocksToMix = [];
};