
/**
 * retorna la cantidad de keys del objeto
 * @param {object} o
 * @returns {int} size
 */
function getObjectSize(o)
{
	var count = 0;
	var i;

	for (i in o) {
		if (o.hasOwnProperty(i)) {
			count++;
		}
	}

	return count;
}

/**
 * Obtiene el nombre del mes en base a la fecha entregada
 * @param {type} dateString
 * @returns {Number}
 */
function getMonthName(dateString)
{
	var months = new Array("Ene", "Feb", "Mar", "Abr", "May", "Jun",
			"Jul", "Ago", "Sept", "Oct", "Nov", "Dic");

	var date = new Date(dateString);

	return months[date.getMonth()];
}

var structurePanelOpen = false;
var structurePanelKeep = false;

var generalInfoPanelOpen = true;
var generalInfoPanelKeep = false;
// Muestra u oculta los paneles, respetando la preferencia del usuario sobre paneles persistentes
function openGeneralInfoPanel(show) {
	openGeneralInfoPanel(show, false);
}

function openGeneralInfoPanel(show, overridePin)
{
	if (show === null)
		show = !generalInfoPanelOpen;

	if (!show) {
		if (!generalInfoPanelKeep || overridePin) {
			$('#right-panel').stop(true).animate({right: '0'}, 250, 'swing', function() {
				generalInfoPanelOpen = show;
			});
		}
	}
	else
	{
		$('#right-panel').stop(true).animate({right: 0}, 250, 'swing', function() {
			generalInfoPanelOpen = show;
		});
	}
	$('#right-panel .tab div').html((!generalInfoPanelOpen) ? '&raquo' : '&laquo;');
}

// Muestra u oculta los paneles, respetando la preferencia del usuario sobre paneles persistentes
function openStructurePanel(show) {
	openStructurePanel(show, false);
}

function openStructurePanel(show, overridePin)
{
	if (show === null)
		show = !structurePanelOpen;

	if (!show)
	{
		if (!structurePanelKeep || overridePin)
		{
			$('#left-panel').stop(true).animate({left: '-18%'}, 250, 'swing', function() {
				structurePanelOpen = show;
			});
		}
	}
	else
	{
		$('#left-panel').stop(true).animate({left: 0}, 250, 'swing', function() {
			structurePanelOpen = show;
		});
	}
}

function showMainPanel(show)
{
	if (show) {
		$('#main-panel').css({display: 'block'});
	}
	else {
		$('#main-panel').css({display: 'none'});
	}
}

function setTreeSelection(name, selectedNodes) {

	if (selectedNodes != null)
	{
		var nodeSelection = JSON.parse(selectedNodes.split("\'").join('"'))
		var companyTree = app.getCompanyTree();
		var companyTreeNodes = companyTree._model.data;
		companyTree.deselect_all();
		$('#selection-name').val(name);

		for (var i = 0; i < nodeSelection.length; i++)
		{
			var node = nodeSelection[i];

			for (var nodeId in companyTreeNodes) {
				var treeNode = companyTreeNodes[nodeId];

				if (treeNode.id === '#') {
					continue;
				}

				if (treeNode.original.oid == node.id && treeNode.original.node_type == node.node_type) {
					companyTree.select_node(nodeId);
					break;
				}
			}
		}
	}
}

function getNodeOid(companyTreeNodes) {

	for (var nodeId in companyTreeNodes) {
		var treeNode = companyTreeNodes[nodeId];

		if (treeNode.id === '#') {
			continue;
		}

		if (treeNode.original.oid === node.id && treeNode.original.node_type === node.node_type) {
			companyTree.select_node(nodeId);
			break;
		}
	}
}

function roundFloat(num, places) {

	if (typeof places === 'undefined')
		places = 2;

	return +(Math.round(num + "e+" + places) + "e-" + places);
}

function truncateString(str, length, addTooltip) {
	var htmlTooltipStart = '';
	var htmlTooltipEnd = '';

	if (addTooltip) {
		htmlTooltipStart = '<span title="' + str + '">';
		htmlTooltipEnd = '</span>';
	}

	return htmlTooltipStart + (str.length > length ? str.substring(0, length - 3) + '...' : str) + htmlTooltipEnd;
}

function ucFirst(string) {
	return string.charAt(0).toUpperCase() + string.slice(1);
}

function latinDateToIso(dateString, separator) {
	if (typeof separator == 'undefined') {
		separator = '/';
	}

	var dateParts = dateString.split(separator);
	return dateParts[2] + '-' + dateParts[1] + '-' + dateParts[0];
}

function pad(num, size) {
	var s = "000000000" + num;
	return s.substr(s.length - size);
}

function millisToDate(millis) {
	var newDate = new Date(millis);
	return newDate.getFullYear() + '-' + pad(newDate.getMonth() + 1, 2) + '-' + pad(newDate.getUTCDate(), 2);
}

/**
 * Iguala 2 elementos al elemento de altura {equalizeTo}
 * @param {string} firstElement
 * @param {string} secondElement
 * @param {string} equalizeTo (min|max)
 * @param {int} fixedMax
 * @param {int} fixedMin
 * @returns void
 */
function equalizeHeight(firstElement, secondElement, equalizeTo, fixedMax, fixedMin) {

	switch (equalizeTo) {
		case 'max':
			var maxHeight = Math.max($(firstElement).height(), $(secondElement).height());

			if (typeof fixedMin !== 'undefined' && fixedMin !== null && fixedMin > maxHeight) {
				maxHeight = fixedMin;
			}

			$(firstElement).css('min-height', maxHeight + 'px');
			$(secondElement).css('min-height', maxHeight + 'px');

			if (typeof fixedMax !== 'undefined' && fixedMax !== null) {
				$(firstElement).css('max-height', fixedMax + 'px');
				$(secondElement).css('max-height', fixedMax + 'px');
			}
			break;
		case 'min':
			var minHeight = Math.min($(firstElement).height(), $(secondElement).height());

			if (typeof fixedMax !== 'undefined' && fixedMax !== null && fixedMax < minHeight) {
				minHeight = fixedMax;
			}

			$(firstElement).css('max-height', minHeight + 'px');
			$(secondElement).css('max-height', minHeight + 'px');

			if (typeof fixedMin !== 'undefined' && fixedMin !== null) {
				$(firstElement).css('min-height', fixedMin + 'px');
				$(secondElement).css('min-height', fixedMin + 'px');
			}
			break;
		case 'exact':
			var maxHeight = Math.max($(firstElement).height(), $(secondElement).height());

			$(firstElement).height(maxHeight);
			$(secondElement).height(maxHeight);
			break;

	}

}

function sortByValue(b, a, value) {
	var aValue = a.sortable_value;
	var bValue = b.sortable_value;
	return ((aValue < bValue) ? -1 : ((aValue > bValue) ? 1 : 0));
}

function generateNameValueTable(array, tableTitle, limit, classes) {

	var tableHtml = '<table class="' + classes + '">';
	tableHtml += '<tr><th>' + tableTitle + '</th><th>' + lang.line('label_value_js') + '</th></tr>';
	for (var i = 0; i < Math.min(array.length, limit); i++) {
		tableHtml += '<tr><td>' + array[i].name + '</td><td>' + array[i].value + '</td></tr>';
	}

	tableHtml += '</table>';

	return tableHtml;
}