var selectedOperations = [];
var currentOperations = [];
var selectedAreas = [];
var currentAreas = [];

var selectedElements = [];

var selecting = false;

var indicatorsWrapper = new Indicators();

var currentMenu = '';


/*
 * Carga los ultimos graficos disponibles para los subsystems seleccionados
 * agrupando los indicadores con caracteristicas similares como fechas y tipo indicador
 * @returns {undefined}
 */
function loadLatestsCharts() {
	// Se obtiene la lista de elementos seleccionados
	var groupedIndicators = {};
	var selectedElements = $("#jstree").jstree().get_selected(true);
	if (selectedElements.length <= 0) {
		return;
	}

	// Se eliminan todos los graficos anteriores
	indicatorsWrapper.destroy();
	indicatorsWrapper.setSelector('#main-panel-content');
	indicatorsWrapper.setType(SETTINGS_TYPE_GENERAL_INDICATORS);

	groupedIndicators = {};
	var noRef = [];
	var ajaxQueryGroup = [];
	var reportSelectedNames = [];

	// Para cada subsystem_ref disponible se realiza una query ajax
	for (var i = 0; i < selectedElements.length; i++) {
		var selectedElement = selectedElements[i];
		reportSelectedNames.push(selectedElement.text);
		if (selectedElement.data.subsystem_ref != null) {
			// Query ajax granular para aprovechar cache de browser

			// Si es area, se obtiene indicadores de produccion
			if (selectedElement.original.node_type == NODE_TYPE_AREA) {

				var ajax = new Ajax();
				ajax.reserveAccess();
				ajax.setAddress(BASE_URL + 'indicators/get_indicator');
				ajax.setMethod('post');
				ajax.setParameters({
					indicator_type_id: INDICATOR_PRODUCTION,
					metric_id: null,
					report_id: null,
					subsystem_id: selectedElement.data.subsystem_ref
				});

				ajax.setCallback(function(ajaxObject) {
					for (var k = 0, size = ajaxObject.response.length; k < size; k++) {
						var indicator = ajaxObject.response[k];

						var compatibilityHash = indicator.compatibility_hash;
						if (typeof groupedIndicators[compatibilityHash] === 'undefined') {
							groupedIndicators[compatibilityHash] = [];
						}

						// Se agrega el indicador en el grupo correspondiente segun el hash de compatibilidad
						groupedIndicators[compatibilityHash].push(indicator);
					}
				});
				var queryRef = ajax.query();
				ajaxQueryGroup.push(queryRef);
			}






			var ajaxQuery = $.getJSON(BASE_URL + 'indicators/get_latests_graphs/' + selectedElement.data.subsystem_ref, function(json) {

				// Se agrupan indicadores compatibles
				for (var i = 0, size = json.length; i < size; i++) {
					var indicator = json[i];

					// Si no existe la key en el objeto, se crea
					var compatibilityHash = indicator.compatibility_hash;

					if (typeof groupedIndicators[compatibilityHash] === 'undefined') {
						groupedIndicators[compatibilityHash] = [];
					}

					// Se agrega el indicador en el grupo correspondiente segun el hash de compatibilidad
					groupedIndicators[compatibilityHash].push(indicator);
				}
			});

			// se agrega la referencia del objeto (jquery) de consulta ajax a un arreglo
			ajaxQueryGroup.push(ajaxQuery);
		}
		else {
			noRef.push(selectedElement);
		}
	}

	indicatorsWrapper.setSelectedElements(selectedElements);

	// Se espera a que todas las consultas ajax se hayan ejecutado
	$.when.apply($, ajaxQueryGroup).done(function() {

		var chartsAvailable = getObjectSize(groupedIndicators);


		if (chartsAvailable > 0) {

			if (selectedElements.length === 1) {
				// TODO: Order List Config
				var order = [];
				if (selectedElements[0].type === 'area') {
					order = [4, 5, 21, 24];
				}
				else {
					if (selectedElements[0].data.isleaf == 1) {
					    order = [4, 5, 9, 7, 17, 11, 12];
					}
					else {
					    order = [22, 4, 5, 17, 11, 12, 21, 24];
					}
				}

				var orderedGroupedIndicators = [];
				for (var i = 0; i < order.length; i++) {

					for (var chartGroupKey in groupedIndicators) {
						var chartGroup = groupedIndicators[chartGroupKey];
						if (chartGroup[0].indicator_type === order[i]) {
							orderedGroupedIndicators.push(chartGroup);
							break;
						}
					}
				}

				// Se itera sobre todos las agrupaciones de indicadores a graficar
				for (var chartGroupKey in orderedGroupedIndicators) {
					var chartGroup = orderedGroupedIndicators[chartGroupKey];
					indicatorsWrapper.createIndicatorBlock(chartGroup);
				}
			}
			else {
				// Se itera sobre todos las agrupaciones de indicadores a graficar
				for (var chartGroupKey in groupedIndicators) {
					var chartGroup = groupedIndicators[chartGroupKey];
					indicatorsWrapper.createIndicatorBlock(chartGroup);
				}
			}

			indicatorsWrapper.loadPreferences();
			indicatorsWrapper.addBlockFactory('#main-panel-content');
		}
		else {
			app.showMessage(lang.line('msg_selection_without_indicators'), 'warning');
		}

	});

	var titleHtml = '';
	switch (reportSelectedNames.length) {
		case 0:
			titleHtml = '';
			break;
		case 1:
			titleHtml = lang.line('label_available_indicators_for_js') + ' ' + reportSelectedNames[0];
			break;
		default:
			titleHtml = '<span data-uk-tooltip="{pos:\'bottom-right\'}"  title="' + reportSelectedNames.join(', <br/>') + '">';
			titleHtml += lang.line('label_available_indicators_for_js') + ' ' + reportSelectedNames.length + ' ' + lang.line('label_nodes_js');
			titleHtml += '</span>';
	}

	$('.report-title').html(titleHtml);
}

$(document).ready(function()
{


	// Verificar que se esta intentando seleccionar múltiples elementos
	window.onkeydown = function(e) {
		selecting = ((e.keyIdentifier == 'Control') || (e.ctrlKey == true));
	};
	window.onkeyup = function(e) {
		selecting = false;
	};

	// Define si el panel se desea mantener fijo o no
	$('#left-panel .panel-options .keep-panel').click(function() {
		var leftPanel = $('#left-panel');
		if (leftPanel.hasClass('uk-width-large-1-5')) {
			$('#left-panel').removeClass('uk-width-large-1-5').addClass('uk-width-large-1-2 uk-width-small-1-1 uk-width-medium-1-1');
		}
		else {
			$('#left-panel').removeClass('uk-width-large-1-2 uk-width-small-1-1 uk-width-medium-1-1').addClass('uk-width-large-1-5');
		}
	});

	// Define si el panel se desea mantener fijo o no
	$('#left-panel .panel-options .hide-panel').click(function() {
		var leftPanel = $('#left-panel');
		if (leftPanel.hasClass('uk-width-small-1-1')) {
			$('#left-panel').removeClass('uk-width-small-1-1').addClass('uk-hidden-small');
		}
		else {
			$('#left-panel').removeClass('uk-hidden-small').addClass('uk-width-small-1-1');
		}
	});

	// Define si el panel se desea mantener fijo o no
	$('#right-panel .panel-options .keep-panel').click(function() {
		generalInfoPanelKeep = !generalInfoPanelKeep;
		if (generalInfoPanelKeep)
		{
			$(this).addClass('active');
			$(this).children('i').removeAttr('class').addClass('icon-pinned');
		}
		else
		{
			$(this).removeClass('active');
			$(this).children('i').removeAttr('class').addClass('icon-unpinned');
		}
	});

	// Se muestra u oculta el panel
	$('#right-panel .tab').click(function() {
		openGeneralInfoPanel(null, true);
	});

	$('#get-indicators-button').click(function() {
		$('#map').addClass('blurry');
		app.close('#right-panel', false);
		showMainPanel(true);
		openGeneralInfoPanel(false);
		loadLatestsCharts();

		if ($('#left-panel .panel-options .hide-panel').css('display') != 'none') {
			$('#left-panel .panel-options .hide-panel').click();
		}
	});

	$('#main-panel .panel-options .close-panel').click(function() {
		showMainPanel(false);
		$('#map').removeClass('blurry');
	});

	$('#main-panel .panel-options .save-config').click(function() {
		if (indicatorsWrapper.isInitialized()) {
			indicatorsWrapper.saveConfig();
		}
	});

	// Se muestra u oculta el panel
	$('#left-panel .tab').click(function() {
		openStructurePanel(null, true);
	});
});

function getIndicators() {
	$('#map').addClass('blurry');
	showMainPanel(true);
	openGeneralInfoPanel(false);
	loadLatestsCharts();
	app.open('#left-panel', true);
}

$('.uk-sortable').on('sortable-change', function() {
	var targetElement = $('.uk-sortable-over');

	if (targetElement.length > 0) {
		targetElement = targetElement[0];

		var blockIndex = indicatorsWrapper.findBlockByElement(targetElement);
		mixer.finishMixingBlocks(indicatorsWrapper.getBlock(blockIndex));
	}
});

$('.uk-sortable').on('sortable-start', function(a, b, c) {
	var blockIndex = indicatorsWrapper.findBlockByElement(c);
	mixer.startMixingBlocks(indicatorsWrapper.getBlock(blockIndex));
});
